@@ -27,9 +27,9 @@
 
 #include "PaintP.h"
 #include "xpaint.h"
+#include "image.h"
 #include "misc.h"
 
-
 typedef void (*func_t) (Widget, void *, XEvent *, Boolean);
 typedef struct data_s {
     func_t func;
@@ -106,8 +106,6 @@ static XtResource resources[] =
      offset(paint), XtRImmediate, (XtPointer) None},
     {XtNdirty, XtCDirty, XtRBoolean, sizeof(Boolean),
      offset(dirty), XtRImmediate, (XtPointer) False},
-    {XtNeditable, XtCEditable, XtRBoolean, sizeof(Boolean),
-     offset(editable), XtRImmediate, (XtPointer) True},
     {XtNlocked, XtCLocked, XtRBoolean, sizeof(Boolean),
      offset(locked), XtRImmediate, (XtPointer) False},
     {XtNfillRule, XtCFillRule, XtRInt, sizeof(int),
@@ -210,13 +208,109 @@ PaintClassRec paintClassRec =
 
 WidgetClass paintWidgetClass = (WidgetClass) & paintClassRec;
 
+Pixel
+AlphaTwistPixel(Pixel p, unsigned char *u)
+{
+    unsigned char a;
+    unsigned short s;
+
+    /* u[3] is alpha_mode */
+    if (u[3] == 2) {
+        a = u[0];
+        s = u[2] * (255-a);
+        get_color_components(p, u);
+        u[0] = (u[0] * a + s)>>8;
+        u[1] = (u[1] * a + s)>>8;
+        u[2] = (u[2] * a + s)>>8;
+    } else 
+    if (u[3] == 3) {
+    	u[1] = u[2] = u[0];
+    } else {
+        /* mode == 1 */
+        a = u[0];
+        s = 255 - a;
+        get_color_components(p, u);
+        u[0] = (u[0] * a + Global.bg[0] * s)>>8;
+        u[1] = (u[1] * a + Global.bg[1] * s)>>8;
+        u[2] = (u[2] * a + Global.bg[2] * s)>>8;
+    }
+    return get_pixel_from_colors(u);
+}
+
+Pixel
+AlphaTwistPixel8(Pixel p, unsigned char *u)
+{
+    unsigned char a;
+
+    a = u[0]/23;
+    if (u[3] == 3) {
+	if (u[1] >= a)
+            return Global.bg[4];
+	else
+	    return Global.bg[6];
+    } else
+    if (u[3] == 1) {
+	if (u[1] >= a)
+	    return Global.bg[6];
+        else
+	    return p;
+    } else {
+	if (u[1] >= a) {
+	    if (u[2])
+	        return Global.bg[6];
+	    else
+		return p;
+	} else
+	    return p;
+    }
+}
+
+void
+AlphaTwistImage(PaintWidget pw, XImage *xim, 
+                unsigned char *alpha, int width, 
+                int dx, int dy, int ax, int ay)
+{
+    int x, y;
+    Pixel p = 0;
+    unsigned char u[4];
+    PaintWidget pp = (pw->paint.paint)? (PaintWidget)pw->paint.paint : pw;
+
+    u[3] = (unsigned char) pp->paint.alpha_mode;
+
+    if (Global.depth <= 8) {
+        u[2] = 0;
+        for (y=0; y<xim->height; y++) {
+	    for (x=0; x<xim->width; x++) {
+                u[0] = alpha[dx+x+(dy+y)*width];
+                u[1] = (unsigned char)((71*x+29*y)%12);
+                if (u[3] <= 2) {
+                    p = xxGetPixel(xim, x, y);
+                    if (u[3] == 2) u[2] = ((ax+x)/12+(ay+y)/12)&1;
+		}
+                xxPutPixel(xim, x, y, AlphaTwistPixel8(p, u));
+	    }
+	}
+    } else {
+        for (y=0; y<xim->height; y++) {
+	    for (x=0; x<xim->width; x++) {
+	        u[0] = alpha[dx+x+(dy+y)*width];
+                if (u[3] <= 2) {
+                    p = xxGetPixel(xim, x, y);
+                    if (u[3] == 2) u[2] = ALPHA_PIXEL_MODE(ax+x,ay+y);
+	        }
+	        xxPutPixel(xim, x, y, AlphaTwistPixel(p, u));
+	    }
+	}
+    }
+}
+
 /*
  * Attach a new undo buffer to the head of the undo list for this widget.
  */
 static UndoStack *
 undoBuffer(PaintWidget w)
 {
-    UndoStack *new = (UndoStack *) XtMalloc(sizeof(UndoStack));
+    UndoStack *new = (UndoStack *) xmalloc(sizeof(UndoStack));
 
     if (new == NULL)
 	return NULL;
@@ -352,8 +446,8 @@ GetRectangle(PaintWidget w)
 
     rect.x = dx = 0;
     rect.y = dy = 0;
-    rect.width = w->core.width - dx;
-    rect.height = w->core.height - dy;
+    rect.width = w->core.width;
+    rect.height = w->core.height;
 
     /*
     **	Climb the tree to find the visible area.
@@ -587,8 +681,9 @@ ResizeProc(Widget w)
 **
  */
 
+/*
 static void
-zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
+ZoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	      Boolean isExpose, int xstart, int ystart, int zoom,
 	      XRectangle * rect)
 {
@@ -597,19 +692,23 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
     Display *dpy = XtDisplay(w);
     Window win = XtWindow(w);
     int width = rect->width;
-    int count = rect->width * rect->height;
+    int count = (rect->width+1) * (rect->height+1);
     XRectangle rects[256];
-    int index, i;
+    int index, i, n;
+    unsigned char *alpha = pw->paint.current.alpha;
+    unsigned char a, ap, u[4];
+    Pixel p, pp, q;
 
     if (flagsSize < count) {
 	int allocSize = (count + 4) * sizeof(Boolean);
 	if (flags == NULL)
-	    flags = (Boolean *) XtMalloc(allocSize);
+	    flags = (Boolean *) xmalloc(allocSize);
 	else
 	    flags = (Boolean *) XtRealloc((XtPointer) flags, allocSize);
 	flagsSize = count;
     }
     memset(flags, 0, flagsSize * sizeof(Boolean));
+
     if (mask != NULL) {
 	int x, y;
 
@@ -621,7 +720,15 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	    }
 	}
     }
-    for (i = 0; i < XtNumber(rects) && i < count; i++)
+
+    n = XtNumber(rects);
+    if (alpha) {
+        u[3] = (unsigned char) pw->paint.alpha_mode;
+        if (u[3] == 2) n = 1;
+    } else
+        u[3] = 0;
+
+    for (i = 0; i < n && i < count; i++)
 	if (zoom > ZOOM_THRESH)
 	    rects[i].width = rects[i].height = zoom - 1;
 	else
@@ -632,14 +739,22 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	    int xindex = index % width + rect->x;
 	    int yindex = index / width + rect->y;
 	    int xindexEnd = rect->x + rect->width;
-	    Pixel p = xxGetPixel(xim, xindex, yindex);
 	    int xpos = (index % width + xstart) * zoom;
 	    int ypos = (index / width + ystart) * zoom;
 	    int pos, flag;
 
+            p = xxGetPixel(xim, xindex, yindex);
+            if (u[3]) {
+	        a = alpha[xindex + yindex*pw->paint.drawWidth];
+                u[0] = a;
+	        if (u[3] == 2) u[2] = ALPHA_PIXEL_MODE(xindex, yindex);
+                q = AlphaTwistPixel(p, u);
+	    } else
+	        q = p;
+
 	    flag = (isExpose && p == w->core.background_pixel);
 
-	    XSetForeground(dpy, gc, p);
+	    XSetForeground(dpy, gc, q);
 	    for (pos = index, i = 0; pos != count;
 		 pos++, xindex++, xpos += zoom) {
 		if (xindex == xindexEnd) {
@@ -648,8 +763,13 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 		    yindex++;
 		    ypos += zoom;
 		}
-		if (flags[pos] || xxGetPixel(xim, xindex, yindex) != p)
+                pp = xxGetPixel(xim, xindex, yindex);
+                if (u[3])
+	            ap = alpha[xindex + yindex*pw->paint.drawWidth];
+
+		if (flags[pos] || pp != p || ap != a)
 		    continue;
+
 		flags[pos] = True;
 
 		if (flag)
@@ -659,7 +779,14 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 		rects[i].y = ypos;
 		i++;
 
-		if (i == XtNumber(rects)) {
+                if (u[3] == 2) {
+		    u[0] = a;
+		    u[2] = ALPHA_PIXEL_MODE(xindex, yindex);
+                    q = AlphaTwistPixel(p, u);
+	            XSetForeground(dpy, gc, q);
+	        }
+
+		if (i == n) {
 		    XFillRectangles(dpy, win, gc, rects, i);
 		    i = 0;
 		}
@@ -669,132 +796,141 @@ zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	}
     }
 }
+*/
 
 static void
-zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
+ZoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	      Boolean isExpose, int xstart, int ystart, int zoom,
 	      XRectangle * rect)
 {
     Display *dpy = XtDisplay(w);
-    Pixmap pix;
     int width = rect->width;
     int height = rect->height;
     int widthp, heightp;
+    Pixmap pix;
     XImage *dst;
     XImage *dstMask = NULL;
+    Pixel p;
     int bytes_per_pixel, zoom_bytes_per_pixel;
-    int i, j, x, y, x1, y1, dx, dy;
+    int k, l, m, x, y, x1, y1, x2, y2, dx, dy;
     int numDraw;
     int endX;
     int endY;
     int interpolation;
+    unsigned char u[4];
+    unsigned int s[3];
+    unsigned char *alpha;
+    PaintWidget pp = (pw->paint.paint)? (PaintWidget)pw->paint.paint : pw;
+
+    alpha = pp->paint.current.alpha;
+    if (alpha)
+        u[3] =  (unsigned char)pp->paint.alpha_mode;
+    else
+        u[3] = 0;
 
     if (zoom<0) {
         zoom = -zoom;
-        XtVaGetValues((Widget)pw, XtNinterpolation, &interpolation, NULL);
+        XtVaGetValues((Widget)pp, XtNinterpolation, &interpolation, NULL);
+        if (Global.depth<=8) interpolation = 0;
         widthp = (width+zoom-1)/zoom;
         heightp = (height+zoom-1)/zoom;
-        dst = XCreateImage(dpy, pw->paint.visual, xim->depth, xim->format,
+        dst = XCreateImage(dpy, pp->paint.visual, xim->depth, xim->format,
 		           0, NULL, widthp, heightp, 32, 0);
-        dst->data = (char *) XtMalloc(heightp * dst->bytes_per_line);
+        dst->data = (char *) xmalloc(heightp * dst->bytes_per_line);
 
-        if (mask != NULL) {
-            dstMask = XCreateImage(dpy, pw->paint.visual,
+        if (mask) {
+            dstMask = XCreateImage(dpy, pp->paint.visual,
 			           mask->depth, mask->format,
 			           0, NULL, widthp, heightp, 8, 0);
-	dstMask->data = (char *) 
-                     XtMalloc(heightp * dstMask->bytes_per_line);
-	memset(dstMask->data, 0, heightp * dstMask->bytes_per_line);
+	    dstMask->data = (char *) 
+                     xmalloc(heightp * dstMask->bytes_per_line);
+	    memset(dstMask->data, 0, heightp * dstMask->bytes_per_line);
 	}
-        if (mask == NULL) {
-	    bytes_per_pixel = xim->bits_per_pixel/8;
-            zoom_bytes_per_pixel = zoom*bytes_per_pixel;
-	    if (xim->bits_per_pixel != 8*bytes_per_pixel)
-            /* slow procedure */
+
+	bytes_per_pixel = xim->bits_per_pixel >> 3;
+        zoom_bytes_per_pixel = zoom*bytes_per_pixel;
+	if (!interpolation || xim->bits_per_pixel != 8*bytes_per_pixel) {
+        /* fallback procedure */
 	    for (y = 0; y < heightp; y++) {
 	        dy = y*zoom ;
 	        for (x = 0; x < widthp; x++) {
-		    Pixel p;
-	            dx = x*zoom ;
-		    p = XGetPixel(xim, rect->x+dx, rect->y+dy);
-		    XPutPixel(dst, x, y, p);
-		}
-	    } else
-            /* faster procedure */
-	    for (y = 0; y < heightp; y++) {
-	        dy = y*zoom;
-                j = (rect->y+dy) * xim->bytes_per_line +
-                    rect->x * bytes_per_pixel;
-                i = y * dst->bytes_per_line;
-                if (interpolation) {
-		    unsigned int u;
-		    int k, l, m, zx, zy;
-                    if (y<heightp-1) zy = zoom; else zy = height-y*zoom;
-	            for (x = 0; x < widthp; x++) {
-                        if (x<widthp-1) zx = zoom; else zx = width-x*zoom;
-		        for (k=0; k<bytes_per_pixel; k++) { 
-		             u = 0;
-		             for (l=0; l<zx; l++)
-		             for (m=0; m<zy; m++)
-			       u += (unsigned char)xim->data[j+k+l*bytes_per_pixel+m*xim->bytes_per_line];
-                             dst->data[i+k]= (u/(zx*zy))&255;
-			}
-		        i += bytes_per_pixel;
-                        j += zoom_bytes_per_pixel;
+	            dx = x*zoom;
+                    if (mask) {
+                        if (!XGetPixel(mask, rect->x+dx, rect->y+dy)) continue;
+		        XPutPixel(dstMask, x, y, True);
 		    }
-		} else
-	        for (x = 0; x < widthp; x++) {
-		    memcpy(&dst->data[i], &xim->data[j], bytes_per_pixel);
-		    i += bytes_per_pixel;
-                    j += zoom_bytes_per_pixel;
+                    if (u[3]) {
+                        u[0] = alpha[rect->x+dx+(rect->y+dy)*pp->paint.drawWidth];
+                        if (u[3] <= 2) {
+                            p = xxGetPixel(xim, rect->x+dx, rect->y+dy);
+                            if (u[3] == 2)
+                                u[2] = ALPHA_PIXEL_MODE(rect->x+dx, rect->y+dy);
+                        }
+                        p = AlphaTwistPixel(p, u);
+		    } else
+		        p = xxGetPixel(xim, rect->x+dx, rect->y+dy);
+		    xxPutPixel(dst, x, y, p);
 		}
 	    }
-            XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
-		      xstart/zoom, ystart/zoom,
-		      widthp, heightp);
 	} else {
-	    bytes_per_pixel = xim->bits_per_pixel/8;
-            zoom_bytes_per_pixel = zoom*bytes_per_pixel;
-	    if (xim->bits_per_pixel != 8*bytes_per_pixel)
-	    /* slow procedure */
-	    for (y = 0; y < heightp; y++)
-	    for (x = 0; x < widthp; x++) {
-		Pixel p;
-	        dx = x*zoom;
-	        dy = y*zoom;
-                if (!XGetPixel(mask, rect->x+dx, rect->y+dy)) continue;
-		p = XGetPixel(xim, rect->x+dx, rect->y+dy);
-		XPutPixel(dst, x, y, p);
-		XPutPixel(dstMask, x, y, True);
-	    } else 
-            /* faster procedure */
+        /* regular procedure */
 	    for (y = 0; y < heightp; y++) {
-	        dy = y * zoom;
-                j = (rect->y+dy) * xim->bytes_per_line +
-                    rect->x * bytes_per_pixel;
-                i = y * dst->bytes_per_line;
+	        dy = y*zoom;
+	        y1 = rect->y + dy;
+                y2 = y1 + zoom;
+                if (y2 > (k=rect->y + height)) y2 = k;
 	        for (x = 0; x < widthp; x++) {
-		    if (XGetPixel(mask, rect->x+x*zoom, rect->y+dy)) {
-		        memcpy(&dst->data[i], &xim->data[j], bytes_per_pixel);
-                        XPutPixel(dstMask, x, y, True);
+		    dx = x*zoom;
+                    if (mask) {
+                        if (!XGetPixel(mask, rect->x+dx, rect->y+dy)) continue;
+		        XPutPixel(dstMask, x, y, True);
 		    }
-		    i += bytes_per_pixel;
-                    j += zoom_bytes_per_pixel;
+                    x1 = rect->x + dx;
+                    x2 = x1 + zoom;
+                    if (x2 > (k=rect->x + width)) x2 = k;
+                    for (k=0; k<=2; k++) s[k] = 0;
+  		    for (m=y1; m<y2; m++) {
+		        for (l=x1; l<x2; l++) {
+                            if (u[3]) {
+				u[0] = alpha[l+m*pp->paint.drawWidth];
+			        if (u[3] <= 2) {
+			            p = xxGetPixel(xim, l, m);
+                                    if (u[3] == 2)
+                                        u[2] = ALPHA_PIXEL_MODE(l, m);
+				}
+                                p = AlphaTwistPixel(p, u);
+			    } else {
+			        p = xxGetPixel(xim, l, m);
+                                get_color_components(p, u);
+			    }
+                            for (k=0; k<=2; k++) 
+                                s[k] += (unsigned int)u[k];
+			}
+		    }
+                    l = (x2-x1)*(y2-y1);
+                    for (k=0; k<=2; k++) 
+			u[k] = (unsigned char)(s[k]/(unsigned int)l);
+                    p = get_pixel_from_colors(u);
+                    xxPutPixel(dst, x, y, p);
 		}
 	    }
-            /* {static int jj=0; printf("??? %d\n",  ++jj);} */
-	    pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
+	}
+        if (mask) {
+	    pix = XCreatePixmap(dpy, XtWindow(w),
 				widthp, heightp, 1);
-  	    XPutImage(XtDisplay(w), pix, pw->paint.mgc, dstMask,
+  	    XPutImage(dpy, pix, pp->paint.mgc, dstMask,
 		      0, 0, 0, 0, widthp, heightp);
-	    XSetClipMask(XtDisplay(w), gc, pix);
-	    XSetClipOrigin(XtDisplay(w), gc, xstart/zoom, ystart/zoom);
-	    XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
+	    XSetClipMask(dpy, gc, pix);
+	    XSetClipOrigin(dpy, gc, xstart/zoom, ystart/zoom);
+	    XPutImage(dpy, XtWindow(w), gc, dst, 0, 0,
 		      xstart/zoom, ystart/zoom,
 		      widthp, heightp);
-            XSetClipMask(XtDisplay(w), gc, None);
-	    XFreePixmap(XtDisplay(w), pix);
-	}
+            XSetClipMask(dpy, gc, None);
+	    XFreePixmap(dpy, pix);
+	} else
+            XPutImage(dpy, XtWindow(w), gc, dst, 0, 0,
+	              xstart/zoom, ystart/zoom,
+		      widthp, heightp);
         goto end;
     }
 
@@ -802,63 +938,53 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
     endX = rect->x + rect->width;
     endY = rect->y + rect->height;
 
-    dst = XCreateImage(dpy, pw->paint.visual, xim->depth, xim->format,
+    dst = XCreateImage(dpy, pp->paint.visual, xim->depth, xim->format,
   	               0, NULL, width * zoom, height * zoom, 32, 0);
-    dst->data = (char *) XtMalloc(height * zoom * dst->bytes_per_line);
+    dst->data = (char *) xmalloc(height * zoom * dst->bytes_per_line);
 
-    if (mask != NULL) {
-        dstMask = XCreateImage(dpy, pw->paint.visual,
+    if (mask) {
+        dstMask = XCreateImage(dpy, pp->paint.visual,
 			       mask->depth, mask->format,
 			       0, NULL, width * zoom, height * zoom, 8, 0);
 	dstMask->data = (char *) 
-                     XtMalloc(height * zoom * dstMask->bytes_per_line);
+                     xmalloc(height * zoom * dstMask->bytes_per_line);
 	memset(dstMask->data, 0, height * zoom * dstMask->bytes_per_line);
     }
 
-    if (mask == NULL) {
-	for (dy = 0, y = rect->y; y < endY; y++, dy += zoom) {
-	    for (dx = 0, x = rect->x; x < endX; x++, dx += zoom) {
-		Pixel p;
-		p = xxGetPixel(xim, x, y);
-		for (y1 = 0; y1 < numDraw; y1++) {
-		    for (x1 = 0; x1 < numDraw; x1++) {
-			xxPutPixel(dst, dx + x1, dy + y1, p);
-		    }
-		    if (numDraw != zoom)
-			xxPutPixel(dst, dx + x1, dy + y1,
-				   w->core.background_pixel);
+    for (dy = 0, y = rect->y; y < endY; y++, dy += zoom) {
+	for (dx = 0, x = rect->x; x < endX; x++, dx += zoom) {
+	    if (mask && !XGetPixel(mask, x, y))
+		continue;
+            if (u[3]) {
+                u[0] = alpha[x+y*pw->paint.drawWidth];
+                if (u[3] <= 2) {
+		    p = xxGetPixel(xim, x, y);
+		    if (u[3] == 2) u[2] = ALPHA_PIXEL_MODE(x, y);
+		}
+                p = AlphaTwistPixel(p, u);
+	    } else
+	        p = xxGetPixel(xim, x, y);
+	    for (y1 = 0; y1 < numDraw; y1++) {
+		for (x1 = 0; x1 < numDraw; x1++) {
+		    xxPutPixel(dst, dx + x1, dy + y1, p);
+		    if (mask) XPutPixel(dstMask, dx + x1, dy + y1, True);
 		}
 		if (numDraw != zoom)
-		    for (x1 = 0; x1 < zoom; x1++)
-			xxPutPixel(dst, dx + x1, dy + y1,
-				   w->core.background_pixel);
+		    xxPutPixel(dst, dx + x1, dy + y1,
+			       w->core.background_pixel);
 	    }
+	    if (numDraw != zoom)
+		for (x1 = 0; x1 < zoom; x1++)
+		    xxPutPixel(dst, dx + x1, dy + y1,
+			       w->core.background_pixel);
 	}
-	XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
-		  xstart * zoom, ystart * zoom,
-		  width * zoom, height * zoom);
-    } else {
-	for (dy = 0, y = rect->y; y < endY; y++, dy += zoom) {
-	    for (dx = 0, x = rect->x; x < endX; x++, dx += zoom) {
-	        Pixel p;
+    }
 
-		if (!XGetPixel(mask, x, y))
-		    continue;
-
-		p = xxGetPixel(xim, x, y);
-
-		for (y1 = 0; y1 < numDraw; y1++) {
-		    for (x1 = 0; x1 < numDraw; x1++) {
-			xxPutPixel(dst, dx + x1, dy + y1, p);
-			xxPutPixel(dstMask, dx + x1, dy + y1, True);
-		    }
-		}
-	    }
-	}
+    if (mask) {
         pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
 			    width * zoom, height * zoom, 1);
 
-	XPutImage(XtDisplay(w), pix, pw->paint.mgc, dstMask,
+	XPutImage(XtDisplay(w), pix, pp->paint.mgc, dstMask,
 		  0, 0, 0, 0, width * zoom, height * zoom);
 
 	XSetClipOrigin(XtDisplay(w), gc, xstart * zoom, ystart * zoom);
@@ -868,7 +994,10 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 		  width * zoom, height * zoom);
 	XSetClipMask(XtDisplay(w), gc, None);
 	XFreePixmap(XtDisplay(w), pix);
-    }
+    } else
+	XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
+		  xstart * zoom, ystart * zoom,
+		  width * zoom, height * zoom);
  
  end:
     XDestroyImage(dst);
@@ -886,7 +1015,7 @@ zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect)
     XImage *xim;
 
     if ((isec = RectIntersect(rect, GetRectangle(pw))) == NULL)
-	return;
+        return;
 
     if (zoom>0) {
         zsx = sx = isec->x / zoom;
@@ -919,13 +1048,13 @@ zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect)
     real.width = tmp->width;
     real.height = tmp->height;
 
-    _PwZoomDraw(pw, (Widget) pw, pw->paint.tgc, xim, NULL,
-		isExpose, sx, sy, zoom, &real);
+    PwZoomDraw(pw, (Widget) pw, pw->paint.tgc, xim, NULL,
+	       isExpose, sx, sy, zoom, &real);
 }
 
 /*
 **  pw	= parent paint widget
-**  w	= widget who's window will be updated
+**  w	= widget whose window will be updated
 **  gc	= gc to use (read/written)
 **  src = source XImage to use
 **  mask = mask XImage to use
@@ -937,26 +1066,31 @@ zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect)
 **	   a start x,y with the specified width,height
  */
 void
-_PwZoomDraw(PaintWidget pw, Widget w, GC gc, XImage * src, XImage * mask,
+PwZoomDraw(PaintWidget pw, Widget w, GC gc, XImage * src, XImage * mask,
 	    Boolean flag, int sx, int sy, int zoom, XRectangle * rect)
 {
-    if (zoom>0 && rect->width * rect->height < 1024 + 256)
-	zoomDrawRects(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
+  /*
+    PaintWidget pp = (pw->paint.paint)? (PaintWidget) pw->paint.paint : pw;
+    if (0 && zoom>0 && pp == pw &&
+        rect->width * rect->height < 1024 + 256)
+	ZoomDrawRects(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
     else
-	zoomDrawImage(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
+  */
+    ZoomDrawImage(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
 
     if (pw->paint.grid)
 	PwDrawVisibleGrid(pw, w, True, sx, sy, 
                           sx + rect->width, sy + rect->height);
 }
 
-
 void
 PwUpdateDrawable(Widget w, Drawable draw, XRectangle * rect)
 {
     PaintWidget pw = (PaintWidget) w;
+    PaintWidget pp = (pw->paint.paint)? (PaintWidget)pw->paint.paint : pw;
     XRectangle all;
     Pixmap pix = GET_PIXMAP(pw);
+    XImage *xim;
 
     if (rect == NULL || rect->width == 0 || rect->height == 0) {
 	rect = &all;
@@ -964,17 +1098,37 @@ PwUpdateDrawable(Widget w, Drawable draw, XRectangle * rect)
 	all.width = pw->core.width;
 	all.height = pw->core.height;
     }
-    XCopyArea(XtDisplay(w), pix, draw, pw->paint.gc,
-	      rect->x + pw->paint.zoomX,
-	      rect->y + pw->paint.zoomY,
-	      rect->width, rect->height,
-	      rect->x, rect->y);
+
+    if (pp->paint.current.alpha && pp->paint.alpha_mode!=0 &&
+        draw == XtWindow(w)) {        
+        xim = XGetImage(XtDisplay(w), pix, 
+                        rect->x + pw->paint.zoomX,
+                        rect->y + pw->paint.zoomY,
+                        rect->width, rect->height,
+                        AllPlanes, ZPixmap);
+        AlphaTwistImage(pw, xim, pp->paint.current.alpha,
+                        pw->paint.drawWidth,
+                        rect->x + pw->paint.zoomX,
+                        rect->y + pw->paint.zoomY,
+                        rect->x + pw->paint.zoomX,
+                        rect->y + pw->paint.zoomY);
+        XPutImage(XtDisplay(w), draw, pw->paint.gc,
+	          xim, 0, 0, rect->x, rect->y,
+	          rect->width, rect->height);
+        XDestroyImage(xim);
+    } else {
+        XCopyArea(XtDisplay(w), pix, draw, pw->paint.gc,
+	          rect->x + pw->paint.zoomX,
+	          rect->y + pw->paint.zoomY,
+	          rect->width, rect->height,
+	          rect->x, rect->y);
+    }
 }
 
 static void
 realExposeProc(PaintWidget pw, XExposeEvent * event, Boolean flag)
 {
-  int x, y, width, height, zoom;
+    int x, y, width, height, zoom;
     XRectangle rect;
 
     if (!XtIsRealized(((Widget) pw)))
@@ -1006,6 +1160,7 @@ realExposeProc(PaintWidget pw, XExposeEvent * event, Boolean flag)
 	height = pw->core.height - y;
 
     zoom = GET_ZOOM(pw);
+
     rect.x = x;
     rect.y = y;
     rect.width = width+(zoom<0);
@@ -1015,7 +1170,7 @@ realExposeProc(PaintWidget pw, XExposeEvent * event, Boolean flag)
 	PwUpdateDrawable((Widget) pw, XtWindow((Widget) pw), &rect);
         if (pw->paint.grid)
 	    PwDrawVisibleGrid(pw, (Widget)pw, True, x, y, 
-                             x + rect.width, y + rect.height);
+                              x + rect.width, y + rect.height);
     } else
 	zoomUpdate(pw, flag, &rect);
 
@@ -1051,8 +1206,8 @@ resizePixmap(PaintWidget w, AlphaPixmap * alphapix)
     if (alphapix->alpha) {
         GetPixmapWHD(XtDisplay(w), alphapix->pixmap, &w1, &h1, &y);
         y = w->paint.drawWidth * w->paint.drawHeight;
-        alpha = (unsigned char *) XtMalloc(y);
-        memset(alpha, (unsigned char)Global.alpha_default, y);
+        alpha = (unsigned char *) xmalloc(y);
+        memset(alpha, (unsigned char)Global.alpha_bg, y);
         w2 = w1;
         if (w2>w->paint.drawWidth) w2 = w->paint.drawWidth;
         h2 = h1;
@@ -1188,7 +1343,7 @@ SetValuesProc(Widget curArg, Widget request,
 	event.height = new->core.height;
 
 	realExposeProc(new, &event, False);
-	pwRegionZoomPosChanged(new);
+	PwRegionZoomPosChanged(new);
     }
     zoom = GET_ZOOM(new);
     if (cur->paint.grid != new->paint.grid) {
@@ -1242,7 +1397,7 @@ SetValuesProc(Widget curArg, Widget request,
                 if (cur->paint.current.alpha) {
                     int m = cur->paint.drawWidth * cur->paint.drawHeight;
 		    if (!this->alphapix.alpha)
-		        this->alphapix.alpha = XtMalloc(m);
+		        this->alphapix.alpha = xmalloc(m);
                     memcpy(this->alphapix.alpha, cur->paint.current.alpha, m);
 		}
 	    } else {
@@ -1256,7 +1411,7 @@ SetValuesProc(Widget curArg, Widget request,
 			       new->paint.drawWidth, new->paint.drawHeight);
                 if (this->alphapix.alpha)
 		    memset(this->alphapix.alpha, 
-                           (unsigned char) Global.alpha_default, 
+                           (unsigned char) Global.alpha_bg, 
                            new->paint.drawWidth * new->paint.drawHeight);
 	    }
 	}
@@ -1383,7 +1538,7 @@ PwGetImage(Widget w, XRectangle * rect)
 				       pw->paint.drawWidth, pw->paint.drawHeight,
 				       32, 0);
 	pw->paint.image->data = (char *)
-	    XtMalloc(pw->paint.drawHeight * pw->paint.image->bytes_per_line);
+	    xmalloc(pw->paint.drawHeight * pw->paint.image->bytes_per_line);
 	pw->paint.imageRegion = XCreateRegion();
 	pw->paint.invalidateRegion = False;
     }
@@ -1483,12 +1638,13 @@ fatCallback(Widget parent, XtPointer w, XtPointer rectArg)
 {
     XRectangle *rect = (XRectangle *) rectArg;
     PaintWidget pw = (PaintWidget) w;
-    XExposeEvent event;
+    static XExposeEvent event;
     int zoom = GET_ZOOM(pw);
 
     /*
     **	Make this look like an expose event on the fatbits region
      */
+
     if (zoom>0) {
         event.x = (rect->x - pw->paint.zoomX) * zoom;
         event.y = (rect->y - pw->paint.zoomY) * zoom;
@@ -1542,11 +1698,81 @@ PwUpdate(Widget w, XRectangle * rect, Boolean force)
     XtCallCallbackList((Widget) usePW, usePW->paint.fatcalls,
 		       (XtPointer) rect);
 
-    if (force || (abs(pw->paint.zoom) > 1) || parent != None)
-	fatCallback((Widget) usePW, (XtPointer) usePW, (XtPointer) rect);
+    /* ugly hack to update alpha channel... */
+    if (usePW->paint.current.alpha) {
+        XImage *xim, *ximp;
+        int x, y, x1, y1, x2, y2;
+        Pixmap pix, pixp;
+        unsigned char c;
+        if (usePW->paint.undostack == NULL) goto finish;
+        if (usePW->paint.undoitems <= 0) goto finish;
+        x = getIndexOp();
+        if (1 << x != Global.operation) goto finish;
+        if ((x%3)==2 && x>=14 && x<=26) goto finish;
+        if (x==11) 
+	    c = (unsigned char)Global.alpha_bg;
+        else
+	    c = (unsigned char)Global.alpha_fg;
+        x1 = rect->x;
+        if (x1<0) x1 = 0;
+        if (x1>usePW->paint.drawWidth) x1 = usePW->paint.drawWidth;
+        x2 = rect->x+rect->width;
+        if (x2<0) x2 = 0;
+        if (x2>usePW->paint.drawWidth) x2 = usePW->paint.drawWidth;
+        x2 = x2-x1;
+        if (x2<0) { x1 = x1+x2; x2 = -x2; }
+        y1 = rect->y;
+        if (y1<0) y1 = 0;
+        if (y1>usePW->paint.drawHeight) y1 = usePW->paint.drawHeight;
+        y2 = rect->y+rect->height;
+        if (y2<0) y2 = 0;
+        if (y2>usePW->paint.drawHeight) y2 = usePW->paint.drawHeight;
+        y2 = y2-y1;
+        if (y2<0) { y1 = y1+y2; y2 = -y2; }
+        if (x2==0 || y2==0) goto finish;
+        pix = GET_PIXMAP(usePW);
+        if (!pix) goto finish;
+        x = (usePW->paint.undobot+usePW->paint.undoitems-1) % usePW->paint.nlevels;
+        pixp = usePW->paint.undostack[x].pixmap;
+        if (!pixp) goto finish;
+        xim = XGetImage(XtDisplay(w), pix, x1, y1, x2, y2, AllPlanes, ZPixmap);
+        if (!xim) goto finish;
+        ximp = XGetImage(XtDisplay(w), pixp, x1, y1, x2, y2, AllPlanes,ZPixmap);
+        if (!ximp) goto finishp;
+        for (y = 0; y < y2; y++)
+	    for (x = 0; x < x2; x++) {
+	        if ((xxGetPixel(xim, x, y)) !=  (xxGetPixel(ximp, x, y)))
+ 	     usePW->paint.current.alpha[(x+x1)+(y+y1)*usePW->paint.drawWidth]=c;
+	    }
+	XDestroyImage(ximp);
+    finishp:
+	XDestroyImage(xim);
+    }
 
-    if (pw->paint.grid)
-	PwDrawVisibleGrid(pw, w, True, rect->x, rect->y, 
+    finish:
+    if (force || (abs(pw->paint.zoom)>1) || parent ||
+        (pw->paint.current.alpha && pw->paint.alpha_mode)) {
+        if (pw->paint.zoom>1 && usePW!=pw) {
+           XExposeEvent event;
+	   /*
+           event.x = (rect->x - pw->paint.zoomX) * pw->paint.zoom;
+	   event.y = (rect->y - pw->paint.zoomY) * pw->paint.zoom;
+           event.width = rect->width * pw->paint.zoom;
+           event.height = rect->height * pw->paint.zoom;
+	   */
+           /* JPD : curious hack - don't know why this is necessary... */
+           event.x = 0;
+           event.y = 0;
+           XtVaGetValues((Widget)pw, 
+                         XtNwidth, &event.width, XtNheight, &event.height,
+                         NULL);
+           realExposeProc(pw, &event, False);
+	} else
+           fatCallback((Widget) usePW, (XtPointer) pw, rect);
+    }
+
+    if (usePW->paint.grid)
+	PwDrawVisibleGrid(usePW, w, True, rect->x, rect->y, 
                           rect->x + rect->width, rect->y + rect->height);
 
     XFlush(XtDisplay(pw));
@@ -1609,7 +1835,7 @@ PwAddChild(Widget w, Widget child)
 
     if (pw->paint.paintChildren == NULL)
 	pw->paint.paintChildren = (Widget *)
-	    XtMalloc(sizeof(Widget) * (pw->paint.paintChildrenSize + 2));
+	    xmalloc(sizeof(Widget) * (pw->paint.paintChildrenSize + 2));
     else
 	pw->paint.paintChildren = (Widget *)
 	    XtRealloc((XtPointer) pw->paint.paintChildren,
