@@ -68,7 +68,6 @@
 #define XtNdrawHeight	 "drawHeight"
 #define XtNcompress	 "compress"
 #define XtNdirty	 "dirty"
-#define XtNeditable	 "editable"
 #define XtNfillRule	 "fillRule"
 #define XtNregionCallback	"regionSetCallback"
 #define XtNdownX		"downX"
@@ -161,13 +160,6 @@ typedef struct {
     Pixmap base;
 } OpInfo;
 
-typedef struct {
-    float wpercent, hpercent;
-    int wbbox, hbbox, width, height, rx, ry, wsubdiv, hsubdiv;
-    Boolean orient, eps, gray, compress, pdf;
-    char *title;
-} PageInfo;
-
 typedef float pwMatrix[2][2];
 
 /* Brush Item structure */
@@ -210,6 +202,11 @@ void PwPutPixmap(Widget w, Pixmap pix);
 XRectangle *PwScaleRectangle(Widget, XRectangle *);
 XImage *PwGetImage(Widget, XRectangle *);
 void zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect);
+Pixel AlphaTwistPixel(Pixel p, unsigned char *u);
+Pixel AlphaTwistPixel8(Pixel p, unsigned char *u);
+void AlphaTwistImage(PaintWidget w, XImage *xim, 
+                     unsigned char *alpha, int width,
+                     int dx, int dy, int ax, int ay);
 
 /*
 **  Region routines
