@@ -25,7 +25,6 @@
 #include <X11/CoreP.h>
 #include <X11/CompositeP.h>
 #include <X11/Xutil.h>
-#include <X11/Xft/Xft.h>
 
 /* define unique representation types not found in <X11/StringDefs.h> */
 
@@ -69,13 +68,15 @@ typedef struct {
     int undobot, undoitems, redobot, redoitems;
     int nlevels;		/* Maximum stack depth */
     int undoSize;		/* number of undo stacks */
+    int alpha_mode;             /* alpha_mode = 0,1,2,3 default 0 */
 
-    Boolean dirty, editable, locked, menubar, fullmenu, snapOn, grid, compress;
+    Boolean dirty, locked, menubar, fullmenu, snapOn, grid, compress;
     int snap_x, snap_y, gridmode, interpolation, transparent;
     void *eventList;
     XtCallbackList fatcalls;
     XtCallbackList sizecalls;
     XtCallbackList regionCalls;
+
     int zoomX, zoomY, zoom;
     GC tgc;			/* scratch GC, for stuffing clip masks into */
     GC mgc;			/* region mask GC, 1 bit deep */
@@ -111,6 +112,8 @@ typedef struct {
 
 	Boolean isAttached, isVisible, needResize;
 	Pixmap unfilterPixmap;
+        AlphaPixmap undo_alphapix;
+
 	/*
 	**  The scale{X,Y} are combined with the rotMat
 	**    to produce mat.
@@ -159,9 +162,11 @@ typedef struct _PaintRec {
 #define GET_MGC(pw, msk)	((pw)->paint.mgc == None ? \
 	(pw->paint.mgc = XCreateGC(XtDisplay(pw), msk, 0, 0)) : pw->paint.mgc)
 
-void pwRegionZoomPosChanged(PaintWidget);
-void _PwZoomDraw(PaintWidget, Widget, GC, XImage *, XImage *,
-		 Boolean, int, int, int, XRectangle *);
+#define ALPHA_PIXEL_MODE(x, y)     ((((x)/12+(y)/12)&1)? 153:102)
+
+void PwRegionZoomPosChanged(PaintWidget);
+void PwZoomDraw(PaintWidget, Widget, GC, XImage *, XImage *,
+		Boolean, int, int, int, XRectangle *);
 
 /*
 **  Try to optimize XGetPixel and XPutPixel
@@ -172,11 +177,11 @@ void _PwZoomDraw(PaintWidget, Widget, GC, XImage *, XImage *,
 		((img->bits_per_pixel == 8) ?			\
 			img->data[ZINDEX8(x, y, img)] :		\
 			XGetPixel(img, x, y))
-#define xxPutPixel(img, x, y, p) do {				\
+#define xxPutPixel(img, x, y, p) {				\
 		if (img->bits_per_pixel == 8)			\
 			img->data[ZINDEX8(x, y, img)] = p;	\
 		else						\
 			XPutPixel(img, x, y, p);		\
-	} while (0)
+	}
 
 #endif				/* _PaintP_h */
