@@ -40,6 +40,7 @@
 
 #include "xpaint.h"
 #include "PaintP.h"
+#include "image.h"
 #include "protocol.h"
 
 #define SHAPE
@@ -114,10 +115,9 @@ static pwMatrix matIdentity =
 static void 
 doCallbacks(PaintWidget pw, int flag)
 {
-    PaintWidget tpw = (pw->paint.paint == None) ?
-    pw : (PaintWidget) pw->paint.paint;
+    PaintWidget tpw = (pw->paint.paint)? (PaintWidget) pw->paint.paint : pw;
     int i;
-
+ 
     XtCallCallbackList((Widget) tpw, tpw->paint.regionCalls, 
                        (XtPointer)(long)flag);
     for (i = 0; i < tpw->paint.paintChildrenSize; i++) {
@@ -178,6 +178,7 @@ buildSources(PaintWidget pw)
 void 
 RegionTransparency(PaintWidget pw)
 {
+    PaintWidget tpw;
     Widget w;
     XImage *src1, *src2, *src3;
     Pixel pix;
@@ -188,6 +189,7 @@ RegionTransparency(PaintWidget pw)
     int opaque, transparent;
 
     if (pw->paint.region.source == None) return;
+    tpw = (pw->paint.paint)? (PaintWidget)pw->paint.paint : pw;
 
     XtVaGetValues((Widget)pw, XtNtransparent, &transparent, NULL);
     if (transparent==0) return;
@@ -248,6 +250,13 @@ RegionTransparency(PaintWidget pw)
     src1 = XGetImage(XtDisplay(pw), pw->paint.region.source,
        	            (zoom==1)?rx:0, (zoom==1)?ry:0, widthp, heightp,
  		    AllPlanes, ZPixmap);
+   
+    if (tpw->paint.alpha_mode && tpw->paint.region.alpha)
+        AlphaTwistImage(tpw, src1, tpw->paint.region.alpha, 
+                        src1->width, 
+                        0, 0,
+                        pw->paint.region.orig.x, pw->paint.region.orig.y);
+
     if (!src1) return;
  
     if ((zoom==1) && opaque) {
@@ -258,6 +267,11 @@ RegionTransparency(PaintWidget pw)
  		         px, py, widthp, heightp,
  		         AllPlanes, ZPixmap);
         if (src2) {
+            if (tpw->paint.alpha_mode && tpw->paint.current.alpha)
+                AlphaTwistImage(tpw, src2, tpw->paint.current.alpha, 
+                        tpw->paint.drawWidth, 
+                        px, py,
+                        px, py);
             bytes_per_pixel = src2->bits_per_pixel/8;
             if (zoom>0)
             for (y=0; y<heightp; y++) {
@@ -302,7 +316,7 @@ RegionTransparency(PaintWidget pw)
 	}
         bytes_per_pixel = src3->bits_per_pixel/8;
         XtVaGetValues((Widget)pw, XtNbackground, &pix, NULL);
-        XPutPixel(src3, 0, 0, pix);
+        xxPutPixel(src3, 0, 0, pix);
         memcpy(bg, src3->data, bytes_per_pixel);
         if (zoom>0) {
             skip = -(zoom<=ZOOM_THRESH);
@@ -480,7 +494,7 @@ resizeImg(PaintWidget pw, pwMatrix inv, Pixmap * pix, XImage * pixSrc,
 		    XPutPixel(shapeDst, x, y, True);
 #endif
 	    } else {
-                XPutPixel(pixDst, x, y, pixel);
+                xxPutPixel(pixDst, x, y, pixel);
 	        if (maskSrc != NULL)
 		    XPutPixel(maskDst, x, y, False);
 #ifdef SHAPE
@@ -1262,7 +1276,7 @@ PwRegionExpose(Widget w, PaintWidget pw, XEvent * event, Boolean * junk)
     int zoom = GET_ZOOM(pw);
     int pixW, pixH;
     int dx, dy;
-
+  
     if (!pw->paint.region.isVisible)
 	return;
 
@@ -1329,10 +1343,26 @@ PwRegionExpose(Widget w, PaintWidget pw, XEvent * event, Boolean * junk)
 
     if (isExpose) {
 	if (zoom == 1) {
-	    XCopyArea(XtDisplay(w), pw->paint.region.source, XtWindow(w),
-		      pw->paint.region.fg_gc == None ?
-		      pw->paint.tgc : pw->paint.region.fg_gc,
-		rect.x, rect.y, rect.width, rect.height, rect.x, rect.y);
+	    if (pw->paint.region.alpha && pw->paint.alpha_mode) {
+	        src = XGetImage(XtDisplay(pw), pw->paint.region.source,
+			        rect.x, rect.y, rect.width, rect.height,
+			        AllPlanes, ZPixmap);
+                AlphaTwistImage(pw, src, pw->paint.region.alpha, 
+                                pw->paint.region.rect.width,
+                                rect.x, rect.y, 
+                                rect.x+pw->paint.region.orig.x,
+                                rect.y+pw->paint.region.orig.y);
+                XPutImage(XtDisplay(w), XtWindow(w),
+                          pw->paint.region.fg_gc == None ?
+		          pw->paint.tgc : pw->paint.region.fg_gc, src,
+                          0, 0, rect.x, rect.y, rect.width, rect.height);
+	        XDestroyImage(src);
+	    } else
+	        XCopyArea(XtDisplay(w), pw->paint.region.source, XtWindow(w),
+		          pw->paint.region.fg_gc == None ?
+		          pw->paint.tgc : pw->paint.region.fg_gc,
+		          rect.x, rect.y, rect.width, rect.height, 
+                          rect.x, rect.y);
             if (pw->paint.grid)
 	        PwDrawVisibleGrid(pw, w, True, rect.x, rect.y, 
                                   rect.x+rect.width, rect.y+rect.height);
@@ -1342,17 +1372,25 @@ PwRegionExpose(Widget w, PaintWidget pw, XEvent * event, Boolean * junk)
 	    tr.y = 0;
             tr.width = width;
             tr.height = height;
-
 	    src = XGetImage(XtDisplay(pw), pw->paint.region.source,
 			    nrect.x, nrect.y, nrect.width, nrect.height,
 			    AllPlanes, ZPixmap);
-
+	    tpw = (pw->paint.paint) ? (PaintWidget) pw->paint.paint : pw;
+	    if (tpw->paint.region.alpha && tpw->paint.alpha_mode)
+	        AlphaTwistImage(pw, src, tpw->paint.region.alpha, 
+                                pw->paint.region.orig.width,
+                                nrect.x, nrect.y, 
+                                nrect.x + pw->paint.region.orig.x,
+                                nrect.y + pw->paint.region.orig.y);	    
 	    if (pw->paint.region.mask != None)
 		mask = XGetImage(XtDisplay(pw), pw->paint.region.mask,
 			        nrect.x, nrect.y, nrect.width, nrect.height,
 			        AllPlanes, ZPixmap);
-	    _PwZoomDraw(pw, w, pw->paint.tgc, src, mask,
-			False, nrect.x, nrect.y, zoom, &tr);
+            dx =  tpw->paint.alpha_mode;
+            tpw->paint.alpha_mode = 0;
+	    PwZoomDraw(pw, w, pw->paint.tgc, src, mask,
+		       False, nrect.x, nrect.y, zoom, &tr);
+            tpw->paint.alpha_mode = dx;
 	    XDestroyImage(src);
 	    if (mask != NULL)
 		XDestroyImage(mask);
@@ -1372,9 +1410,8 @@ PwRegionExpose(Widget w, PaintWidget pw, XEvent * event, Boolean * junk)
 	**  Copy in the background picture
 	 */
 	if (zoom != 1) {
-	    tpw = (pw->paint.paint == None) ?
-	          pw : (PaintWidget) pw->paint.paint;
-             
+	    tpw = (pw->paint.paint) ? (PaintWidget) pw->paint.paint : pw;
+
             if (zoom>0) {
 	        tr.x = (nrect.x + x) / zoom + pw->paint.zoomX;
 	        tr.y = (nrect.y + y) / zoom + pw->paint.zoomY;
@@ -1411,14 +1448,15 @@ PwRegionExpose(Widget w, PaintWidget pw, XEvent * event, Boolean * junk)
 		xim = XGetImage(XtDisplay(w), GET_PIXMAP(pw),
 				tr.x, tr.y, tr.width, tr.height,
 				AllPlanes, ZPixmap);
+
 		tr.x = tr.y = 0;
                 if (pw->paint.region.notMask) {
 		    mask = XGetImage(XtDisplay(pw), pw->paint.region.notMask,
 		   	        nrect.x + dx, nrect.y + dy, tr.width, tr.height,
 				AllPlanes, ZPixmap);
                     if (mask) {
-		        _PwZoomDraw(pw, w, pw->paint.tgc, xim, mask, False,
-			            dx, dy, zoom, &tr);
+		        PwZoomDraw(pw, w, pw->paint.tgc, xim, mask, False,
+			           dx, dy, zoom, &tr);
 		        XDestroyImage(mask);
 		    }
 		}
@@ -1590,19 +1628,20 @@ writeRegionAlphaChannel(PaintWidget pw)
     int i, j, x, y, x0, x1, y0, y1;
     pwMatrix inv;
     float sx, sy, dx, dy, cx, cy;
+    PaintWidget tpw = (pw->paint.paint) ? (PaintWidget)pw->paint.paint : pw;
 
-    if (!pw->paint.current.alpha && !pw->paint.region.alpha) return;
+    if (!tpw->paint.current.alpha && !tpw->paint.region.alpha) return;
 
-    if (!pw->paint.region.alpha) {
+    if (!tpw->paint.region.alpha) {
         i = pw->paint.region.orig.width * pw->paint.region.orig.height;
-        pw->paint.region.alpha = (unsigned char *) XtMalloc(i);
-        memset(pw->paint.region.alpha, (unsigned char)Global.alpha_default, i);
+        tpw->paint.region.alpha = (unsigned char *) XtMalloc(i);
+        memset(tpw->paint.region.alpha, (unsigned char)Global.alpha_bg, i);
     }
     
-    if (!pw->paint.current.alpha) {
-	i = pw->paint.drawWidth * pw->paint.drawHeight;
-        pw->paint.current.alpha = (unsigned char *) XtMalloc(i);
-        memset(pw->paint.current.alpha, (unsigned char)Global.alpha_default, i);
+    if (!tpw->paint.current.alpha) {
+	i = tpw->paint.drawWidth * tpw->paint.drawHeight;
+        tpw->paint.current.alpha = (unsigned char *) XtMalloc(i);
+        memset(tpw->paint.current.alpha, (unsigned char)Global.alpha_bg, i);
     }
 
     if (pw->paint.region.mask != None && pw->paint.region.maskImg == NULL) {
@@ -1616,9 +1655,9 @@ writeRegionAlphaChannel(PaintWidget pw)
 
     cx = (pw->paint.region.orig.width+1) * 0.5;
     cy = (pw->paint.region.orig.height+1) * 0.5;
-    dx = pw->paint.region.rect.x + pw->paint.region.rect.width * 0.5;
-    dy = pw->paint.region.rect.y + pw->paint.region.rect.height * 0.5;
-    
+    dx = pw->paint.region.rect.x + pw->paint.region.orig.width * 0.5;
+    dy = pw->paint.region.rect.y + pw->paint.region.orig.height * 0.5;
+
     XFORM(-cx, -cy, pw->paint.region.mat, sx, sy);
     x1 = x0 = (int)(sx + dx + 0.5);
     y1 = y0 = (int)(sy + dy + 0.5);
@@ -1644,20 +1683,17 @@ writeRegionAlphaChannel(PaintWidget pw)
     if (x1>=pw->paint.drawWidth) x1 = pw->paint.drawWidth-1;
     if (y1>=pw->paint.drawHeight) y1 = pw->paint.drawHeight-1;
 
-    cx = pw->paint.region.orig.width * 0.5;
-    cy = pw->paint.region.orig.height * 0.5;
-
     for (y=y0; y<=y1; y++)
     for (x=x0; x<=x1; x++) {
         XFORM((x-dx), (y-dy), inv, sx, sy);
-        i = (int)(sx + cx + 0.5);
-        j = (int)(sy + cy + 0.5);
+        i = (int)(sx + cx);
+        j = (int)(sy + cy);
         if (i>=0 && i<pw->paint.region.orig.width &&
             j>=0 && j<pw->paint.region.orig.height) {
             if (pw->paint.region.mask == None ||
                 XGetPixel(pw->paint.region.maskImg, i, j)) {
-                pw->paint.current.alpha[x+y*pw->paint.drawWidth] =
-		    pw->paint.region.alpha[i+j*pw->paint.region.orig.width];
+                tpw->paint.current.alpha[x+y*tpw->paint.drawWidth] =
+		    tpw->paint.region.alpha[i+j*pw->paint.region.orig.width];
 	    }
 	}
     }
@@ -1701,14 +1737,15 @@ writeRegion(PaintWidget pw)
     /*
     **  If we've already modified the background (aka ripped the
     **    image up)  then don't get a new undo buffer.
-    **    The second case of the if is if you've done an undo inbetween.
+    **    The second case of the if is if you've done an undo in between.
      */
-    /*
-    pix = PwUndoStart((Widget) pw, &nr);
+    if (pw->paint.region.undo_alphapix.pixmap == None ||
+	pw->paint.region.undo_alphapix.pixmap != GET_PIXMAP(pw))
+	pix = PwUndoStart((Widget) pw, &nr);
+    else
+	pix = pw->paint.region.undo_alphapix;
+
     PwUndoAddRectangle((Widget) pw, &nr);
-    */
-   
-    pix = pw->paint.current;
 
     if (pw->paint.region.fg_gc != None) {
 	gc = pw->paint.region.fg_gc;
@@ -1728,6 +1765,7 @@ writeRegion(PaintWidget pw)
 		       pw->paint.region.orig.height, AllPlanes, ZPixmap);
 	    made = True;
 	}
+
 	src = (*pw->paint.region.proc) ((Widget) pw, sim, pw->paint.region.mat);
 	if (made)
 	    XDestroyImage(sim);
@@ -1799,7 +1837,7 @@ zoomValueChanged(Widget w, XtPointer junk1, XtPointer junk2)
 }
 
 void 
-pwRegionZoomPosChanged(PaintWidget pw)
+PwRegionZoomPosChanged(PaintWidget pw)
 {
     int nx, ny;
     int zoom;
@@ -1861,6 +1899,9 @@ writeCleanRegion(PaintWidget pw, Boolean flag, Boolean write)
 	pw->paint.region.unfilterPixmap = None;
     }
 
+    pw->paint.region.undo_alphapix.pixmap = None;
+    pw->paint.region.undo_alphapix.alpha = NULL;
+
     if (flag) {
 	if (pw->paint.region.child != None)
 	    XtUnmapWidget(pw->paint.region.child);
@@ -1880,7 +1921,7 @@ PwRegionFinish(Widget w, Boolean flag)
     int i;
     
     if (flag) {
-	tpw = (pp == None) ? pw : pp;
+	tpw = (pp) ? pp : pw;
 	writeCleanRegion(tpw, True, True);
 	for (i = 0; i < tpw->paint.paintChildrenSize; i++)
 	    writeCleanRegion((PaintWidget) tpw->paint.paintChildren[i], True, True);
@@ -1905,7 +1946,7 @@ PwRegionOff(Widget w, Boolean flag)
 
     tpw = pw;
     if (flag)
-	tpw = (pp == None) ? pw : pp;
+	tpw = (pp) ? pp : pw;
 
     if (!tpw->paint.region.isVisible)
 	return False;
@@ -1924,16 +1965,19 @@ void
 PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
 {
     PaintWidget pw = (PaintWidget) w;
+    PaintWidget tpw = (pw->paint.paint)? (PaintWidget)pw->paint.paint : pw;
     int i, j, k1, k2, l1, l2;
     int nx, ny, x, y, width, height;
     int zoom = GET_ZOOM(pw);
     Boolean setIsAttached = False;
 
+
     /*
     **  If there is an image, write it
     **     rect == NULL, then this is just a "write" & "unmap" request
      */
     PwRegionFinish(w, True);
+
     if (pw->paint.region.sourceImg) {
         XDestroyImage(pw->paint.region.sourceImg);
         pw->paint.region.sourceImg = NULL;
@@ -2003,7 +2047,7 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
     /* 
      * import alpha channel from PaintWidget
      */
-    if (setIsAttached && pw->paint.current.alpha) {
+    if (setIsAttached && tpw->paint.current.alpha) {
         if (pw->paint.region.mask == None) 
 	    pw->paint.region.maskImg = NULL;
         else
@@ -2012,8 +2056,8 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
 		       pw->paint.region.orig.width,
 		       pw->paint.region.orig.height, AllPlanes, ZPixmap);
         i = rect->width * rect->height;
-        pw->paint.region.alpha = (unsigned char *) XtMalloc(i);
-        memset(pw->paint.region.alpha, (unsigned char)Global.alpha_default, i);
+        tpw->paint.region.alpha = (unsigned char *) XtMalloc(i);
+        memset(tpw->paint.region.alpha, (unsigned char)Global.alpha_bg, i);
         k1 = -pw->paint.region.rect.x;
         if (k1<0) k1 = 0;
         k2 = pw->paint.region.orig.width;
@@ -2030,17 +2074,17 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
             if (pw->paint.region.mask != None) {
                 for (x=k1; x<k2; x++) {
 		    if (XGetPixel(pw->paint.region.maskImg, x, y)) {
-		        pw->paint.region.alpha[x+i] = pw->paint.current.alpha[x+j];
+		        tpw->paint.region.alpha[x+i] = tpw->paint.current.alpha[x+j];
                         if (setIsAttached) 
-                            pw->paint.current.alpha[x+j] = 
-			        (unsigned char) Global.alpha_default;
+                            tpw->paint.current.alpha[x+j] = 
+			        (unsigned char) Global.alpha_bg;
 		    }
 	        }
             } else {
-                memcpy(pw->paint.region.alpha+i+k1, pw->paint.current.alpha+j+k1, k2-k1);
+                memcpy(tpw->paint.region.alpha+i+k1, tpw->paint.current.alpha+j+k1, k2-k1);
                 if (setIsAttached) 
-                    memset(pw->paint.current.alpha+j+k1, 
-                           (unsigned char) Global.alpha_default, k2-k1);
+                    memset(tpw->paint.current.alpha+j+k1, 
+                           (unsigned char) Global.alpha_bg, k2-k1);
 	    }
         }
     }
@@ -2191,8 +2235,7 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
 static PaintWidget
 getActiveRegion(PaintWidget pw)
 {
-    PaintWidget pp = (PaintWidget) pw->paint.paint;
-    PaintWidget tpw = (pp == None) ? pw : pp;
+    PaintWidget tpw = (pw->paint.paint) ? (PaintWidget) pw->paint.paint : pw;
     int i;
 
     if (pw->paint.region.isVisible && pw->paint.region.source != None)
@@ -2326,6 +2369,7 @@ PwRegionTear(Widget w)
     int zoom;
 
     pw = getActiveRegion((PaintWidget) w);
+
     if (pw == None)
 	return;
     if (!pw->paint.region.isAttached || !pw->paint.region.isVisible)
@@ -2342,14 +2386,17 @@ PwRegionTear(Widget w)
         nr.height *= (-zoom);
     }
 
+    pw->paint.region.undo_alphapix = PwUndoStart((Widget) pw, &nr);
+
     if (pw->paint.region.mask != None) {
 	XSetClipOrigin(XtDisplay(pw), pw->paint.igc, nr.x, nr.y);
 	XSetClipMask(XtDisplay(pw), pw->paint.igc, pw->paint.region.mask);
     }
-    XFillRectangles(XtDisplay(pw), pw->paint.current.pixmap,
-		    pw->paint.igc, &nr, 1);
 
-    PwUpdate((Widget) pw, &nr, False);
+    XFillRectangles(XtDisplay(pw), pw->paint.region.undo_alphapix.pixmap,
+                    pw->paint.igc, &nr, 1);
+
+    PwUpdate((Widget) pw, &nr, True);
 
     if (pw->paint.region.mask != None) {
 	XSetClipOrigin(XtDisplay(pw), pw->paint.igc, 0, 0);
@@ -2507,6 +2554,7 @@ RegionMove(PaintWidget pw, int dx, int dy)
 
     if (getActiveRegion(pw) == None)
 	return;
+
     PwRegionTear((Widget) pw);
 
     pw->paint.region.rect.x += dx;
