@@ -38,11 +38,16 @@ static AlphaPixmap RedoPop(PaintWidget pw);
 void
 UndoInitialize(PaintWidget pw, int n)
 {
+    int i;
     pw->paint.nlevels = n;
     pw->paint.undostack = n ? xmalloc(n*sizeof(AlphaPixmap)) : NULL;
     pw->paint.redostack = n ? xmalloc(n*sizeof(AlphaPixmap)) : NULL;
     pw->paint.undobot = pw->paint.undoitems = 0;
     pw->paint.redobot = pw->paint.redoitems = 0;
+    for (i=0; i<n; i++) {
+        pw->paint.undostack[i].pixmap = None;
+        pw->paint.undostack[i].alpha = NULL;
+    }
 }
 
 static void
