@@ -885,8 +885,11 @@ setStandardCursor(Widget w)
 void
 selectBrush(Widget w, XtPointer nc)
 {
+    Display *dpy = XtDisplay(w);
+    GC gc;
     BrushItem * brush;
-    int i;
+    int i, j, s, t, u, v;
+    Pixmap oldpix = Global.brushpix;
 
     currentBrush = (int)((long)nc);
     brush =  &brushList[currentBrush];
@@ -894,6 +897,34 @@ selectBrush(Widget w, XtPointer nc)
     for (i=0; i<brushnum; i++)
         XtVaSetValues(brushList[i].icon, XtNstate, (i==currentBrush), NULL);
 
+    /* Create ad hoc icon for canvas button */
+    Global.brushpix = XCreatePixmap(dpy, DefaultRootWindow(dpy),
+                                    ICONWIDTH-8, ICONHEIGHT-8, 
+                                    1);
+    gc = XCreateGC(dpy, Global.brushpix, 0, 0);
+    XFillRectangle(dpy, Global.brushpix,  gc, 0, 0, ICONWIDTH-8, ICONHEIGHT-8);
+  
+    if (brush->width >= ICONWIDTH-8) {
+        i = 0;
+        s = (brush->width-ICONWIDTH+8)/2;
+        u = ICONWIDTH-8;
+    } else {
+        i = (ICONWIDTH-7-brush->width)/2;
+        s = 0;
+        u = brush->width;
+    }
+    if (brush->height >= ICONHEIGHT-8) {
+        j = 0;
+        t = (brush->height-ICONHEIGHT+8)/2;
+        v = ICONHEIGHT-8;
+    } else {
+        j = (ICONHEIGHT-7-brush->height)/2;
+        t = 0;
+        v = brush->height;
+    }
+    XCopyArea(dpy, brush->pixmap, Global.brushpix, gc, s, t, u, v, i, j);
+    XFreeGC(dpy, gc);
+
     if ((CurrentOp->add == BrushAdd) ||
 	(CurrentOp->add == EraseAdd) ||
 	(CurrentOp->add == SmearAdd)) {
@@ -902,6 +933,9 @@ selectBrush(Widget w, XtPointer nc)
     }
     GraphicAll(setBrushIconPixmap, (void *) brush);
 
+    if (oldpix)
+        XFreePixmap(dpy, oldpix);
+
     if (brushImage != NULL)
 	XDestroyImage(brushImage);
     brushImage = NewXImage(XtDisplay(brush->icon), NULL,
@@ -1212,12 +1246,11 @@ void
 BrushSelect(Widget w)
 {
     XtPopup(Global.brushpopup, XtGrabNone);
-    XMapRaised(XtDisplay(Global.brushpopup), XtWindow(Global.brushpopup));
+    RaiseWindow(XtDisplay(Global.brushpopup), XtWindow(Global.brushpopup));
 }
 
 void setBrushIconOnWidget(Widget w)
 {
-    BrushItem * brush = &brushList[currentBrush];
     if (w != None)
-        XtVaSetValues(w, XtNbitmap, brush->pixmap, NULL);
+        XtVaSetValues(w, XtNbitmap, Global.brushpix, NULL);
 }
