@@ -191,7 +191,9 @@ addSB(Widget parent, Widget above, char *title, Widget * bar, Widget * text)
 				    XtNlength, 5,
 				    XtNstring, "0",
 				    NULL);
-
+    XtAddEventHandler(*text, 
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, NULL);
     return form;
 }
 
@@ -302,7 +304,7 @@ ChromaDialog(Widget w, Palette * map)
 	    XtVaSetValues(l->shell, XtNcolormap, map->cmap, NULL);
 	}
 	XtPopup(l->shell, XtGrabNone);
-	XMapRaised(XtDisplay(l->shell), XtWindow(l->shell));
+	RaiseWindow(XtDisplay(l->shell), XtWindow(l->shell));
 	return;
     }
 #endif
