@@ -103,8 +103,12 @@ press(Widget w, LocalInfo * l,
 	return;
     if (event->button == Button3) return;
 
-    SetCapAndJoin(w, info->first_gc, JoinRound);
-    SetCapAndJoin(w, info->second_gc, JoinRound);
+    SetCapAndJoin(w, info->first_gc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinRound));
+    SetCapAndJoin(w, info->second_gc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinRound));
 
     l->rx = info->x;
     l->ry = info->y;
