@@ -653,6 +653,9 @@ Ctrl<Key>J: color-text-ok()\n";
 				    XtNwidth, 50,
 				    XtNlength, 5,
 				    NULL);
+    XtAddEventHandler(*text, 
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, NULL);
     return subform;
 }
 
