@@ -80,6 +80,7 @@ ColorEditor(Widget w, Pixel pixel, Palette * map, Boolean allowWrite,
     LocalInfo *info = XtNew(LocalInfo);
     Arg args[8];
     int nargs = 0;
+    char *ptr;
     
     info->okProc = okProc;
     info->closure = closure;
@@ -140,6 +141,9 @@ ColorEditor(Widget w, Pixel pixel, Palette * map, Boolean allowWrite,
     info->map = map;
 
     XtPopup(shell, XtGrabNone);
+    XtVaGetValues(shell, XtNtitle, &ptr, NULL);
+    StoreName(shell, ptr);
+
     if (pick) {
         XtUnmanageChild(pick);
         XMapWindow(XtDisplay(pick), XtWindow(pick));
