@@ -135,6 +135,7 @@ CreateFatCursor(Widget paint, Pixmap cursor, FatLineCursor * fc)
 
     GetPixmapWHD(dpy, cursor, &w, &h, NULL);
     src = XGetImage(dpy, cursor, 0, 0, w, h, AllPlanes, ZPixmap);
+    
     xoff = w / 2;
     yoff = h / 2;
     for (y = 0; y < h; ++y)
@@ -586,7 +587,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, Boolean * flg)
 
     XtVaGetValues(w, XtNx, &x, XtNy, &y, NULL);
     if (abs(event->time-prev_time) < 300)
-        XMapRaised(XtDisplay(l->shell), XtWindow(l->shell));
+        RaiseWindow(XtDisplay(l->shell), XtWindow(l->shell));
     prev_time = event->time;
 
     l->offX = event->x;
@@ -626,6 +627,36 @@ sizeChanged(Widget w, XtPointer l, XtPointer junk)
     resizeCursor((LocalInfo *) l);
 }
 
+static void 
+refreshCallback(Widget w, XtPointer l, XtPointer junk)
+{
+     XRectangle rect;
+     LocalInfo *li =  (LocalInfo *) l;
+     PaintWidget pw = (PaintWidget) li->view;
+     /* brutal way : update entire window ... */
+     rect.x = 0;
+     rect.y = 0;
+     rect.width = pw->core.width;
+     rect.height = pw->core.height;
+     zoomUpdate(pw, True, &rect);
+     /* update region if any */
+     if (pw->paint.region.child)
+         PwRegionExpose(pw->paint.region.child, pw, NULL, NULL);
+}
+
+static void 
+fatbitsExposed(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
+{
+     XRectangle rect;
+     PaintWidget pw = (PaintWidget) w;
+     /* brutal way : update entire window ... */
+     rect.x = 0;
+     rect.y = 0;
+     rect.width = pw->core.width;
+     rect.height = pw->core.height;
+     zoomUpdate(pw, True, &rect);
+ }
+
 /*
 **  The parent box widget changed size, resize to fit.
  */
@@ -677,7 +708,7 @@ buttonCallback(Widget w, XtPointer lArg, void *junk2)
 	return;
 
     nz = 0;
-    sscanf(lbl, "%*d:%d", &nz);
+    nz = atoi(lbl);
     if (nz == 0)
 	return;
 
@@ -801,14 +832,15 @@ void
 FatbitsEdit(Widget paint)
 {
     Widget shell, form, fat;
-    Widget button, box, prev=None, *zoombut;
+    Widget done, box, prev=None, *zoombut;
+    Widget refresh;
     int i, zoom;
     Colormap cmap;
     Position x, y, lx, ly;
     Dimension width = 48, height = 48;
     WidgetList wlist;
-
     LocalInfo *l;
+    char *ptr;
     Arg args[4];
     int nargs = 0;
     static char *zoomList[] =
@@ -826,7 +858,7 @@ FatbitsEdit(Widget paint)
     for (l = head; l != NULL && l->paint != paint; l = l->next);
 
     if (l != NULL) {
-	XMapRaised(XtDisplay(l->shell), XtWindow(l->shell));
+	RaiseWindow(XtDisplay(l->shell), XtWindow(l->shell));
 	return;
     }
     /* Construct a new fatbits popup */
@@ -850,7 +882,7 @@ FatbitsEdit(Widget paint)
     PaletteAddUser(PaletteFind(shell, cmap), shell);
     form = XtVaCreateManagedWidget("form", formWidgetClass, shell, NULL);
 
-    button = XtVaCreateManagedWidget("done", commandWidgetClass, form,
+    done = XtVaCreateManagedWidget("done", commandWidgetClass, form,
 				     /*
 				     XtNfromVert, box,
 				     XtNtop, XtChainBottom,
@@ -860,20 +892,14 @@ FatbitsEdit(Widget paint)
 				     */
 				     NULL);
 
-    XtAddCallback(button, XtNcallback, doneCallback, (XtPointer) l);
+    XtAddCallback(done, XtNcallback, doneCallback, (XtPointer) l);
 
     zoombut = (Widget *) xmalloc(XtNumber(zoomList) * sizeof(Widget));
     for (i = 0; i < XtNumber(zoomList); i++) {
 	zoombut[i] = XtVaCreateManagedWidget(zoomList[i],
 					 toggleWidgetClass, form,
-					 XtNfromHoriz, (i)?prev:button,
-				         /*
-					 XtNfromVert, box,
-					 XtNtop, XtChainBottom,
-					 XtNbottom, XtChainBottom,
-					 XtNleft, XtChainRight,
-					 XtNright, XtChainRight,
-					 */
+					 XtNheight, 22,
+					 XtNfromHoriz, (i)?prev:done,
 					 XtNradioGroup, prev,
 					 XtNtranslations, trans,
 					 NULL);
@@ -881,10 +907,16 @@ FatbitsEdit(Widget paint)
 	XtAddCallback(prev, XtNcallback, buttonCallback, (XtPointer) l);
 
 	if (i == XtNumber(zoomList) / 2) {
-	    XtVaSetValues(button, XtNstate, True, NULL);
+	    XtVaSetValues(prev, XtNstate, True, NULL);
 	}
     }
 
+    refresh = XtVaCreateManagedWidget("refresh", commandWidgetClass, form,
+                                      XtNfromHoriz, 
+                                          zoombut[XtNumber(zoomList)-1],
+				      NULL);
+    XtAddCallback(refresh, XtNcallback, refreshCallback, (XtPointer) l);
+
     box = XtVaCreateManagedWidget("paintBox", boxWidgetClass, form,
 			  XtNbackgroundPixmap, GetBackgroundPixmap(form),
 			  XtNvertDistance, 30,
@@ -893,7 +925,6 @@ FatbitsEdit(Widget paint)
     XtAddEventHandler(box, StructureNotifyMask, False,
 		      (XtEventHandler) fatbitsResized, (XtPointer) l);
 
-
     fat = XtVaCreateManagedWidget("paint", paintWidgetClass, box,
 				  XtNpaint, paint,
 				  XtNbottom, XtChainBottom,
@@ -908,19 +939,29 @@ FatbitsEdit(Widget paint)
     AddDestroyCallback(shell, (DestroyCallbackFunc) doneCallback, (XtPointer) l);
 
     XtAddCallback(fat, XtNdestroyCallback, FatCursorDestroyCallback, fat);
+    XtAddEventHandler(fat, ExposureMask, False,
+		      (XtEventHandler) fatbitsExposed, (XtPointer) l);
+
     ccpAddStdPopup(fat, NULL);
 
     XtPopup(shell, XtGrabNone);
+    XtVaGetValues(shell, XtNtitle, &ptr, NULL);
+    StoreName(shell, ptr);
 
-    lx -= width / 2;
-    ly -= height / 2;
-    if (lx < 0)
-	lx = 0;
-    if (ly < 0)
-	ly = 0;
+    lx -= width/2;
+    ly -= height/2;
+    if (lx < 0) lx = 0;
+    if (ly < 0) ly = 0;
 
-    x = lx - (PADDING + 2);
-    y = ly - (PADDING + 2);
+    i = PADDING + 2;
+
+    if (l->zoom>0) { 
+        x = lx * l->zoom - i;
+        y = ly * l->zoom - i;
+    } else {
+        x = lx / (-l->zoom) - i;
+        y = ly / (-l->zoom) - i;
+    }
 
     box = XtVaCreateManagedWidget("paintBox", boxWidgetClass, paint,
 				  XtNx, x,
@@ -951,8 +992,6 @@ FatbitsEdit(Widget paint)
 				          XtRString, "fleur", sizeof(Cursor),
 					  NULL);
 
-    XtAddCallback(l->view, XtNsizeChanged, sizeChanged, (XtPointer) l);
-
     XtVaGetValues(l->subpaint, XtNx, &l->spX,
 		  XtNy, &l->spY,
 		  XtNborderWidth, &l->spBW,
@@ -960,16 +999,19 @@ FatbitsEdit(Widget paint)
 
     XtVaSetValues(l->subpaint, XtNzoomX, lx, XtNzoomY, ly, NULL);
     XtVaSetValues(l->view, XtNzoomX, lx, XtNzoomY, ly, NULL);
+    XtAddCallback(l->view, XtNsizeChanged, sizeChanged, (XtPointer) l);
 
     /*
     **	Set the current operator
      */
     StateAddParent(shell, paint);
 
-    XtUnmanageChild(button);
-    XMapWindow(XtDisplay(button), XtWindow(button));
+    XtUnmanageChild(done);
+    XMapWindow(XtDisplay(done), XtWindow(done));
     XtUnmanageChild(box);
     XMapWindow(XtDisplay(box), XtWindow(box));
+    XtUnmanageChild(refresh);
+    XMapWindow(XtDisplay(refresh), XtWindow(refresh));
     XFlush(XtDisplay(box));
     
     for (i=0; i<XtNumber(zoomList); i++) {
