@@ -71,8 +71,6 @@
 #endif
 #endif				/* _Xconst */
 
-#define LOADED 4
-
 typedef struct {
     Widget shell, pane, name, program, cclog;
     int mode;
@@ -86,6 +84,7 @@ extern int file_transparent;
 extern int file_numpages;
 extern int file_force;
 extern int file_bbox;
+extern int file_specified_zoom;
 
 void *
 GetFileNameGetLastId()
@@ -197,25 +196,17 @@ saveRegionFileCallback(Widget paint, XtPointer str, XtPointer func)
 static void 
 saveFileCallback(Widget paint, XtPointer str, XtPointer func)
 {
-    char *cp;
-
     stdSaveCommonCallback(paint, (char *) str, True, (RWwriteFunc) func);
-
-    if ((cp = strrchr(str, '/')) == NULL)
-	cp = str;
-    else
-	cp++;
-
-    XtVaSetValues(GetShell(paint), XtNtitle, str, XtNiconName, cp, NULL);
+    StoreName(GetShell(paint), str);
 }
 
-void 
+void
 StdSaveRegionFile(Widget w, XtPointer paintArg, XtPointer junk)
 {
     Widget paint = (Widget) paintArg;
 
     if (PwRegionGet(paint, NULL, NULL))
-	GetFileName(paint, 1, NULL, saveRegionFileCallback, NULL);
+	GetFileName(paint, BROWSER_SAVE, NULL, saveRegionFileCallback, NULL);
     else
 	Notice(paint, msgText[NO_REGION_SELECTED_PRESENTLY]);
 }
@@ -224,40 +215,33 @@ void
 StdSaveAsFile(Widget w, XtPointer paintArg, XtPointer junk)
 {
     Widget paint = (Widget) paintArg;
-    Boolean bool;
     String name;
     String nm = XtName(GetShell(paint));
 
-    XtVaGetValues(paint, XtNeditable, &bool, NULL);
-    if (!bool) return;
-
     XtVaGetValues(GetShell(paint), XtNtitle, &name, NULL);
 
-    if (strcmp(name, DEFAULT_TITLE) == 0 || strcmp(nm, name) == 0)
+    if (strcmp(name, msgText[DEFAULT_TITLE]) == 0 || strcmp(nm, name) == 0)
 	name = NULL;
 
-    GetFileName(paint, 1, name, saveFileCallback, NULL);
+    GetFileName(paint, BROWSER_SAVE, name, saveFileCallback, NULL);
 }
 
 void 
 StdSaveFile(Widget w, XtPointer paintArg, XtPointer junk)
 {
     Widget paint = (Widget) paintArg;
-    Boolean bool;
     String name = NULL;
     String nm = XtName(GetShell(paint));
     RWwriteFunc f = NULL;
     void *id;
 
-    XtVaGetValues(paint, XtNeditable, &bool, NULL);
-    if (!bool) return;
-
     if (strcmp(nm, "Canvas") == 0) {
 	XtVaGetValues(GetShell(paint), XtNtitle, &name, NULL);
 
-	if (strcmp(name, DEFAULT_TITLE) == 0 || strcmp(nm, name) == 0)
+	if (strcmp(name, msgText[DEFAULT_TITLE]) == 0 || strcmp(nm, name) == 0)
 	    name = NULL;
     }
+
     if (name != NULL) {
 	if ((id = getArgType(paint)) != NULL) {
 	    f = (RWwriteFunc) RWtableGetWriter(id);
@@ -267,10 +251,9 @@ StdSaveFile(Widget w, XtPointer paintArg, XtPointer junk)
 	if (f != NULL) {
 	    stdSaveCommonCallback(paint, name, True, f);
 	    return;
-
 	}
     }
-    GetFileName(paint, 1, name, saveFileCallback, NULL);
+    GetFileName(paint, BROWSER_SAVE, name, saveFileCallback, NULL);
 }
 
 /*
@@ -296,12 +279,11 @@ typedef struct arg_s {
     char dirname[MAX_PATH];
     Widget shell, list, browser, parent, 
            form, title_w, name, vport, 
-           home, root, dot, hidden, cwd_w, info, ok, cancel,
+           home, root, dot, hidden, ongoing, cwd_w, info, ok, cancel,
            refresh, delete, edit, create,
-           dpilabel, dpi, zoomlabel, zoom,
-           alphalabel, alphaleft, alpha, alpharight,
-           pagelabel, page, pagetotal, pageleft, pageLeft, pageright, pageRight,
-           bboxlabel, bboxleft, bbox, bboxright;
+           zoomlabel, zoom, dpilabel, dpi, 
+           bboxlabel, bboxleft, bbox, bboxright,
+           pagelabel, page, pagetotal, pageleft, pageLeft, pageright, pageRight;
     Widget format[NUM_FORMATS+2];
     int first, numformat, oldwidth;
     /*
@@ -399,9 +381,13 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     XtVaGetValues(l->cwd_w, XtNx, &x, NULL);
     if (l->home) {
         XtMoveWidget(l->home, 5, height-84);
-        XtMoveWidget(l->root, 30, height-84);
-        XtMoveWidget(l->dot, 55, height-84);
-        XtMoveWidget(l->hidden, 80, height-84);
+        XtMoveWidget(l->root, 29, height-84);
+        XtMoveWidget(l->dot, 53, height-84);
+        XtMoveWidget(l->hidden, 77, height-84);
+        if (l->ongoing) {
+            XtVaGetValues(l->hidden, XtNwidth, &width1, NULL);
+            XtMoveWidget(l->ongoing, 80+width1, height-84);
+	}
         XtMoveWidget(l->cwd_w, x, height-52);
         XtMoveWidget(l->info, x, height-30);
         return;
@@ -410,7 +396,7 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     XtMoveWidget(l->cwd_w, x, height-158);
     XtMoveWidget(l->info, x, height-134);
 
-    im = &l->bboxright - &l->refresh;
+    im = &l->pageRight - &l->refresh;
     widget = &l->refresh;
     for (i=0; i<=im; i++) {
         XtVaGetValues(widget[i], XtNx, &x, NULL);
@@ -541,7 +527,7 @@ setCWD(arg_t * arg, char *dir)
     Widget sb;
     Boolean state;
 
-    if (arg->browserType == LOADED) {
+    if (arg->browserType == BROWSER_LOADED) {
         emptyList(arg);
         fileCount = Global.numfiles;
         list = (String *) XtCalloc(sizeof(String *), fileCount+2);
@@ -601,7 +587,7 @@ setCWD(arg_t * arg, char *dir)
 	    if (nm[1] == '\0')
 		continue;
 	    if (nm[1] == '.' && nm[2] == '\0') {
-		list[i++] = XtNewString("../ (Go up 1 directory level)");
+	        list[i++] = XtNewString(msgText[GO_UP_ONE_DIRECTORY_LEVEL]);
 		continue;
 	    }
             if (!state) continue;
@@ -652,7 +638,7 @@ setCWD(arg_t * arg, char *dir)
     XawListChange(arg->list, list, i, 0, True);
     XtVaSetValues(arg->info, XtNlabel, infoStr, NULL);
 
-    if (arg->browserType != LOADED || arg->isToRefresh) {
+    if (arg->browserType != BROWSER_LOADED || arg->isToRefresh) {
         XtVaSetValues(arg->name, XtNstring, "", NULL);
         if ((sb = XtNameToWidget(arg->vport, "vertical")) != None) {
 	    float top = 0.0;
@@ -690,9 +676,14 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     char *cp;
     char *nm;
     int zoom = Global.default_zoom, zoom_prev = Global.default_zoom;
+    Boolean ongoing;
     static char buf[20];
     static int np = 0;
 
+    ongoing = False;
+    if (arg->ongoing)
+        XtVaGetValues(arg->ongoing, XtNstate, &ongoing, NULL);
+
     XtVaGetValues(arg->name, XtNstring, &str, NULL);
     if (str == NULL || *str == '\0') {
 	XawListReturnStruct *lr = XawListShowCurrent(arg->list);
@@ -716,10 +707,11 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     }
     if ((cp = strrchr(file, '/')) != NULL) {
 	*cp = '\0';
-	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0) {
-	    if (arg->browserType != LOADED || arg->isToRefresh) 
+	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0 &&
+            !ongoing) {
+	    if (arg->browserType != BROWSER_LOADED || arg->isToRefresh) 
                 setCWD(arg, file);
-            if (arg->browserType != LOADED)
+            if (arg->browserType != BROWSER_LOADED)
                 browserResized(arg->parent, arg, NULL, False);
 	}
 	*cp = '/';
@@ -728,17 +720,15 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     arg->isToRefresh = False;
     arg->oldwidth = 0;
 
-    if (arg->browserType == LOADED) {
-      char *dpistr, *pagestr, *zoomstr, *alphastr, *bboxstr;
+    if (arg->browserType == BROWSER_LOADED) {
+        char *dpistr, *pagestr, *zoomstr, *bboxstr;
         Widget paint = GetNonDirtyCanvas();
         XtVaGetValues(arg->dpi, XtNstring, &dpistr, NULL);
         XtVaGetValues(arg->page, XtNstring, &pagestr, NULL);
         XtVaGetValues(arg->zoom, XtNstring, &zoomstr, NULL);
-        XtVaGetValues(arg->alpha, XtNstring, &alphastr, NULL);
         XtVaGetValues(arg->bbox, XtNstring, &bboxstr, NULL);
         Global.dpi = atof(dpistr);
         Global.numpage = atoi(pagestr);
-        Global.alpha_mode = abs(atoi(alphastr))%4;
         file_bbox = abs(atoi(bboxstr))%4;
         zoom = StrToZoom(zoomstr);
         if (zoom>32) zoom = 32;
@@ -759,8 +749,24 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
             return;
 	} else
             XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(zoom), NULL);
-    } else
-	XtPopdown(GetShell(w));
+    } else {
+	if (ongoing) {
+            Widget paint = GetNonDirtyCanvas();
+            if (paint) {
+	        if (file_specified_zoom)
+		    zoom = file_specified_zoom;
+                else
+	            XtVaGetValues(paint, XtNzoom, &zoom, NULL);
+                Global.default_zoom = zoom;
+	        loadPrescribedFile(paint, file);
+                AddFileToGlobalList(file);
+                setZoomButtonLabel(paint, zoom);
+                Global.default_zoom = zoom_prev;
+                return;
+	    }
+        } else
+            XtPopdown(GetShell(w));
+    }
 
     if (arg->isSimple) {
         if (arg->okFunc != NULL)
@@ -775,7 +781,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
 
     if (arg->type != NULL &&
 	strcmp(nm = RWtableGetId(arg->type), MAGIC_READER) != 0)
-	lastId = (void *) nm;
+        lastId = (void *) nm;
     else
 	lastId = NULL;
 
@@ -788,6 +794,8 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
             AddFileToGlobalList(file);
 	} else {
 	    Image *image = f(file);
+	    if (file_specified_zoom)
+	        Global.default_zoom = file_specified_zoom;
 	    if (image == NULL)
 		Notice(w, msgText[UNABLE_TO_OPEN_INPUT_FILE], file, 
                           RWGetMsg());
@@ -802,21 +810,6 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     }
 }
 
-static void 
-alphaCallback(Widget w, XtPointer argArg, XtPointer junk)
-{
-    arg_t *arg = (arg_t *) argArg;
-    static char value[4];
-    if (!file_transparent) return;
-    if (strstr(XtName(w), "xright"))
-        Global.alpha_mode = (Global.alpha_mode+1)%4;
-    else
-        Global.alpha_mode = (Global.alpha_mode+3)%4;
-    sprintf(value, "%d",  Global.alpha_mode);
-    XtVaSetValues(arg->alpha, XtNstring, value, NULL);
-    okCallback(w, argArg, junk);
-}
-
 static void 
 pageCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
@@ -906,7 +899,7 @@ static void
 refreshCallback(Widget w, XtPointer argArg, XtPointer itemArg)
 {
     arg_t *arg = (arg_t *) argArg;
-    if (Global.numfiles)
+    if (Global.numfiles || w==arg->hidden)
         setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False); 
 }
@@ -1032,7 +1025,7 @@ createCallback(Widget w, XtPointer argArg, XtPointer itemArg)
     *ptr = '\0';
     ++ptr;
     if (strcmp(ptr, "image.c")) return;
-    GetFileName(w, 2, NULL, createLXPCallback, arg);
+    GetFileName(w, BROWSER_SIMPLESAVE, NULL, createLXPCallback, arg);
     free(dir);
 }
 
@@ -1056,7 +1049,7 @@ listCallback(Widget bar, XtPointer argArg, XtPointer itemArg)
             arg->okFunc = GraphicOpenFile;
 	okCallback(bar, argArg, NULL);
     } else {
-        if (arg->browserType == LOADED) {
+        if (arg->browserType == BROWSER_LOADED) {
             static char value[2];
             Global.numpage = 1;
             *value = '1'; value[1] = '\0';
@@ -1150,7 +1143,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 				   NULL);
     arg->list = list;
 
-    if (arg->browserType != LOADED) {
+    if (arg->browserType != BROWSER_LOADED) {
         arg->home = XtVaCreateManagedWidget("home",
 				  commandWidgetClass, form,
 				  XtNlabel, "~",
@@ -1165,7 +1158,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 				  XtNlabel, "/",
                                   XtNfromVert, vport,
                                   XtNfromHoriz, arg->home,
-                                  XtNhorizDistance, 4,
+                                  XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
                                   XtNwidth, 20,
                                   XtNheight, 20,
@@ -1175,7 +1168,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 				  XtNlabel, ".",
                                   XtNfromVert, vport,
                                   XtNfromHoriz, arg->root,
-                                  XtNhorizDistance, 4,
+                                  XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
                                   XtNwidth, 20,
                                   XtNheight, 20,
@@ -1184,10 +1177,22 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 				  toggleWidgetClass, form,
                                   XtNfromVert, vport,
                                   XtNfromHoriz, arg->dot,
-                                  XtNhorizDistance, 4,
+                                  XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
                                   XtNheight, 20,
 				  NULL);
+        if (arg->browserType <= BROWSER_MULTIREAD)
+            arg->ongoing = XtVaCreateManagedWidget("ongoing",
+				  toggleWidgetClass, form,
+                                  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->hidden,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNheight, 20,
+				  XtNstate, (arg->browserType==BROWSER_MULTIREAD),
+				  NULL);
+        else
+	    arg->ongoing = None;
     }
 
     cwd = XtVaCreateManagedWidget("cwd",
@@ -1229,7 +1234,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 
     arg->first = True;
 
-    if (arg->browserType != LOADED) XtSetKeyboardFocus(form, arg->name);
+    if (arg->browserType != BROWSER_LOADED) XtSetKeyboardFocus(form, arg->name);
 
     setCWD(arg, doDirname(arg, NULL));
     return form;
@@ -1251,7 +1256,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
     
     
     shell = XtVisCreatePopupShell("filebrowser",
-				  ((mode==LOADED)?
+				  ((mode==BROWSER_LOADED)?
 				   topLevelShellWidgetClass:
                                    transientShellWidgetClass),
                                   GetToplevel(w),
@@ -1338,7 +1343,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
     arg->isRead = True;
     arg->numformat = k;
 
-    if (arg->browserType == LOADED) {
+    if (arg->browserType == BROWSER_LOADED) {
         arg->refresh = XtVaCreateManagedWidget("refresh",
 				       commandWidgetClass, browser,
 				       XtNborderWidth, 0,
@@ -1374,10 +1379,35 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNhorizDistance, 10,
 				       NULL);
 
+        arg->zoomlabel = XtVaCreateManagedWidget("zoom",
+				       labelWidgetClass, browser,
+				       XtNborderWidth, 0,
+                                       XtNfromVert, arg->refresh,
+				       XtNvertDistance, 10,
+				       XtNhorizDistance, 10,
+				       NULL);
+
+        arg->zoom = XtVaCreateManagedWidget("zoomW",
+				       asciiTextWidgetClass, browser,
+ 				       XtNeditType, XawtextEdit,
+				       XtNwrap, XawtextWrapNever,
+				       XtNresize, XawtextResizeWidth,
+				       XtNtranslations, trans,
+                                       XtNfromVert, arg->refresh,
+				       XtNvertDistance, 10,
+                                       XtNfromHoriz, arg->zoomlabel,
+				       XtNhorizDistance, 10,
+				       XtNwidth, 50,
+				       XtNheight, 20,
+				       NULL);
+        XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(Global.default_zoom),
+                      NULL);
+
         arg->dpilabel = XtVaCreateManagedWidget("dpi",
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
                                        XtNfromVert, arg->refresh,
+                                       XtNfromHoriz, arg->zoom,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
 				       NULL);
@@ -1398,78 +1428,51 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
         sprintf(value, "%g", Global.dpi);
         XtVaSetValues(arg->dpi, XtNstring, value, NULL);
 
-        arg->zoomlabel = XtVaCreateManagedWidget("zoom",
+        arg->bboxlabel = XtVaCreateManagedWidget("BBox",
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
+                                       XtNheight, 20,
                                        XtNfromVert, arg->refresh,
-				       XtNvertDistance, 10,
+				       XtNvertDistance, 7,
                                        XtNfromHoriz, arg->dpi,
 				       XtNhorizDistance, 10,
 				       NULL);
 
-        arg->zoom = XtVaCreateManagedWidget("zoomW",
+        arg->bboxleft = XtVaCreateManagedWidget("xxleft",
+				       commandWidgetClass, browser,
+                                       XtNfromVert, arg->refresh,
+				       XtNvertDistance, 7,
+				       XtNfromHoriz, arg->bboxlabel,
+				       XtNhorizDistance, 3,
+				       XtNheight, 20,
+				       XtNwidth, 18,
+				       NULL);
+
+        arg->bbox = XtVaCreateManagedWidget("bbox",
 				       asciiTextWidgetClass, browser,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
 				       XtNtranslations, trans,
                                        XtNfromVert, arg->refresh,
-				       XtNvertDistance, 10,
-                                       XtNfromHoriz, arg->zoomlabel,
-				       XtNhorizDistance, 10,
-				       XtNwidth, 50,
-				       XtNheight, 20,
+				       XtNvertDistance, 7,
+                                       XtNfromHoriz, arg->bboxleft,
+				       XtNhorizDistance, 3,
+				       XtNwidth, 18,
 				       NULL);
-        XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(Global.default_zoom),
-                      NULL);
+        sprintf(value, "%d", file_bbox);
+        XtVaSetValues(arg->bbox, XtNstring, value, NULL);
 
-        arg->alphalabel = XtVaCreateManagedWidget("alpha",
-				       labelWidgetClass, browser,
-				       XtNborderWidth, 0,
-                                       XtNfromVert, arg->refresh,
-				       XtNvertDistance, 10,
-                                       XtNfromHoriz, arg->zoom,
-				       XtNhorizDistance, 16,
-				       NULL);
-
-        arg->alphaleft = XtVaCreateManagedWidget("xxleft",
+        arg->bboxright = XtVaCreateManagedWidget("xxright",
 				       commandWidgetClass, browser,
                                        XtNfromVert, arg->refresh,
 				       XtNvertDistance, 7,
-				       XtNfromHoriz, arg->alphalabel,
+				       XtNfromHoriz, arg->bbox,
 				       XtNhorizDistance, 3,
 				       XtNheight, 20,
 				       XtNwidth, 18,
 				       NULL);
 
-        arg->alpha = XtVaCreateManagedWidget("alphaW",
-				       asciiTextWidgetClass, browser,
- 				       XtNeditType, XawtextEdit,
-				       XtNwrap, XawtextWrapNever,
-				       XtNresize, XawtextResizeWidth,
-				       XtNtranslations, trans,
-                                       XtNfromVert, arg->refresh,
-				       XtNvertDistance, 10,
-                                       XtNfromHoriz, arg->alphaleft,
-				       XtNhorizDistance, 3,
-				       XtNwidth, 18,
-				       XtNheight, 20,
-				       NULL);
-
-        arg->alpharight = XtVaCreateManagedWidget("xxright",
-				       commandWidgetClass, browser,
-                                       XtNfromVert, arg->refresh,
-				       XtNvertDistance, 7,
-				       XtNfromHoriz, arg->alpha,
-				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
-				       NULL);
-
-        Global.alpha_mode = abs(Global.alpha_mode)%4;
-        sprintf(value, "%d", Global.alpha_mode);
-        XtVaSetValues(arg->alpha, XtNstring, value, NULL);
-
         arg->pagelabel = XtVaCreateManagedWidget("page",
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
@@ -1542,64 +1545,14 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNwidth, 18,
 				       NULL);
 
-        arg->bboxlabel = XtVaCreateManagedWidget("BBox",
-				       labelWidgetClass, browser,
-				       XtNborderWidth, 0,
-                                       XtNheight, 20,
-                                       XtNfromVert, arg->dpi,
-				       XtNvertDistance, 7,
-                                       XtNfromHoriz, arg->pageRight,
-				       XtNhorizDistance, 10,
-				       NULL);
-
-        arg->bboxleft = XtVaCreateManagedWidget("xxleft",
-				       commandWidgetClass, browser,
-                                       XtNfromVert, arg->dpi,
-				       XtNvertDistance, 7,
-				       XtNfromHoriz, arg->bboxlabel,
-				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
-				       NULL);
-
-        arg->bbox = XtVaCreateManagedWidget("bbox",
-				       asciiTextWidgetClass, browser,
- 				       XtNeditType, XawtextEdit,
-				       XtNwrap, XawtextWrapNever,
-				       XtNresize, XawtextResizeWidth,
-				       XtNtranslations, trans,
-                                       XtNfromVert, arg->dpi,
-				       XtNvertDistance, 7,
-                                       XtNfromHoriz, arg->bboxleft,
-				       XtNhorizDistance, 3,
-				       XtNwidth, 18,
-				       NULL);
-        sprintf(value, "%d", file_bbox);
-        XtVaSetValues(arg->bbox, XtNstring, value, NULL);
-
-        arg->bboxright = XtVaCreateManagedWidget("xxright",
-				       commandWidgetClass, browser,
-                                       XtNfromVert, arg->dpi,
-				       XtNvertDistance, 7,
-				       XtNfromHoriz, arg->bbox,
-				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
-				       NULL);
-
         sprintf(value, "%d", Global.numpage);
         XtVaSetValues(arg->page, XtNstring, value, NULL);
 
         XtInstallAccelerators(arg->page, okButton);
         XtInstallAccelerators(arg->dpi, okButton);
         XtInstallAccelerators(arg->zoom, okButton);
-        XtInstallAccelerators(arg->alpha, okButton);
         XtInstallAccelerators(arg->bbox, okButton);
 
-        XtAddCallback(arg->alphaleft, XtNcallback,
-                      alphaCallback, (XtPointer) arg);
-        XtAddCallback(arg->alpharight, XtNcallback,
-                      alphaCallback, (XtPointer) arg);
         XtAddCallback(arg->pageLeft, XtNcallback,
                       pageCallback, (XtPointer) arg);
         XtAddCallback(arg->pageleft, XtNcallback,
@@ -1627,7 +1580,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    if (arg->browserType != LOADED)
+    if (arg->browserType != BROWSER_LOADED)
         XtSetKeyboardFocus(form, arg->name);
     XtInstallAccelerators(arg->name, okButton);
 
@@ -1644,7 +1597,8 @@ buildSaveBrowser(Widget w, arg_t * arg)
     XtTranslations toglt;
     int i, j, k;
     char *name_format;
-    char **list, *rdr = NULL;
+    char **list;
+    /* char *rdr = NULL; */
     Arg args[4];
     int nargs=0;
     
@@ -1681,9 +1635,11 @@ buildSaveBrowser(Widget w, arg_t * arg)
     toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
     list = RWtableGetWriterList();
 
+    /*
     if (RWtableGetWriter(GraphicGetReaderId(w)) != NULL) {
 	rdr = (char *) GraphicGetReaderId(w);
     }
+    */
 
     j = 0;
     k = 0;
@@ -1722,13 +1678,18 @@ buildSaveBrowser(Widget w, arg_t * arg)
 	    arg->type = NULL;
 	    XtVaSetValues(toggle, XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
+	    /*
 	    if (rdr == NULL)
-		XtVaSetValues(toggle, XtNstate, True, NULL);
+	    */
+	        XtVaSetValues(toggle, XtNstate, True, NULL);
 	}
+
+	/*
 	if (rdr != NULL && strcmp(rdr, list[i]) == 0) {
 	    arg->type = RWtableGetEntry(list[i]);
 	    XtVaSetValues(toggle, XtNstate, True, NULL);
 	}
+	*/
 	XtAddCallback(toggle, XtNcallback, fileTypeCallback, (XtPointer) arg);
     }
 
@@ -1830,8 +1791,8 @@ getArgType(Widget w)
 
     for (cur = argList; cur != NULL; cur = cur->next)
 	if (cur->parent == w &&
-	    (cur->browserType == 0 ||
-	     cur->browserType == 1))
+	    (cur->browserType >= BROWSER_READ &&
+	     cur->browserType <= BROWSER_SAVE))
 	    break;
 
     if (cur == NULL)
@@ -1856,27 +1817,26 @@ getArg(Widget w, Boolean type, Boolean * built)
     memset(cur, 0, sizeof(arg_t));
     cur->parent = p;
     cur->browserType = type;
-    /*
-    cur->isLoaded = False;
-    cur->isToRefresh = False;
-    */
 
     switch (type) {
-    case 0:
-      shell = buildOpenBrowser(w, cur, 250, 380, 0);
+    case BROWSER_READ:
+        shell = buildOpenBrowser(w, cur, 250, 380, BROWSER_READ);
 	break;
-    case 1:
+    case BROWSER_MULTIREAD:
+        shell = buildOpenBrowser(w, cur, 250, 380, BROWSER_MULTIREAD);
+        break;
+    case BROWSER_LOADED:
+        shell = buildOpenBrowser(w, cur, 340, 300, BROWSER_LOADED);
+        break;
+    case BROWSER_SAVE:
         shell = buildSaveBrowser(w, cur);
 	break;
-    case 2:
-	shell = buildSimpleBrowser(w, cur, True);
-	break;
-    case 3:
+    case BROWSER_SIMPLEREAD:
 	shell = buildSimpleBrowser(w, cur, False);
 	break;
-    case LOADED:
-        shell = buildOpenBrowser(w, cur, 300, 300, LOADED);
-        break;
+    case BROWSER_SIMPLESAVE:
+	shell = buildSimpleBrowser(w, cur, True);
+	break;
     }
 
     cur->next = argList;	/* Add cur to front of list */
@@ -1899,11 +1859,11 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     Widget shell, *widget;
     int i, j, im;
 
-    if (type == LOADED) {
+    if (type == BROWSER_LOADED) {
         /* if (!Global.numfiles) return; */
         /* Check whether "loaded files" browser is already popped up */
         for (arg = argList; arg != NULL; arg = arg->next)
-	    if (arg->browserType == LOADED) {
+	    if (arg->browserType == BROWSER_LOADED) {
                 XWindowAttributes win_attributes;
                 XGetWindowAttributes(XtDisplay(arg->bbox), 
                                      XtWindow(arg->bbox), &win_attributes);
@@ -1960,9 +1920,11 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     shell = GetShell(arg->name);
 
     XtPopup(shell, XtGrabNone);
+    XtVaGetValues(shell, XtNtitle, &def, NULL);
+    StoreName(shell, def);
 
     if (arg->bbox)
-        im = &arg->bboxright - &arg->browser;
+        im = &arg->pageRight - &arg->browser;
     else
         im = &arg->cancel - &arg->browser;
 
