@@ -767,7 +767,7 @@ buildGradientPixmap(Widget w, OpInfo * info,
     }
 
     pixmap = None;		/* force creation of new pixmap */
-    ImageToPixmapCmap(image, w, &pixmap, cmap, 0);
+    ImageToPixmapCmap(image, w, &pixmap, cmap);
     return pixmap;
 }
 
@@ -795,7 +795,7 @@ buildFractalPixmap(Widget w, OpInfo * info,
 
     XtVaGetValues(w, XtNcolormap, &cmap, NULL);
     pixmap = None;		/* force creation of new pixmap */
-    ImageToPixmapCmap(image, w, &pixmap, cmap, 0);
+    ImageToPixmapCmap(image, w, &pixmap, cmap);
     return pixmap;
 }
 #endif
