@@ -33,10 +33,18 @@
 #include "xaw_incdir/Command.h"
 #include "xaw_incdir/Viewport.h"
 #include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/Toggle.h"
 
 #include "Paint.h"
+#include "menu.h"
+
+#define M_PI_180 M_PI/180
 
 #define XftNameUnparseAlloc FcNameUnparse
+#define DEFAULT_XFT_TEXT "\
+ABCDEFGHIJKLMNOPQRSTUVWXYZ\n\
+abcdefghijklmnopqrstuvwxyz\n\
+0123456789-+*/=<>.,:;?!&~#(){}[]'`\"^@$\\\\\n"
 
 /*
 **  swiped from X11/Xfuncproto.h
@@ -69,6 +77,8 @@
 #define BORDERWIDTH 0
 #endif
 
+extern void  removeTempFile(void);
+
 typedef struct {
     int num_fonts, num_families, num_weights;
     char *font_select;
@@ -77,11 +87,13 @@ typedef struct {
     char **font_family;
     char **weight_list;
     XftFont * xft_font;
+    char * xft_text;
     char * xft_name;
     double xft_size, xft_height, xft_ascent, xft_descent, xft_maxadv;
-    double xft_rot, xft_linespacing;
+    double xft_rotation, xft_inclination, xft_dilation, xft_linespacing;
     Pixmap pixmap;
-    Widget shell, form1, vport0, vport1, vport2, subform, text, paint;
+    Widget shell, form1, form2, subform;
+    Widget vport0, vport1, vport2, sample, paint;
     Widget family, familyLabel;
     Widget weight, weightLabel;
     Widget pointBar, pointSelect, pointSelectLabel;
@@ -89,6 +101,7 @@ typedef struct {
     Widget dilation, dilationLabel;
     Widget linespacing, linespacingLabel;
     Widget inclination, inclinationLabel;
+    Widget editButton, applyButton, doneButton, menubar;
 } arg_t;
 
 static arg_t *theArg = NULL;
@@ -101,12 +114,28 @@ strqsortcmp(char **a, char **b)
     return strcasecmp(*a, *b);
 }
 
+int GetCharLength(char *buf)
+{
+    int j;
+    char c = buf[0];
+    if (c & 0x80) 
+        c = c<<1;
+    else 
+        return 1;
+    j = 1;
+    do {
+        ++j;
+        c = c<<1;
+    } while ((c & 0x80) && j <= 4);
+    return j;
+}
+
 void
 FreeTypeDrawString(Display *dpy, arg_t * arg,
                    XftDraw *draw, XftColor * color,
                    char *buf, int len, int x, int y, double r)
 {
-    int j;
+    int j, charlen;
     double dx=0.5, dy=0.5, a, u, v;
     XGlyphInfo extents = {};
 
@@ -114,28 +143,30 @@ FreeTypeDrawString(Display *dpy, arg_t * arg,
     v = sin(r);
 
     for (j=0; j<len; j++) {
-        XftDrawString8(draw, color, arg->xft_font,
+        charlen = GetCharLength(buf+j);
+        XftDrawStringUtf8(draw, color, arg->xft_font,
 		       x+((int)dx)+ v*arg->xft_ascent, 
                        y+((int)dy)+ u*arg->xft_ascent, 
-                       (XftChar8*)(buf+j), 1);
-        XftTextExtents8(dpy, arg->xft_font,
-			(XftChar8*)(buf+j), 1, 
+                       (XftChar8*)(buf+j), charlen);
+        XftTextExtentsUtf8(dpy, arg->xft_font,
+			(XftChar8*)(buf+j), charlen, 
                         (XGlyphInfo*)&extents);
         a = sqrt(extents.xOff*extents.xOff+extents.yOff*extents.yOff);
         dx += a * u;
         dy -= a * v;
+        j += charlen-1;
     }
 }
 
 static void 
 setSamplePixmap(arg_t * arg)
 {
-    XtVaSetValues(arg->text, XtNbackgroundPixmap, None, NULL);
+    XtVaSetValues(arg->sample, XtNbackgroundPixmap, None, NULL);
     if (arg->pixmap)
-        XtVaSetValues(arg->text, XtNbackgroundPixmap, arg->pixmap, NULL);
+        XtVaSetValues(arg->sample, XtNbackgroundPixmap, arg->pixmap, NULL);
     /* force refresh  */
-    XUnmapWindow(XtDisplay(arg->text), XtWindow(arg->text));
-    XMapWindow(XtDisplay(arg->text), XtWindow(arg->text));
+    XUnmapWindow(XtDisplay(arg->sample), XtWindow(arg->sample));
+    XMapWindow(XtDisplay(arg->sample), XtWindow(arg->sample));
 }
 
 static void
@@ -154,26 +185,69 @@ cleanSamplePixmap(arg_t * arg)
     setSamplePixmap(arg);
 }
 
+static void editCallback(Widget w, XtPointer argArg, XtPointer junk);
+
+static void string_extract(char **str, char **ret)
+{
+    char *ptr;
+    int bs, k;
+ 
+    ptr = *str;
+    k = 0;
+    bs = 0;
+    *ret = (char *)realloc(*ret, 1);
+    (*ret)[0] = '\0';
+
+    while (*ptr) {
+	if (!bs && *ptr == '\\' && *(ptr+1) == '*') {
+            ptr =  strstr(ptr+2, "*\\");
+            if (!ptr)
+                ptr =  strchr(ptr+2, '\n');
+            if (!ptr) break;
+            ptr += 2;
+	}
+        if (bs) {
+	    if (*ptr == '\n') {
+	        ++ptr;
+                bs = 0;
+	    }
+	} else {
+	    if (*ptr == '\n') {
+	        ++ptr;
+                break;
+	    }
+        }
+        if (!bs && *ptr == '\\') {
+	    bs = 1;
+            ++ptr;
+            continue;
+	}
+        bs = 0;
+	(*ret)[k] = *ptr;
+	++k;
+        *ret = (char *)realloc(*ret, k+1);
+        (*ret)[k] = '\0';
+        ++ptr;
+    }
+    *str = ptr;
+}
+
 static void 
 setXftFont(arg_t * arg)
 {
-    char * sample[3] = {
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
-      "abcdefghijlmnopqrstuvwxyz",
-      "0123456789-+*/=<>.,:;?!&~#(){}[]\\'`\"^@$"
-    };
     Display *dpy = XtDisplay(arg->shell);
     XGlyphInfo extents = {};
     Colormap cmap;
     GC gc;
     Dimension width, height;
+    Boolean bool;
 
     XftColor color;
     XftDraw *draw = NULL;
     XRenderColor xre_color;
     char matrix[80];
-    char *ptr;
-    double r=0.0, i=0.0, d=1.0, ls=1.0, f = M_PI/180, hd;
+    char *ptr, *str;
+    double r=0.0, i=0.0, d=1.0, ls=1.0, hd;
     double mat[4];
     int j, x, y, w, x0, x1, x2, y0, y1, y2;
 
@@ -195,24 +269,26 @@ setXftFont(arg_t * arg)
     x = strlen(matrix);
 
     XtVaGetValues(arg->rotation, XtNstring, &ptr, NULL);
-    if (ptr) r = atof(ptr)*f;
-    arg->xft_rot = r;
+    if (ptr) r = atof(ptr) * M_PI_180;
+    arg->xft_rotation = r;
 
     XtVaGetValues(arg->dilation, XtNstring, &ptr, NULL);
     if (ptr) d = atof(ptr);
+    arg->xft_dilation = d;
 
     XtVaGetValues(arg->linespacing, XtNstring, &ptr, NULL);
     if (ptr) ls = atof(ptr);
     arg->xft_linespacing = ls;
 
     XtVaGetValues(arg->inclination, XtNstring, &ptr, NULL);
-    if (ptr) i = atof(ptr)*d;
+    if (ptr)
+        i = arg->xft_inclination = atof(ptr);
 
     if (r!=0.0 || d!= 1.0 || i!=0.0) {
         mat[0] = cos(r);
         mat[2] = sin(r);
-        mat[1] = i*mat[0]-d*mat[2];
-        mat[3] = i*mat[2]+d*mat[0];
+        mat[1] = d*(i*mat[0]-mat[2]);
+        mat[3] = d*(i*mat[2]+mat[0]);
         sprintf(matrix, ":matrix=%g %g %g %g",
 		mat[0], mat[1], mat[2], mat[3]);
     } else {
@@ -267,6 +343,12 @@ setXftFont(arg_t * arg)
         return;
     }
 
+    XtVaGetValues(arg->editButton, XtNstate, &bool, NULL);
+    if (bool) {
+        XtVaSetValues(arg->editButton, XtNstate, False, NULL);
+        editCallback(arg->editButton, arg, NULL);
+    }
+
     w = arg->xft_maxadv;
     hd = arg->xft_height*ls;
 
@@ -276,9 +358,14 @@ setXftFont(arg_t * arg)
     if (x<=x1) x1 = x; if (x>=x2) x2 = x;
     if (y<=y1) y1 = y; if (y>=y2) y2 = y;
 
-    for (j=0; j<=2; j++) {    
+    j = 0;
+
+    ptr = arg->xft_text;
+    str = NULL;
+    while (ptr && *ptr) {
+        string_extract(&ptr, &str);
         XftTextExtents8(dpy, arg->xft_font,
-		        (XftChar8*)sample[j], strlen(sample[j]), 
+		        (XftChar8*)str, strlen(str), 
                         (XGlyphInfo*)&extents);
         x = extents.xOff + (int)(0.5+(j-0.2)*hd*mat[2]);
         y = extents.yOff + (int)(0.5+(j-0.2)*hd*mat[0]);
@@ -288,15 +375,16 @@ setXftFont(arg_t * arg)
         y = extents.yOff + (int)(0.5+(j+1.2)*hd*mat[0]);
 	if (x<=x1) x1 = x; if (x>=x2) x2 = x;
 	if (y<=y1) y1 = y; if (y>=y2) y2 = y;
+        ++j;
     }
 
     if ((j=x2-x1+6+w/4)>= width) width = j;
     if ((j=y2-y1+6+((int)arg->xft_height*0.2501))>= height) height = j;
 
-    XtResizeWidget(arg->text, width, height, 0);
-    XtVaSetValues(arg->text, XtNwidth, width, XtNheight, height, NULL);
+    XtResizeWidget(arg->sample, width, height, 0);
+    XtVaSetValues(arg->sample, XtNwidth, width, XtNheight, height, NULL);
     if (arg->pixmap) XFreePixmap(dpy, arg->pixmap);
-    arg->pixmap = XCreatePixmap(dpy, XtWindow(arg->text),
+    arg->pixmap = XCreatePixmap(dpy, XtWindow(arg->sample),
                    width, height,    
                    DefaultDepthOfScreen(XtScreen(arg->shell)));
     gc = XCreateGC(dpy, arg->pixmap, 0, NULL);
@@ -316,12 +404,17 @@ setXftFont(arg_t * arg)
     if (x1<0) x0 += -x1;
     if (y1<0) y0 += -y1;
 
-    for (j=0; j<=2; j++) {
+    j = 0;
+
+    ptr = arg->xft_text;
+    while (ptr && *ptr) {
+        string_extract(&ptr, &str);
         x = x0 + (int)(0.5+j*hd*mat[2]);
         y = y0 + (int)(0.5+j*hd*mat[0]);     
-        FreeTypeDrawString(dpy, arg, draw, &color, 
-                           sample[j], strlen(sample[j]), x, y, r);
+        FreeTypeDrawString(dpy, arg, draw, &color, str, strlen(str), x, y, r);
+        ++j;
     }
+    free(str);
 
     setSamplePixmap(arg);
     XFreeGC(dpy, gc);
@@ -398,6 +491,16 @@ applyCallback(Widget w, XtPointer argArg, XtPointer junk)
     Display *dpy = XtDisplay(w);
     arg_t *arg = (arg_t *) argArg;
     XftFont * font;
+    Boolean bool;
+    char *str;
+
+    XtVaGetValues(arg->editButton, XtNstate, &bool, NULL);
+    if (bool)
+        XtVaGetValues(arg->sample, XtNstring, &str, NULL);
+    else
+        str = arg->xft_text;
+    if (Global.xft_text) free(Global.xft_text);
+    Global.xft_text = strdup(str);
 
     setXftFont(arg);
     if (!arg->xft_font) return;
@@ -419,7 +522,9 @@ applyCallback(Widget w, XtPointer argArg, XtPointer junk)
     Global.xft_ascent = arg->xft_ascent;
     Global.xft_descent = arg->xft_descent;
     Global.xft_maxadv = arg->xft_maxadv;
-    Global.xft_rot = arg->xft_rot;
+    Global.xft_rotation = arg->xft_rotation;
+    Global.xft_dilation = arg->xft_dilation;
+    Global.xft_inclination = arg->xft_inclination;
     Global.xft_linespacing = arg->xft_linespacing;
     GraphicAll((GraphicAllProc) applySetCallback, NULL);
 }
@@ -428,16 +533,10 @@ static void
 closeCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     arg_t *arg = (arg_t *) argArg;
+    PopdownMenusGlobal();
     XtPopdown(arg->shell);
 }
 
-static void 
-clickCallback(Widget w, XtPointer argArg, XEvent * event, Boolean * flg)
-{
-    arg_t *arg = (arg_t *) argArg;
-    XtSetKeyboardFocus(arg->shell, w);
-}
-
 static void 
 unselectCallback(Widget w, XtPointer argArg, XEvent * event, Boolean * flg)
 {
@@ -543,7 +642,8 @@ void
 fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
 {
     Display *dpy = XtDisplay(w);
-    Dimension width, height, height1, height2;
+    Dimension width, height, height1, height2, width2;
+    Position x, y;
     char *val;
     double m = 18.0;
 
@@ -580,26 +680,36 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
 
     XtResizeWidget(arg->vport2, width-8, height2, BORDERWIDTH);
     XtVaSetValues(arg->vport2, XtNwidth, width-8, NULL);
-    XtResizeWidget(arg->text, width-8, height2, 0);
+    XtResizeWidget(arg->sample, width-8, height2, 0);
+    XtVaGetValues(arg->editButton, XtNwidth, &width2, NULL);
+    XtResizeWidget(arg->editButton, width2, 22, 0);
 
     XtResizeWidget(arg->subform, width, 60, 0);
     XtMoveWidget(arg->pointSelectLabel, 4, 8);
+    XtVaGetValues(arg->form2, XtNheight, &height2, NULL);
+    y = height2-25;
+    XtMoveWidget(arg->editButton, 4, y);
+    XtVaGetValues(arg->applyButton, XtNx, &x, NULL);
+    XtMoveWidget(arg->applyButton, x, y);
+    XtVaGetValues(arg->menubar, XtNx, &x, NULL);
+    XtMoveWidget(arg->menubar, x, y-4);
+    XtMoveWidget(arg->doneButton, 304, y);
 
     XtResizeWidget(arg->pointBar, 350, 20, 0);
     XtMoveWidget(arg->pointBar, 150, 8);
     XtMoveWidget(arg->pointSelect, 506, 8);
 
     XtMoveWidget(arg->rotationLabel, 4, 38);
-    XtMoveWidget(arg->rotation, 80, 38);
+    XtMoveWidget(arg->rotation, 90, 38);
 
-    XtMoveWidget(arg->inclinationLabel, 159, 38);
-    XtMoveWidget(arg->inclination, 235, 38);
+    XtMoveWidget(arg->inclinationLabel, 164, 38);
+    XtMoveWidget(arg->inclination, 250, 38);
 
-    XtMoveWidget(arg->dilationLabel, 314, 38);
-    XtMoveWidget(arg->dilation, 390, 38);
+    XtMoveWidget(arg->dilationLabel, 324, 38);
+    XtMoveWidget(arg->dilation, 410, 38);
 
-    XtMoveWidget(arg->linespacingLabel, 469, 38);
-    XtMoveWidget(arg->linespacing, 545, 38);
+    XtMoveWidget(arg->linespacingLabel, 484, 38);
+    XtMoveWidget(arg->linespacing, 570, 38);
 
     XtMoveWidget(arg->vport2, 4, 76);
 
@@ -608,13 +718,15 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
     if (m<=2.0001) m = 2.0001;
     if (30*m>=width) width = (int) (30*m);
     if (5*m>=height2-10) height2 = (int) (5*m+10);
-    XtVaSetValues(arg->text, XtNwidth, width, XtNheight, height2, NULL);
+    XtVaSetValues(arg->sample, XtNwidth, width, XtNheight, height2, NULL);
     if (!arg->pixmap) {
-        arg->pixmap = XCreatePixmap(dpy, XtWindow(arg->text),
+        arg->pixmap = XCreatePixmap(dpy, XtWindow(arg->sample),
                            width, height2,    
                            DefaultDepthOfScreen(XtScreen(arg->shell)));
         cleanSamplePixmap(arg);
     }
+
+
     if (!XPending(XtDisplay(w)))
         setXftFont(arg);
 }
@@ -637,6 +749,343 @@ inputDataAction(Widget w, XEvent * event, String * prms, Cardinal * nprms)
     fontboxResized(w, theArg, (XConfigureEvent *)event, NULL);
 }
 
+static PaintMenuItem fileMenu[] =
+{
+#define FILE_LOAD 0
+    MI_SIMPLE("load"),
+#define FILE_SAVE 1
+    MI_SIMPLE("save"),
+#define FILE_SAVEAS 2
+    MI_SIMPLE("saveas"),
+#define FILE_EDIT 3
+    MI_SIMPLE("editor"),
+};
+
+static PaintMenuItem editMenu[] =
+{
+#define EDIT_DEFAULT 0
+    MI_SIMPLE("default"),
+#define EDIT_ERASE 1
+    MI_SIMPLE("erase"),
+                /* 2 */
+    MI_SEPARATOR(),  
+#define EDIT_FONT 3
+    MI_SIMPLE("\\* font: ?*\\"),
+#define EDIT_COLOR 4
+    MI_SIMPLE("\\* color: ?*\\"),
+#define EDIT_ROTATION 5
+    MI_SIMPLE("\\* rotation: ?*\\"),
+#define EDIT_PLUS_ROTATION 6
+    MI_SIMPLE("\\* +rotation: ?*\\"),
+#define EDIT_INCLINATION 7
+    MI_SIMPLE("\\* inclination: ?*\\"),
+#define EDIT_DILATION 8
+    MI_SIMPLE("\\* dilation: ?*\\"),
+#define EDIT_LINESPACING 9
+    MI_SIMPLE("\\* linespacing: ?*\\"),
+#define EDIT_X 10
+    MI_SIMPLE("\\* x: ?*\\"),
+#define EDIT_Y 11
+    MI_SIMPLE("\\* y: ?*\\"),
+#define EDIT_PLUS_X 12
+    MI_SIMPLE("\\* +x: ?*\\"),
+#define EDIT_PLUS_Y 13
+    MI_SIMPLE("\\* +y: ?*\\"),
+#define EDIT_SX 14
+    MI_SIMPLE("\\* sx: ?*\\"),
+#define EDIT_SY 15
+    MI_SIMPLE("\\* sy: ?*\\"),
+#define EDIT_PLUS_SX 16
+    MI_SIMPLE("\\* +sx: ?*\\"),
+#define EDIT_PLUS_SY 17
+    MI_SIMPLE("\\* +sy: ?*\\"),
+#define EDIT_NOOP 18
+};
+
+static PaintMenuBar textMenuBar[] =
+{
+    {None, "text_file", XtNumber(fileMenu), fileMenu},
+    {None, "text_sample", XtNumber(editMenu), editMenu}
+};
+
+static void
+popupHandler(Widget w, arg_t * argArg, XEvent * event, Boolean * flag)
+{
+    if (Global.popped_up) {
+        if (event->type == ButtonRelease || event->type == ButtonPress)
+            PopdownMenusGlobal();
+        event->type = None;       
+    }
+}
+
+static void 
+loadFileCallbackOK(Widget w, XtPointer argArg, char *file)
+{
+   arg_t *arg = (arg_t *) argArg;
+   FILE *fd;
+   static char *str;
+   char buf[2048];
+   int l;
+
+   if (!file || !*file) return;
+   fd = fopen(file, "r");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], file);
+      return;
+   }
+   /* XtVaSetValues(arg->textname, XtNlabel, file, NULL); */
+   str = xmalloc(2);
+   *str = '\0';
+   l = 0;
+   while (fgets(buf, 2040, fd)) {
+      l += strlen(buf);
+      str = realloc(str, l+2);
+      strcat(str, buf);
+   }
+   fclose(fd);
+   XtVaSetValues(arg->sample, XtNstring, str, NULL);
+}
+
+static void
+loadFileCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   GetFileName(GetShell(w), BROWSER_SIMPLEREAD, 
+               buf, (XtCallbackProc) loadFileCallbackOK, argArg);
+}
+
+static void 
+saveFileCallbackOK(Widget w, XtPointer argArg, char *file)
+{
+   arg_t *arg = (arg_t *) argArg;
+   FILE *fd;
+   char *ptr, *str;
+ 
+   if (!file || !*file) return;
+   fd = fopen(file, "w");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], file);
+      return;
+   }
+
+   /* XtVaSetValues(arg->textname, XtNlabel, file, NULL); */
+   XtVaGetValues(arg->sample, XtNstring, &str, NULL);
+  
+   if (!str || !*str) {
+      fclose(fd);
+      return;
+   }
+   ptr = str;
+   while (*ptr) {
+      fputc(*ptr, fd);
+      ++ptr;
+   }
+   fclose(fd);
+}
+
+static void
+saveFileCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   GetFileName(GetShell(w), BROWSER_SIMPLESAVE, 
+               buf, (XtCallbackProc) saveFileCallbackOK, argArg);
+}
+
+static void 
+defaultTextCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+    arg_t *arg = (arg_t *) argArg;
+    XtVaSetValues(arg->sample, XtNstring, DEFAULT_XFT_TEXT, NULL);
+}
+
+static void 
+eraseTextCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+    arg_t *arg = (arg_t *) argArg;
+    XtVaSetValues(arg->sample, XtNstring, "", NULL);
+}
+
+static void 
+insertTextCallback(Widget w, XtPointer argArg, XtPointer ptr)
+{
+    arg_t *arg = (arg_t *) argArg;
+    char buf[256];
+    int j;
+    XawTextPosition p;
+    XawTextBlock b;
+
+    for (j=EDIT_FONT; j<=EDIT_NOOP; j++)
+        if (editMenu[j].widget == w) break;
+    
+    switch(j) {
+        case EDIT_FONT:
+             sprintf(buf, "\\*font:%s*\\", 
+		     (arg->xft_name)? arg->xft_name:"Liberation-18");
+             break;
+        case EDIT_COLOR:
+             strcpy(buf, "\\*color:#2367ab*\\");
+             break;
+        case EDIT_ROTATION:
+	     sprintf(buf, "\\*rotation:%g*\\", arg->xft_rotation);
+             break;
+        case EDIT_PLUS_ROTATION:
+             sprintf(buf, "\\*+rotation:%g*\\", arg->xft_rotation);
+             break;
+        case EDIT_INCLINATION:
+             sprintf(buf, "\\*inclination:%g*\\", arg->xft_inclination);
+             break;
+        case EDIT_DILATION:
+             sprintf(buf, "\\*dilation:%g*\\", arg->xft_dilation);
+             break;
+        case EDIT_LINESPACING:
+             sprintf(buf, "\\*linespacing:%g*\\", arg->xft_linespacing);
+             break;
+        case EDIT_X:
+             strcpy(buf, "\\*x:50*\\");
+             break;
+        case EDIT_Y:
+             strcpy(buf, "\\*y:80*\\");
+             break;
+        case EDIT_PLUS_X:
+             strcpy(buf, "\\*+x:50*\\");
+             break;
+        case EDIT_PLUS_Y:
+             strcpy(buf, "\\*+y:80*\\");
+             break;
+        case EDIT_SX:
+             strcpy(buf, "\\*sx:50*\\");
+             break;
+        case EDIT_SY:
+             strcpy(buf, "\\*sy:80*\\");
+             break;
+        case EDIT_PLUS_SX:
+             strcpy(buf, "\\*+sx:50*\\");
+             break;
+        case EDIT_PLUS_SY:
+             strcpy(buf, "\\*+sy:80*\\");
+             break;
+        default:
+	     return;
+    }
+    p = XawTextGetInsertionPoint(arg->sample);
+    b.firstPos = 0;
+    b.length = strlen(buf);
+    b.ptr = buf;
+    b.format = 0;
+    XawTextReplace(arg->sample, p, p, &b);
+}
+
+static void
+saveasFileCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   GetFileName(GetShell(w), BROWSER_SIMPLESAVE, 
+               buf, (XtCallbackProc) saveFileCallbackOK, argArg);
+}
+
+static void
+externCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+   arg_t *arg = (arg_t *) argArg;
+   FILE *fd;
+   int l;
+   char *tmp, *ptr, *str;
+   char name[256];
+   char buf[2048];
+
+   /* write buffer to file */
+   PopdownMenusGlobal();
+   fd = openTempFile(&tmp);
+   fclose(fd);
+   sprintf(name, "%s.c", tmp);
+   removeTempFile();
+   fd = fopen(name, "w");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], name);
+      return;
+   }
+   XtVaGetValues(arg->sample, XtNstring, &str, NULL);
+   if (str && *str) {
+      ptr = str;
+      while (*ptr) {
+         fputc(*ptr, fd);
+         ++ptr;
+      }
+   }
+   fclose(fd);
+
+   /* call external editor */
+   sprintf(buf, "%s %s", EDITOR, name);
+   system(buf);
+
+   /* Now copy edited file into buffer */
+   fd = fopen(name, "r");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], name);
+      return;
+   }
+
+   str = xmalloc(2);
+   *str = '\0';
+   l = 0;
+   while (fgets(buf, 2040, fd)) {
+      l += strlen(buf);
+      str = realloc(str, l+2);
+      strcat(str, buf);
+   }
+   fclose(fd);
+   unlink(name);
+   XtVaSetValues(arg->sample, XtNstring, str, NULL);
+}
+
+static void
+editCallback(Widget w, XtPointer argArg, XtPointer junk)
+{
+    arg_t *arg = (arg_t *) argArg;
+    Boolean bool;
+    Dimension width;
+    char *str;
+
+    XtVaGetValues(w, XtNstate, &bool, XtNwidth, &width, NULL);
+    XtResizeWidget(arg->editButton, width, 22, 0);
+
+    if (bool) {
+        cleanSamplePixmap(arg);
+        XtVaSetValues(arg->sample,
+                      XtNeditType, XawtextEdit,
+                      XtNresizable, True,
+                      XtNwidth, 620,
+                      XtNheight, 300,
+	              XtNdisplayCaret, True,
+		      XtNstring, arg->xft_text, NULL);
+        free(arg->xft_text);
+        arg->xft_text = NULL;
+        XMapWindow(XtDisplay(arg->menubar), XtWindow(arg->menubar));
+    } else {
+        cleanSamplePixmap(arg);
+        XtVaGetValues(arg->sample, XtNstring, &str, NULL);
+        if (str) {
+            if (arg->xft_text) free(arg->xft_text);
+            arg->xft_text = strdup(str);
+	}
+        XtVaSetValues(arg->sample,
+                      XtNeditType, XawtextEdit,
+                      XtNresizable, True,
+                      XtNwidth, 620,
+                      XtNheight, 300,
+	              XtNdisplayCaret, False,
+		      XtNstring, "", NULL);
+        XUnmapWindow(XtDisplay(arg->menubar), XtWindow(arg->menubar));
+        setXftFont(arg);
+    }
+}
+
 void 
 FontSelect(Widget w, Widget paint)
 {
@@ -649,9 +1098,10 @@ FontSelect(Widget w, Widget paint)
     static XtActionsRec dataAct =
       {"input-data-ok", (XtActionProc) inputDataAction};
     static Widget shell = None;
+    static arg_t *arg;
+
     Widget pane, form1, form2;
-    Widget applyButton, doneButton, label;
-    arg_t *arg = XtNew(arg_t);
+    Widget editButton, applyButton, doneButton, label;
     char *ptr1, *ptr2;
     int j, k, l, lmax;
 
@@ -659,18 +1109,18 @@ FontSelect(Widget w, Widget paint)
 
     if (shell != None) {
 	XtPopup(shell, XtGrabNone);
-	XMapRaised(XtDisplay(shell), XtWindow(shell));
+	RaiseWindow(XtDisplay(shell), XtWindow(shell));
         XtSetMinSizeHints(shell, 360, 480);
 	return;
     }
 
     StateSetBusyWatch(True);
+    arg = XtNew(arg_t);
+    memset(arg, 0, sizeof(arg_t));
 
     /*
     **	Init the world
      */
-
-    XftInitFtLibrary();
     fs = XftListFonts(dpy, DefaultScreen(dpy), 0,
                       XFT_FAMILY, XFT_FOUNDRY, XFT_STYLE, (char *)0);
     arg->font_desc = (char **) xmalloc(fs->nfont * sizeof(char *));
@@ -706,7 +1156,6 @@ FontSelect(Widget w, Widget paint)
           (int (*)(_Xconst void *, _Xconst void *)) strqsortcmp);
 
     k = 0;
-    arg->num_fonts = 0;
     arg->font_family = (char **) xmalloc(fs->nfont * sizeof(char *));
 
     for (j = 0; j<fs->nfont; j++) {
@@ -738,6 +1187,8 @@ FontSelect(Widget w, Widget paint)
     arg->num_families = k;
     XftFontSetDestroy(fs);
 
+    Global.xft_text = strdup(DEFAULT_XFT_TEXT);
+
     /*
     **	Init the widgets
      */
@@ -797,7 +1248,7 @@ FontSelect(Widget w, Widget paint)
 					  XtNnumberStrings, 0,
 					  NULL);
 
-    arg->weightLabel = XtVaCreateManagedWidget("weightLabel",
+    arg->weightLabel = XtVaCreateWidget("weightLabel",
 				    labelWidgetClass, form1,
 				    XtNborderWidth, 0,
 				    XtNfromHoriz, arg->vport0,
@@ -834,6 +1285,7 @@ FontSelect(Widget w, Widget paint)
 				   formWidgetClass, pane,
 				   XtNborderWidth, 0,
 				   NULL);
+    arg->form2 = form2;
 
     arg->subform = XtVaCreateManagedWidget("subForm",
 				    formWidgetClass, form2,
@@ -1012,39 +1464,73 @@ FontSelect(Widget w, Widget paint)
 				    XtNbottom, XtChainBottom,
 				    NULL);
 
-    arg->text = XtVaCreateManagedWidget("text",
-					asciiTextWidgetClass, arg->vport2,
-					XtNallowResize, True,
-                                     	XtNstring, "",
-	                                XtNdisplayCaret, False,
-					NULL);
+    arg->sample = XtVaCreateManagedWidget("sample",
+					  asciiTextWidgetClass, arg->vport2,
+					  XtNallowResize, True,
+                                          XtNeditType, XawtextEdit,
+                                          XtNresizable, True,
+                                          XtNwidth, 620,
+                                          XtNheight, 300,
+	                                  XtNdisplayCaret, True,
+		                          XtNstring, Global.xft_text, NULL);
+
+    editButton = XtVaCreateManagedWidget("edittoggle",
+					 toggleWidgetClass, form2,
+                                         XtNstate, True,
+					 XtNfromVert, arg->vport2,
+                                         XtNvertDistance, 3,
+					 XtNtop, XtChainBottom,
+					 XtNbottom, XtChainBottom,
+					 XtNright, XtChainLeft,
+					 XtNleft, XtChainLeft,
+					 NULL);
+    arg->editButton = editButton;
 
     applyButton = XtVaCreateManagedWidget("apply",
 					  commandWidgetClass, form2,
 					  XtNfromVert, arg->vport2,
+                                          XtNheight, 22,
+                                          XtNvertDistance, 3,
+					  XtNfromHoriz, editButton,
 					  XtNtop, XtChainBottom,
 					  XtNbottom, XtChainBottom,
 					  XtNright, XtChainLeft,
 					  XtNleft, XtChainLeft,
 					  NULL);
+    arg->applyButton = applyButton;
 
     doneButton = XtVaCreateManagedWidget("done",
 					 commandWidgetClass, form2,
 					 XtNfromVert, arg->vport2,
+                                         XtNvertDistance, 3,
 					 XtNfromHoriz, applyButton,
 					 XtNtop, XtChainBottom,
 					 XtNbottom, XtChainBottom,
-					 XtNleft, XtChainLeft,
 					 XtNright, XtChainLeft,
 					 XtNleft, XtChainLeft,
 					 NULL);
+    arg->doneButton = doneButton;
+
+    arg->menubar = MenuBarCreate(form2, XtNumber(textMenuBar), textMenuBar);
+    XtVaSetValues(arg->menubar, XtNfromVert, arg->vport2,
+		                XtNfromHoriz, applyButton,
+		                XtNvertDistance, 3,
+		                XtNhorizDistance, 50,
+				XtNtop, XtChainBottom,
+				XtNbottom, XtChainBottom,
+			        XtNright, XtChainLeft,
+				XtNleft, XtChainLeft,
+                                NULL);
 
     arg->paint = paint;
     arg->font_select = NULL;
     arg->xft_name = NULL;
     arg->xft_font = NULL;
+    arg->xft_text = NULL;
     arg->weight_select = NULL;
     arg->pixmap = None;
+    arg->xft_dilation = 1.0;
+    arg->xft_linespacing = 1.0;
 
     arg->weight_list = (char **)xmalloc(sizeof(char*));
     arg->num_weights = 1;
@@ -1055,33 +1541,39 @@ FontSelect(Widget w, Widget paint)
     XawScrollbarSetThumb(arg->pointBar, 0.2125, -1.0);
 
     XtPopup(arg->shell, XtGrabNone);
-    XFlush(XtDisplay(arg->shell));
+    XFlush(dpy);
     XtSetMinSizeHints(arg->shell, 360, 480);
     StateSetBusyWatch(False);
+    XtVaGetValues(arg->shell, XtNtitle, &ptr1, NULL);
+    StoreName(arg->shell, ptr1);
 
+    XtUnmanageChild(arg->menubar);
     XtUnmanageChild(arg->familyLabel);
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->familyLabel));
+    XMapWindow(dpy, XtWindow(arg->familyLabel));
     XtUnmanageChild(arg->weightLabel);
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->weightLabel));
+    XMapWindow(dpy, XtWindow(arg->weightLabel));
 
     XtUnmanageChild(XtParent(arg->pointSelect));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->pointSelect)));
+    XMapWindow(dpy, XtWindow(XtParent(arg->pointSelect)));
+    XtUnmanageChild(arg->applyButton);
 
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->pointSelectLabel));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->pointBar));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->pointSelect));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->rotationLabel));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->rotation));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->inclinationLabel));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->inclination));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->dilationLabel));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->dilation));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->linespacingLabel));
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->linespacing));
+    XMapWindow(dpy, XtWindow(arg->pointSelectLabel));
+    XMapWindow(dpy, XtWindow(arg->pointBar));
+    XMapWindow(dpy, XtWindow(arg->pointSelect));
+    XMapWindow(dpy, XtWindow(arg->rotationLabel));
+    XMapWindow(dpy, XtWindow(arg->rotation));
+    XMapWindow(dpy, XtWindow(arg->inclinationLabel));
+    XMapWindow(dpy, XtWindow(arg->inclination));
+    XMapWindow(dpy, XtWindow(arg->dilationLabel));
+    XMapWindow(dpy, XtWindow(arg->dilation));
+    XMapWindow(dpy, XtWindow(arg->linespacingLabel));
+    XMapWindow(dpy, XtWindow(arg->linespacing));
+    XMapWindow(dpy, XtWindow(arg->applyButton));
+    XMapWindow(dpy, XtWindow(arg->menubar));
 
     XtUnmanageChild(arg->vport2);
-    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->vport2));
-    XFlush(XtDisplay(arg->shell));
+    XMapWindow(dpy, XtWindow(arg->vport2));
+    XFlush(dpy);
 
     XtAddCallback(arg->family, XtNcallback, listCallback, (XtPointer) arg);
     XtAddCallback(arg->weight, XtNcallback, listCallback, (XtPointer) arg);
@@ -1090,7 +1582,7 @@ FontSelect(Widget w, Widget paint)
 		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
     XtAddEventHandler(arg->weight, ButtonPressMask, False,
 		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
-    XtAddEventHandler(arg->text, ButtonPressMask, False,
+    XtAddEventHandler(arg->sample, ButtonPressMask, False,
 		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
 
     XtInsertRawEventHandler(arg->family, ButtonPressMask, False,
@@ -1105,20 +1597,53 @@ FontSelect(Widget w, Widget paint)
     XtAddCallback(arg->pointBar, XtNscrollProc,
 		  (XtCallbackProc) scrollCallback, (XtPointer) arg);
 
-    XtAddEventHandler(arg->pointSelect, ButtonPressMask, False,
-                      (XtEventHandler) clickCallback, (XtPointer) arg);
-    XtAddEventHandler(arg->rotation, ButtonPressMask, False,
-                      (XtEventHandler) clickCallback, (XtPointer) arg);
-    XtAddEventHandler(arg->dilation, ButtonPressMask, False,
-                      (XtEventHandler) clickCallback, (XtPointer) arg);
-    XtAddEventHandler(arg->inclination, ButtonPressMask, False,
-                      (XtEventHandler) clickCallback, (XtPointer) arg);
-    XtAddEventHandler(arg->linespacing, ButtonPressMask, False,
-                      (XtEventHandler) clickCallback, (XtPointer) arg);
+    XtAddEventHandler(arg->pointSelect, 
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
+    XtAddEventHandler(arg->rotation,
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
+    XtAddEventHandler(arg->dilation,
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
+    XtAddEventHandler(arg->inclination,
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
+    XtAddEventHandler(arg->linespacing,
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
 
+    XtAddEventHandler(arg->sample, 
+                      ButtonPressMask|ButtonReleaseMask, False,
+                      (XtEventHandler) clickFocusCallback, (XtPointer) arg->shell);
+    XtInsertRawEventHandler(arg->sample, 
+			    ButtonPressMask | ButtonReleaseMask,
+                            False, (XtEventHandler) popupHandler, arg, 
+                            XtListHead);
+
+    XtAddCallback(editButton, XtNcallback, editCallback, (XtPointer) arg);
     XtAddCallback(applyButton, XtNcallback, applyCallback, (XtPointer) arg);
     XtAddCallback(doneButton, XtNcallback, closeCallback, (XtPointer) arg);
     AddDestroyCallback(arg->shell, (DestroyCallbackFunc) closeCallback, arg);
+
+    XtAddCallback(fileMenu[FILE_LOAD].widget, XtNcallback, 
+		     (XtCallbackProc) loadFileCallback, (XtPointer) arg);
+    XtAddCallback(fileMenu[FILE_SAVE].widget, XtNcallback, 
+		     (XtCallbackProc) saveFileCallback, (XtPointer) arg);
+    XtAddCallback(fileMenu[FILE_SAVEAS].widget, XtNcallback, 
+		     (XtCallbackProc) saveasFileCallback, (XtPointer) arg);
+    XtAddCallback(fileMenu[FILE_EDIT].widget, XtNcallback, 
+		     (XtCallbackProc) externCallback, (XtPointer) arg);
+
+    XtAddCallback(editMenu[EDIT_DEFAULT].widget, XtNcallback, 
+		     (XtCallbackProc) defaultTextCallback, (XtPointer) arg);
+    XtAddCallback(editMenu[EDIT_ERASE].widget, XtNcallback, 
+		     (XtCallbackProc) eraseTextCallback, (XtPointer) arg);
+
+    for (j=EDIT_FONT; j<=EDIT_PLUS_Y; j++)
+    XtAddCallback(editMenu[j].widget, XtNcallback, 
+		     (XtCallbackProc) insertTextCallback, (XtPointer) arg);
+
     XtAddEventHandler(arg->shell, StructureNotifyMask, False,
 		      (XtEventHandler) fontboxResized, (XtPointer) arg);
 }
@@ -1144,7 +1669,9 @@ setDefaultGlobalFont(Display *dpy, char *name)
     Global.xft_ascent = font->ascent;
     Global.xft_descent = font->descent;
     Global.xft_maxadv = font->max_advance_width;
-    Global.xft_rot = 0;
+    Global.xft_rotation = 0;
+    Global.xft_dilation = 1.0;
+    Global.xft_inclination = 0.0;
     Global.xft_linespacing = 1.0;
     str = strdup(name);
     ptr1 = strchr(str, '-');
