@@ -84,7 +84,9 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     save = Global.dashnumber;
     Global.dashnumber = 0;
-    SetCapAndJoin(w, l->gc1, JoinRound);
+    SetCapAndJoin(w, l->gc1,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinRound));
     Global.dashnumber = save;
 
     UndoStartPoint(w, info, event->x, event->y);
