@@ -25,4 +25,4 @@ void RefreshWidget(Widget w);
 void ImageRegionProcess(Widget paint, void * func);
 WidgetList InitializedWidgetList();
 void CheckMarkItems(Widget paint, WidgetList wlist);
-void SetEditable(Widget w, Boolean flag1, Boolean flag2);
+void SetAlphaMode(Widget w, int mode);
