@@ -65,8 +65,6 @@ typedef struct {
     HelpInfo *info;
 } LocalInfo;
 
-static Widget toplevel;
-
 /*
 **  Build hierarchical help information structures.
  */
@@ -344,7 +342,7 @@ buildPopup(LocalInfo * l, Widget parent)
 	return l;
 
     shell = XtVaCreatePopupShell("helpDialog",
-			      topLevelShellWidgetClass, Global.toplevel,
+			         topLevelShellWidgetClass, Global.toplevel,
 				 NULL);
 
     form = XtVaCreateManagedWidget("form",
@@ -491,7 +489,9 @@ buildPopup(LocalInfo * l, Widget parent)
 		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
     XtAddEventHandler(textList, ButtonPressMask, False,
 		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
-#ifdef XAW3D
+
+#ifdef XAW3D_OLD
+    /* no longer needed - and sometimes segfaults ... */
     {
     Pixel pix;
     XtVaGetValues(form, XtNbackground, &pix, NULL);
@@ -512,10 +512,11 @@ HelpDialog(Widget parent, String name)
     static LocalInfo *l = NULL;
     int i;
     HelpInfo *cur;
+    char *str;
 
     PopdownMenusGlobal();
     if (!helpText) return;
-    l = buildPopup(l, toplevel);
+    l = buildPopup(l, Global.toplevel);
 
     for (i = 0, cur = l->info; cur != NULL; cur = cur->next, i++)
 	if (strcmp(name, cur->name) == 0)
@@ -527,9 +528,11 @@ HelpDialog(Widget parent, String name)
     XawListHighlight(l->topicList, cur == NULL ? 0 : i);
 
     XtPopup(l->shell, XtGrabNone);
+    XtVaGetValues(l->shell, XtNtitle, &str, NULL);
+    StoreName(l->shell, str);
 
     display(l, cur);
-    XMapRaised(XtDisplay(l->shell), XtWindow(l->shell));
+    RaiseWindow(XtDisplay(l->shell), XtWindow(l->shell));
     XtSetMinSizeHints(l->shell, 466, 160);
 }
 
@@ -617,6 +620,4 @@ HelpInit(Widget top)
     {"PaintHelp", (XtActionProc) helpAction};
 
     XtAppAddActions(XtWidgetToApplicationContext(top), &act, 1);
-
-    toplevel = top;
 }
