@@ -875,6 +875,19 @@ ImageNormContrast(Image * input)
     }
     whiteval = i;
 
+    if (whiteval == blackval) {
+        for (y = 0; y < h; y++) {
+	    for (x = 0; x < w; x++) {
+	        p = ImagePixel(input, x, y);
+	        *op++ = *p++;
+	        *op++ = *p++;
+	        *op++ = *p;
+	    }
+	}
+        free(hist);
+        return output;
+    }
+
     /*
      * Now create a histogram with a linear ramp
      * from (blackval, 0) to (whiteval, max)
@@ -1470,17 +1483,12 @@ ImageBWMask(Image * input)
 {
     Image *output = ImageNew(input->width, input->height);
     unsigned char *p, *op = output->data;
-    unsigned char bg[3];
     int x, y, v;
 
-    bg[0] = (Global.background>>16)&255;
-    bg[1] = (Global.background>>8)&255;
-    bg[2] = (Global.background)&255;
-
     for (y = 0; y < input->height; y++) {
 	for (x = 0; x < input->width; x++) {
 	    p = ImagePixel(input, x, y);
-            if (memcmp(p, bg, 3)) v = 0xff; else v = 0;
+            if (memcmp(p, Global.bg, 3)) v = 0xff; else v = 0;
 	    *op++ = v;
 	    *op++ = v;
 	    *op++ = v;
@@ -1495,15 +1503,18 @@ ImageBWMask(Image * input)
 Image *
 ImageModifyRGB(Image * input)
 {
-    Image *output = ImageNew(input->width, input->height);
-    unsigned char *p, *op = output->data;
+    Image *output;
+    unsigned char *p, *op;
     int x, y;
     double fr, fg, fb, frp, fgp, fbp;
 
     if (!ImgProcessInfo.redshift &&
         !ImgProcessInfo.greenshift &&
-        !ImgProcessInfo.greenshift) return NULL;
+        !ImgProcessInfo.blueshift) return input;
   
+    output = ImageNew(input->width, input->height);
+    op = output->data;
+
     fr = 2.0*ImgProcessInfo.redshift / (100.01-ImgProcessInfo.redshift);
     fg = 2.0*ImgProcessInfo.greenshift / (100.01-ImgProcessInfo.greenshift);
     fb = 2.0*ImgProcessInfo.blueshift / (100.01-ImgProcessInfo.blueshift);
@@ -1757,8 +1768,9 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
       info->mode = PREDEF_PROCEDURES;
 
    /* compile C script */
-   sprintf(cmd, "cd /tmp\ngcc -fpic -I%s/include -c %s.c -o %s.o\n"
-                "gcc -fpic -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
+   sprintf(cmd, "cd /tmp\ngcc -fPIC -I%s/include -I/usr/include/X11 "
+                "-c %s.c -o %s.o\n"
+                "gcc -fPIC -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
 	        GetShareDir(),
                 tmp, tmp, strstr(tmp, "XPaint"), tmp, tmp);
    XtVaSetValues(info->cclog, XtNstring, cmd, NULL);
@@ -1812,7 +1824,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 	 cmap = XCreateColormap(XtDisplay(Global.toplevel), 
 		    RootWindowOfScreen(screen), visual, AllocNone);
          alpha = image->alpha;
-	 ImageToPixmap(image, Global.toplevel, &pix, &cmap,Global.alpha_mode|4);
+	 ImageToPixmap(image, Global.toplevel, &pix, &cmap);
       }
       if (!pix) {
          Notice(Global.toplevel, msgText[UNABLE_TO_CREATE_IMAGE]);
@@ -1824,9 +1836,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
       }
       paint = graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, 
                             pix, cmap, alpha);
-      SetEditable((Widget)paint,
-                            !(alpha && Global.alpha_mode>=2),
-                            (alpha!=NULL && Global.alpha_mode==0));
+      SetAlphaMode((Widget)paint, -1);
       free(alpha);
       return;
    }
@@ -1845,7 +1855,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 }
 
 static void
-editCallback(Widget w, XtPointer wlArg, XtPointer junk)
+externCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
    FILE *fd;
    LocalInfo *info = (LocalInfo *) wlArg;
@@ -1935,7 +1945,7 @@ loadFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    *buf = '\0';
    if (getcwd(buf, 256)) strcat(buf, "/");
    Global.explore = True;
-   GetFileName(GetShell(w), 3, 
+   GetFileName(GetShell(w), BROWSER_SIMPLEREAD, 
                buf, (XtCallbackProc) loadFileCallbackOK, wlArg);
    Global.explore = False;
 }
@@ -1977,7 +1987,7 @@ saveFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    if (getcwd(buf, 256)) strcat(buf, "/");
    Global.explore = True;
    if (!info->scriptfile || !info->scriptfile[0])
-      GetFileName(GetShell(w), 2, 
+      GetFileName(GetShell(w), BROWSER_SIMPLESAVE, 
                buf, (XtCallbackProc) saveFileCallbackOK, wlArg);
    else
       saveFileCallbackOK(w, wlArg, info->scriptfile);
@@ -1991,7 +2001,7 @@ saveasFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    *buf = '\0';
    if (getcwd(buf, 256)) strcat(buf, "/");
    Global.explore = True;
-   GetFileName(GetShell(w), 2, 
+   GetFileName(GetShell(w), BROWSER_SIMPLESAVE, 
                buf, (XtCallbackProc) saveFileCallbackOK, wlArg);
    Global.explore = False;
 }
@@ -2014,7 +2024,7 @@ loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
    sprintf(directory, "%s/c_scripts/%s/", GetShareDir(),
 	              predefMenu[info->mode].name);
 
-   GetFileName(GetShell(w), 3, directory, 
+   GetFileName(GetShell(w), BROWSER_SIMPLEREAD, directory, 
       (XtCallbackProc) loadFileCallbackOK, wlArg);
    Global.explore = False;
 }
@@ -2171,7 +2181,7 @@ ScriptEditor(Widget w, Widget paint)
     XtAddCallback(fileMenu[FILE_SAVEAS].widget, XtNcallback, 
 		     (XtCallbackProc) saveasFileCallback, (XtPointer) info);
     XtAddCallback(fileMenu[FILE_EDIT].widget, XtNcallback, 
-		     (XtCallbackProc) editCallback, (XtPointer) info);
+		     (XtCallbackProc) externCallback, (XtPointer) info);
     XtAddCallback(fileMenu[FILE_CLOSE].widget, XtNcallback, 
 		     (XtCallbackProc) closeCallback, (XtPointer) info);
     XtAddCallback(compile, XtNcallback,
@@ -2242,7 +2252,7 @@ void ImageToRegion(Image * input, XRectangle rect, Pixmap mask)
 {
     Pixmap pix;
     Colormap cmap;
-    ImageToPixmap(input, Global.curpaint, &pix, &cmap, 0);
+    ImageToPixmap(input, Global.curpaint, &pix, &cmap);
     PwRegionSet(Global.curpaint, &rect, pix, mask);
 }
 
@@ -2256,7 +2266,7 @@ void ImageToCanvas(Image * input, Pixmap mask)
     rect.height = input->height;
     rect.x = 0;
     rect.y = 0;
-    ImageToPixmap(input, Global.curpaint, &pix, &cmap, Global.alpha_mode);
+    ImageToPixmap(input, Global.curpaint, &pix, &cmap);
     PwRegionSet(Global.curpaint, &rect, pix, mask);
     PwRegionSet(Global.curpaint, None, None, None);
 }
