@@ -30,6 +30,8 @@
 #include "Paint.h"
 #include "ops.h"
 
+#define	GET_PW(w)	((PaintWidget)(((PaintWidget)w)->paint.paint == None ? \
+				       w : ((PaintWidget)w)->paint.paint))
 static Boolean multiLine = False;
 static Boolean withArrow = False;
 static int headType = 1;
@@ -302,10 +304,10 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	return;
 
     if (!l->didUndo && info->surface == opPixmap) {
-        PaintWidget pw = (PaintWidget) w;
+        PaintWidget pw = (PaintWidget) GET_PW(w);
         UndoStart(w, info);
 	undo = &pw->paint.undo->box;
-	undo->x = l->rx;
+        undo->x = l->rx;
         undo->y = l->ry;
         undo->width = pw->paint.lineWidth * 2 + 1;
         undo->height = undo->width;
