@@ -41,6 +41,8 @@
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#include <X11/extensions/Xrender.h>
+#include <X11/Xft/Xft.h>
 #include <X11/keysym.h>
 #include <X11/keysymdef.h>
 #include <X11/Shell.h>
@@ -423,6 +425,38 @@ void hexaConvert(unsigned int num, char *val)
   val[1] = (i<10)? '0'+i : 'A'+(i-10);   
 }
 
+#ifdef XAW3DXFT
+extern XftFont * XftDefaultFont;
+#endif
+
+void DrawString(int x, int y, char *str)
+{
+ XRenderColor xre_color;
+ XftColor color;
+ XftDraw *draw = NULL;
+#ifdef XAW3DXFT
+ XftFont * font = XftDefaultFont;
+#else
+ static XftFont * font = NULL;
+ if (!font) font = XftFontOpenName(dpy, DefaultScreen(dpy), "Liberation-8");
+#endif
+
+ xre_color.red = 255<<8;
+ xre_color.green = 0;
+ xre_color.blue = 0;
+ xre_color.alpha = 255<<8;
+ draw = XftDrawCreate(dpy, window,
+                      DefaultVisual(dpy, DefaultScreen(dpy)),
+		      definf.cmap);
+ XftColorAllocValue(dpy, DefaultVisual(dpy, DefaultScreen(dpy)),
+		    definf.cmap, &xre_color, &color);
+ XftDrawStringUtf8(draw, &color, font, x, y, str, strlen(str));
+ XftDrawDestroy(draw);
+ if (DefaultDepth(dpy, DefaultScreen(dpy))>8)
+     XftColorFree(dpy, DefaultVisual(dpy, DefaultScreen(dpy)),
+	          definf.cmap, &color);
+}
+
 void btnUpCB(Widget w_, XtPointer clientData, XEvent *event)
 {
  char buffer[80];
@@ -453,10 +487,9 @@ void btnUpCB(Widget w_, XtPointer clientData, XEvent *event)
   XDestroyImage(xim);
   XSetBackground(dpy, gc, WhitePixelOfScreen(XtScreen(shell)));
   if (event->xbutton.y>LUPEHEIGHT/2)
-    XDrawImageString(dpy, window, gc, 0, 11, buffer, strlen(buffer));
+    DrawString(0, 11, buffer);
   else
-    XDrawImageString(dpy, window, gc, 0, LUPEHEIGHT-2, buffer,
-		     strlen(buffer));      
+    DrawString(0, LUPEHEIGHT-2, buffer);
   return;
  }
 
@@ -1158,35 +1191,36 @@ void infoCB(Widget w, XtPointer clientData, XtPointer callData)
 
  win = Select_Window();
  FillVisinfo(win);
- XClearWindow(dpy, window);   
  l = 18;
  y = 6;
+ XClearWindow(dpy, window);
  sprintf(buffer, "depth:      %d", vinf.depth);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));
+ DrawString(6, (y+=l), buffer);
  sprintf(buffer, "visual:     %s", vinf.vistype);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  sprintf(buffer, "dpy width:  %d", dpywidth);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  sprintf(buffer, "dpy height: %d", dpyheight);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));
+ DrawString(6, (y+=l), buffer);
  sprintf(buffer, "window:     %s", vinf.winname);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));
+ DrawString(6, (y+=l), buffer);
  sprintf(buffer, "win  x:     %d", vinf.x);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  sprintf(buffer, "win  y:     %d", vinf.y);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  sprintf(buffer, "win width:  %d", vinf.width);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  sprintf(buffer, "win height: %d", vinf.height);
- XDrawString(dpy, window, gc, 6, (y+=l), buffer, strlen(buffer));      
+ DrawString(6, (y+=l), buffer);      
  XFlush(dpy) ;
  if (!frozen)
-  usleep(1500000);
+  usleep(3000000);
 }
 
 void jumpCB(Widget w, XtPointer clientData, XtPointer callData)
 {
  Window win;
+ 
  char buffer[80];
 
  if (definf.depth > 8) {
@@ -1203,7 +1237,7 @@ void jumpCB(Widget w, XtPointer clientData, XtPointer callData)
  {
   XClearWindow(dpy, window);    
   sprintf(buffer, "Can't get window.\n");
-  XDrawString(dpy, window, gc, 6, 24, buffer, strlen(buffer));
+  DrawString(6, 24, buffer);
   XFlush(dpy);    
   usleep(1000000);
   return;
