@@ -61,11 +61,19 @@ char lang[10];
 char *routine;
 
 extern int magnifier_closing_down;
+extern int file_specified_zoom;
 extern char *fontNames[];
 
 extern void BrushInit(Widget toplevel);
 extern int TestScriptC(char *file);
 extern Image * ReadScriptC(char *file);
+#ifdef XAW3DXFT
+extern void SetMenuSpacing(int value);
+extern void SetXftEncoding(int value);
+extern void SetXftDefaultFontName(char *name);
+extern void SetXftInsensitiveTwist(char *value);
+extern void SetXawHilitColor(char *value);
+#endif
 
 static char *appDefaults[] =
 {
@@ -76,6 +84,12 @@ static char *appDefaults[] =
 typedef struct {
     String visualType;
     String lang;
+    String encoding;
+    String menufont;
+    String textfont;
+    String spacing;
+    String twistcolor;
+    String hilitcolor;
     String size;
     String winsize;
     String help;
@@ -92,10 +106,11 @@ typedef struct {
     Boolean fullmenu;
     Boolean menubar;
     Boolean canvas;
-    Boolean snapshot;
+    Boolean screenshot;
     Boolean magnifier;
     Boolean horizontal;
     Boolean nowarn;
+    Boolean astext;
 } AppInfo;
 
 static AppInfo appInfo;
@@ -106,6 +121,18 @@ static XtResource resources[] =
      XtOffset(AppInfo *, visualType), XtRImmediate, (XtPointer) "default"},
     {"lang", "Lang", XtRString, sizeof(String),
      XtOffset(AppInfo *, lang), XtRImmediate, (XtPointer) NULL},
+    {"encoding", "Encoding", XtRString, sizeof(String),
+     XtOffset(AppInfo *, encoding), XtRImmediate, (XtPointer) NULL},
+    {"menufont", "MenuFont", XtRString, sizeof(String),
+     XtOffset(AppInfo *, menufont), XtRImmediate, (XtPointer) NULL},
+    {"textfont", "TextFont", XtRString, sizeof(String),
+     XtOffset(AppInfo *, textfont), XtRImmediate, (XtPointer) NULL},
+    {"spacing", "Spacing", XtRString, sizeof(String),
+     XtOffset(AppInfo *, spacing), XtRImmediate, (XtPointer) NULL},
+    {"twistcolor", "TwistColor", XtRString, sizeof(String),
+     XtOffset(AppInfo *, twistcolor), XtRImmediate, (XtPointer) NULL},
+    {"hilitcolor", "HilitColor", XtRString, sizeof(String),
+     XtOffset(AppInfo *, hilitcolor), XtRImmediate, (XtPointer) NULL},
     {"size", "Size", XtRString, sizeof(String),
      XtOffset(AppInfo *, size), XtRImmediate, (XtPointer) DEFAULT_GEOMETRY},
     {"winsize", "WinSize", XtRString, sizeof(String),
@@ -138,14 +165,16 @@ static XtResource resources[] =
      XtOffset(AppInfo *, menubar), XtRImmediate, (XtPointer) True},
     {"canvas", "Canvas", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, canvas), XtRImmediate, (XtPointer) False},
-    {"snapshot", "Snapshot", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, snapshot), XtRImmediate, (XtPointer) False},
+    {"screenshot", "Screenshot", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, screenshot), XtRImmediate, (XtPointer) False},
     {"magnifier", "Magnifier", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, magnifier), XtRImmediate, (XtPointer) False},
     {"horizontal", "Horizontal", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, horizontal), XtRImmediate, (XtPointer) False},
     {"nowarn", "NoWarn", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, nowarn), XtRImmediate, (XtPointer) False},
+    {"astext", "AsText", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, astext), XtRImmediate, (XtPointer) False},
 };
 
 static XrmOptionDescRec options[] =
@@ -153,6 +182,12 @@ static XrmOptionDescRec options[] =
     {"-24", ".visualType", XrmoptionNoArg, (XtPointer) "true24"},
     {"-12", ".visualType", XrmoptionNoArg, (XtPointer) "cmap12"},
     {"-language", ".lang", XrmoptionSepArg, (XtPointer) NULL},
+    {"-encoding", ".encoding", XrmoptionSepArg, (XtPointer) NULL},
+    {"-menufont", ".menufont", XrmoptionSepArg, (XtPointer) NULL},
+    {"-textfont", ".textfont", XrmoptionSepArg, (XtPointer) NULL},
+    {"-spacing", ".spacing", XrmoptionSepArg, (XtPointer) NULL},
+    {"-twistcolor", ".twistcolor", XrmoptionSepArg, (XtPointer) NULL},
+    {"-hilitcolor", ".hilitcolor", XrmoptionSepArg, (XtPointer) NULL},
     {"-size", ".size", XrmoptionSepArg, (XtPointer) NULL},
     {"-winsize", ".winsize", XrmoptionSepArg, (XtPointer) "0x0"},
     {"-sharedir", ".shareDir", XrmoptionSepArg, (XtPointer) SHAREDIR},   
@@ -169,10 +204,11 @@ static XrmOptionDescRec options[] =
     {"-menubar", ".menubar", XrmoptionNoArg, (XtPointer) "True"},
     {"-nomenubar", ".menubar", XrmoptionNoArg, (XtPointer) "False"},
     {"-canvas", ".canvas", XrmoptionNoArg, (XtPointer) "True"},
-    {"-snapshot", ".snapshot", XrmoptionNoArg, (XtPointer) "True"},
+    {"-screenshot", ".screenshot", XrmoptionNoArg, (XtPointer) "True"},
     {"-magnifier", ".magnifier", XrmoptionNoArg, (XtPointer) "True"},
     {"-horizontal", ".horizontal", XrmoptionNoArg, (XtPointer) "True"},
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
+    {"-astext", ".astext", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
     {"+help", ".help", XrmoptionSepArg, (XtPointer) NULL},
     {"-8", ".visualType", XrmoptionNoArg, (XtPointer) "cmap8"},
@@ -366,12 +402,14 @@ static void
 processFile(char *file)
 {
     void *v;
+    int zoom = Global.default_zoom;
 
     StateSetBusy(True);
 
-    if ((v = ReadMagic(file)) != NULL)
-        GraphicOpenFile(Global.toplevel, file, v);
-    else
+    if ((v = ReadMagic(file)) != NULL) {
+        if (file_specified_zoom) zoom = file_specified_zoom;
+        GraphicOpenFileZoom(Global.toplevel, file, v, zoom);
+    } else
 	Notice(Global.toplevel, msgText[UNABLE_TO_OPEN_INPUT_FILE],
 	       file, RWGetMsg());
 
@@ -657,18 +695,19 @@ main(int argc, char *argv[])
     char phrase[128];
     String app_def_file;
     int desiredVisualID = -1;	 
-
+    
+    XftInitFtLibrary();
     InitTypeConverters();
-	 
+
     msgText = NULL;
     helpText = NULL;
     tempargc = argc;
     tempargv = xmalloc(argc * sizeof(char *));
-    
+
     *lang = '\0';
     for (i = 0; i < argc; i++) {
         tempargv[i] = xmalloc(strlen(argv[i])+1);
-	if (i < argc-1 && !strncasecmp(argv[i], "-language", strlen(argv[i])))
+	if (i < argc-1 && !strncasecmp(argv[i], "-language", 5))
 	   strncpy(lang, argv[i+1], 8);
 	strcpy(tempargv[i], argv[i]);
     }
@@ -689,9 +728,19 @@ main(int argc, char *argv[])
     temp = XtAppInitialize(&Global.appContext, app_def_file,
 			   options, XtNumber(options), &tempargc, tempargv,
 			   appDefaults, args, nargs);
+
     XtGetApplicationResources(temp, (XtPointer) & appInfo,
 			      resources, XtNumber(resources), NULL, 0);
 
+#ifdef XAW3DXFT
+    /* set UTF8 (code 0) as default encoding */
+    SetXftEncoding((appInfo.encoding)? atoi(appInfo.encoding) : 0);
+    /* set default Xft police = Liberation-9 point if none defined */
+    SetXftDefaultFontName((appInfo.menufont)? appInfo.menufont:"Liberation-9");
+    SetMenuSpacing((appInfo.spacing)? atoi(appInfo.spacing):1);
+    SetXftInsensitiveTwist((appInfo.twistcolor)? appInfo.twistcolor:"#a00000");
+    SetXawHilitColor((appInfo.hilitcolor)? appInfo.hilitcolor:"#332211");
+#endif
    /*
     **  Load help and messages files
      */
@@ -813,6 +862,10 @@ main(int argc, char *argv[])
 			       appDefaults, args, nargs);
 
     dpy = Global.display = XtDisplay(toplevel);
+    Global.visual = DefaultVisual(dpy, DefaultScreen(dpy));
+    Global.depth = DefaultDepth(dpy, DefaultScreen(dpy));
+    if (Global.depth == 16 && Global.visual->green_mask == 992) 
+        Global.depth = 15;
 
     memset(&appInfo, 0, sizeof(AppInfo));
     XtGetApplicationResources(toplevel, (XtPointer) &appInfo,
@@ -838,12 +891,14 @@ main(int argc, char *argv[])
     Global.clipboard.pix = None;
     Global.clipboard.mask = None;
     Global.clipboard.alpha = NULL;
+    Global.brushpix = None;
     Global.nbrushes = 0;
     Global.brushes = NULL;
     Global.cap = 0;
     Global.join = 0;
     Global.escape = False;
-    Global.transparent = 1;
+    Global.transparent = True;
+    Global.astext = False;
     Global.dashlist[0] = '\0';
     Global.dashnumber = 0;
     Global.dashoffset = 0;
@@ -856,14 +911,19 @@ main(int argc, char *argv[])
     Global.numfiles = 0;
     Global.loadedfiles = NULL;
     Global.numpage = 1;
-    Global.alpha_mode = 2;
-    Global.alpha_default = 255;
+    Global.operation = 0;
+    Global.alpha_bg = 0;
+    Global.alpha_fg = 144;
     Global.alpha_threshold = 128;
-    Global.background = 0xffffff;
+    memset(Global.bg, 0xff, 3);
+    if (Global.depth <= 8) {
+        Global.bg[4] = BlackPixelOfScreen(XtScreen(toplevel));
+        Global.bg[6] = WhitePixelOfScreen(XtScreen(toplevel));
+    }
     /* select by default Times Regular at 18 pt */
+    Global.xft_text = NULL;
     Global.xft_name = NULL;
     Global.xft_font = NULL;
-    setDefaultGlobalFont(dpy, fontNames[1]);
 
     /*
     **
@@ -876,7 +936,7 @@ main(int argc, char *argv[])
     /*
     **  GRR 960525:  check depth and warn user (use AlertBox() instead?)
      */
-    if (!appInfo.nowarn && !appInfo.snapshot) {
+    if (!appInfo.nowarn && !appInfo.screenshot) {
 	int depth = Global.vis.depth;
 
 	/* XtVaGetValues(toplevel, XtNdepth, &depth, NULL); */
@@ -916,6 +976,7 @@ main(int argc, char *argv[])
     /*
     **  Now build and construct the widgets
      */
+
     Global.timeToDie = False;
     Global.explore = False;
 
@@ -933,9 +994,9 @@ main(int argc, char *argv[])
      */
     BrushInit(toplevel);
     HelpInit(toplevel);
-
+    setDefaultGlobalFont(dpy, 
+        (appInfo.textfont)?appInfo.textfont:fontNames[1]);
     SRANDOM(time(0));
-
     SetIconImage(toplevel);
 
     /*
@@ -949,13 +1010,14 @@ main(int argc, char *argv[])
         Global.dpi = atof(appInfo.dpi);
         if (Global.dpi<0) Global.dpi = 0.0;
     }
+    Global.astext = appInfo.astext;
 
     XParseGeometry(appInfo.size, &i, &j, 
                    &Global.default_width, &Global.default_height);
     Global.default_zoom = GetInitZoom();
 
-    if (appInfo.snapshot) {
-        SnapshotImage(toplevel, NULL, 0);
+    if (appInfo.screenshot) {
+        ScreenshotImage(toplevel, NULL, 0);
     }
 
     if (appInfo.magnifier) {
@@ -1036,7 +1098,8 @@ main(int argc, char *argv[])
     }
 
     if (Global.numfiles>opened)
-        GetFileName(Global.toplevel, 4, NULL, GraphicOpenFile, NULL);
+        GetFileName(Global.toplevel, BROWSER_LOADED, NULL, 
+                    GraphicOpenFile, NULL);
 
     /*
     **  MainLoop
@@ -1048,7 +1111,7 @@ main(int argc, char *argv[])
 	XtDispatchEvent(&event);
         if (appInfo.magnifier && argc==1 && magnifier_closing_down) {
 	    XtVaSetValues(toplevel, XtNmappedWhenManaged, True, NULL);
-            XMapRaised(dpy, XtWindow(toplevel));
+            RaiseWindow(dpy, XtWindow(toplevel));
             appInfo.magnifier = 0;
 	}
     }
