@@ -244,12 +244,15 @@ PopdownChild(Widget w, XEvent * event, String * params, Cardinal * nparams)
 
 void SetFocusOn(Widget w)
 {
+  /* No longer useful ? */
+  /*
     XWindowAttributes win_attributes;
     if (!w) return;
     if (!XtIsRealized(w)) return;
     XGetWindowAttributes(XtDisplay(w), XtWindow(w), &win_attributes);
     if (win_attributes.map_state==IsViewable)
         XSetInputFocus(XtDisplay(w), XtWindow(w), RevertToPointerRoot, CurrentTime);
+  */
 }
 
 static void 
@@ -395,8 +398,8 @@ MenuHandler(Widget w, XtPointer l, XEvent * event, Boolean * flg)
     XtVaGetValues(w, XtNwidth, &u, NULL);
 
     if (event->type==KeyPress || event->type == KeyRelease) {
-      len = XLookupString((XKeyEvent *)event, 
-                          buf, sizeof(buf) - 1, &keysym, NULL);
+        len = XLookupString((XKeyEvent *)event, 
+                            buf, sizeof(buf) - 1, &keysym, NULL);
         if (keysym == XK_Escape)
             PopdownMenusGlobal();
     }
@@ -506,6 +509,8 @@ createItem(Widget parent, PaintMenuItem * item,
 	grp = 2;
     else if (item->flags & MF_GROUP4)
 	grp = 3;
+    else if (item->flags & MF_GROUP5)
+	grp = 4;
 
     if (grp != -1) {
 	GroupList *c = findGroup(groups[grp]);
