@@ -137,6 +137,25 @@ GetShell(Widget w)
     return w;
 }
 
+void
+RaiseWindow(Display *dpy, Window win)
+{
+    XEvent event;
+    int j;
+    long mask = SubstructureRedirectMask | SubstructureNotifyMask;
+    char *msg = "_NET_ACTIVE_WINDOW";
+
+    event.xclient.type = ClientMessage;
+    event.xclient.serial = 0;
+    event.xclient.send_event = True;
+    event.xclient.message_type = XInternAtom(dpy, msg, False);
+    event.xclient.window = win;
+    event.xclient.format = 32;
+    for (j=0; j<=4; j++) event.xclient.data.l[j] = 0;
+    XSendEvent(dpy, DefaultRootWindow(dpy), False, mask, &event);
+    XMapRaised(dpy, win);
+}
+
 void 
 SetIBeamCursor(Widget w)
 {
@@ -754,3 +773,10 @@ mousewheelScroll(Widget w, void * l, XEvent * event, Boolean * flg)
     }
 }
 
+void 
+clickFocusCallback(Widget w, XtPointer arg, XEvent * event, Boolean * flg)
+{
+    Widget shell = (Widget) arg;
+    if (event->type == ButtonPress)
+        XtSetKeyboardFocus(((shell)?shell : GetShell(w)), w);
+}
