@@ -424,7 +424,6 @@ gradientFillMenuCallback(Widget w)
 	      strcpy(gradientFillStepsStr, "300");
     }
 
-    printf("%s\n", gradientFillStepsStr);
     value[0].prompt = msgText[GRADIENT_ANGLE];
     value[0].str = gradientFillAngleStr;
     value[0].len = 8;
@@ -589,15 +588,15 @@ exitPaint(Widget w, XtPointer junk, XtPointer junk2)
 **
  */
 void 
-takeSnapshot(Widget w, XtPointer junk, XtPointer junk2)
+takeScreenshot(Widget w, XtPointer junk, XtPointer junk2)
 {
     if (XtParent(w)) w = GetToplevel(XtParent(w));
     if (w) {
        PopdownMenusGlobal();
        XUnmapWindow(XtDisplay(Global.toplevel), XtWindow(Global.toplevel));
        XFlush(XtDisplay(Global.toplevel));
-       usleep(120000);
-       SnapshotImage(w, junk, 0);
+       usleep(200000);
+       ScreenshotImage(w, junk, 0);
        XMapWindow(XtDisplay(Global.toplevel), XtWindow(Global.toplevel));
     }
 }
@@ -1200,7 +1199,7 @@ static IconListItem iconList[] =
     /* Vertical arrangement */
 };
 
-static int 
+int 
 getIndexOp()
 {
     int i;
@@ -1256,65 +1255,6 @@ OperationSet(String names[], int num)
     }
 }
 
-#if 0
-static PaintMenuItem lineMenu[] =
-{
-#if 0
-    MI_FLAGCB("0", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-#endif
-    MI_FLAGCB("1", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-    MI_FLAGCB("2", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-    MI_FLAGCB("4", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-    MI_FLAGCB("6", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-    MI_FLAGCB("8", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-#define LW_SELECT	5
-    MI_FLAGCB("select", MF_CHECK | MF_GROUP1, lineWidth, NULL),
-};
-
-static PaintMenuItem fontMenu[] =
-{
-    MI_FLAGCB("Times 8", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-times-medium-r-normal-*-*-80-*-*-p-*-*-*"),
-    MI_FLAGCB("Times 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-times-medium-r-normal-*-*-120-*-*-p-*-*-*"),
-    MI_FLAGCB("Times 18", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-times-medium-r-normal-*-*-180-*-*-p-*-*-*"),
-    MI_FLAGCB("Times Bold 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-times-bold-r-normal-*-*-120-*-*-p-*-*-*"),
-    MI_FLAGCB("Times Italic 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-times-bold-i-normal-*-*-120-*-*-p-*-*-*"),
-    MI_FLAGCB("Lucida 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-lucida-medium-r-normal-*-*-120-*-*-p-*-*-*"),
-    MI_FLAGCB("Helvetica 12", MF_CHECK | MF_GROUP1,
-	    fontSet, "-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-*-*"),
-    MI_FLAGCB("Helvetica Bold 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-helvetica-bold-r-normal-*-*-120-*-*-p-*-*-*"),
-#ifndef __NetBSD__
-    MI_FLAGCB("Fixed 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-fixed-medium-r-normal-*-*-120-*-*-m-*-*-*"),
-#else
-    MI_FLAGCB("Sony 16", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-fixed-medium-r-normal--16-120-100-100-c-80-iso8859-1"),
-    MI_FLAGCB("Sony 24", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-fixed-medium-r-normal--24-170-100-100-c-120-iso8859-1");
-#endif
-    MI_FLAGCB("Courier 12", MF_CHECK | MF_GROUP1,
-	      fontSet, "-*-courier-medium-r-normal-*-*-120-*-*-m-*-*-*"),
-    MI_SEPARATOR(),
-#define FM_SELECT	11
-    MI_FLAGCB("select", MF_CHECK | MF_GROUP1, fontSet, NULL),
-};
-#endif
-
-#if 0
-static PaintMenuItem otherMenu[] =
-{
-    MI_SIMPLECB("brushSelect", BrushSelect, NULL),
-#define SP_SELECT	1
-    MI_SIMPLECB("sprayEdit", sprayMenuCallback, NULL),
-};
-#endif
-
 static PaintMenuItem canvasMenu[] =
 {
     MI_SIMPLECB("new", GraphicCreate, 0),
@@ -1322,7 +1262,7 @@ static PaintMenuItem canvasMenu[] =
     MI_SIMPLECB("open", GraphicCreate, 2),
     MI_SIMPLECB("loaded", GraphicCreate, 3),
     MI_SIMPLECB("magnifier", GraphicCreate, 4),
-    MI_SIMPLECB("snapshot", takeSnapshot, NULL),
+    MI_SIMPLECB("screenshot", takeScreenshot, NULL),
     MI_SEPARATOR(),
     MI_SIMPLECB("quit", exitPaint, NULL),
 };
@@ -1387,7 +1327,7 @@ OperationAddArg(Arg arg)
 void 
 changeFontAction(Widget w, XEvent * event)
 {
-    FontSelect(w, None);
+    FontSelect(Global.toplevel, None);
 }
 
 static void
@@ -1447,15 +1387,15 @@ changeFractalFillParamAction(Widget w, XEvent * event)
 static void 
 switchtoCanvasCallback(Widget w, XtPointer junk, XEvent * event, Boolean * flg)
 {
-    if (Global.canvas && event->type == ButtonRelease)
-       XMapRaised(XtDisplay(Global.canvas), XtWindow(Global.canvas));
+  if (Global.canvas && event->type == ButtonRelease)
+       RaiseWindow(XtDisplay(Global.canvas), XtWindow(Global.canvas));
 }
 
 void
 processEvent(Widget w, XtPointer arg, XEvent * event, Boolean * flg)
 {
     KeySym keysym;
-    char buf[4];
+    char buf[12];
     int len;
 
     if (event->type == KeyPress || event->type == KeyRelease) {
