@@ -105,6 +105,8 @@ paletteNew(Widget w, Boolean useDefault)
     int end, depth = -1;
     Boolean flg = False;
 
+    PopdownMenusGlobal();
+
     XtVaGetValues(w, XtNvisual, &visual, XtNdepth, &depth, NULL);
     if (visual == NULL || useDefault)
 	visual = DefaultVisualOfScreen(screen);
@@ -539,7 +541,7 @@ PaletteLookup(Palette * map, Pixel pix)
 	col.color.pixel = pix;
 
 	if ((c = (Col *) HashFind(map->ltable, HASH_PIXEL(pix), &col)) == NULL) {
-	    fprintf(stderr,msgText[SHOULDNT_HAPPEN]);
+	    fprintf(stderr,"%s", msgText[SHOULDNT_HAPPEN]);
 	    return NULL;
 	}
 	if (c->invalid) {
