@@ -483,7 +483,7 @@ saveConfigOkCallback(Widget il, LocalInfo * info, char *file)
 void
 saveConfigCallback(Widget w, LocalInfo * info, XtPointer junk)
 {
-    GetFileName(Global.patternshell, 2, ".XPaintrc",
+    GetFileName(Global.patternshell, BROWSER_SIMPLESAVE, ".XPaintrc",
 		(XtCallbackProc) saveConfigOkCallback, (XtPointer) info);
 }
 
@@ -510,7 +510,7 @@ loadConfigOkCallback(Widget il, LocalInfo * info, char *file)
 void
 loadConfigCallback(Widget w, LocalInfo * info, XtPointer junk)
 {
-    GetFileName(Global.patternshell, 3, ".XPaintrc",
+    GetFileName(Global.patternshell, BROWSER_SIMPLEREAD, ".XPaintrc",
 		(XtCallbackProc) loadConfigOkCallback, (XtPointer) info);
 }
 
@@ -1083,7 +1083,6 @@ setAsBackgroundCallback(Widget w, LocalInfo * l, XPointer * junk2)
     Pixel p;
     XColor *color;
     XColor xc;
-    int r, g, b;
 
     if (Global.popped_up) return;
 
@@ -1096,11 +1095,10 @@ setAsBackgroundCallback(Widget w, LocalInfo * l, XPointer * junk2)
     xc.pixel = p;
     xc.flags = DoRed | DoGreen | DoBlue;
     XQueryColor(XtDisplay(w), l->map->cmap, &xc);
-    r = (xc.red >> 8) & 0xff;
-    g = (xc.green >> 8) & 0xff;
-    b = (xc.blue >> 8) & 0xff;
 
-    Global.background = (r<<16) | (g<<8) | b;
+    Global.bg[0] = (xc.red >> 8) & 0xff;
+    Global.bg[1] = (xc.green >> 8) & 0xff;
+    Global.bg[2] = (xc.blue >> 8) & 0xff;
     XtVaSetValues(l->paint, XtNbackground, p, NULL);
     activePixel(l, p);
 }
@@ -1222,7 +1220,7 @@ switchCallback(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
     if (event->type != ButtonRelease);
     if (w == l->icon) {
         if (l->paint)
-           XMapRaised(XtDisplay(l->paint), XtWindow(GetShell(l->paint)));
+           RaiseWindow(XtDisplay(l->paint), XtWindow(GetShell(l->paint)));
 	return;
     }
     l->active = w;
@@ -1263,7 +1261,7 @@ grabPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 
     XtVaGetValues(info->fat, XtNcolormap, &cmap, NULL);
     pix = None;
-    if (ImageToPixmapCmap(image, info->fat, &pix, cmap, 0))
+    if (ImageToPixmapCmap(image, info->fat, &pix, cmap))
 	PwPutPixmap(info->fat, pix);
 }
 
@@ -1387,7 +1385,7 @@ readFileCallback(Widget paint, XtPointer fileArg, XtPointer imageArg)
     int  width, height, depth;
 
     /*  XXX - allocating a new colormap! */
-    if (ImageToPixmap(image, paint, &pix, &cmap, 0)) {
+    if (ImageToPixmap(image, paint, &pix, &cmap)) {
         GetPixmapWHD(XtDisplay(paint), pix, &width, &height, &depth);
         XtVaSetValues(paint, XtNdrawWidth, width,
 		      XtNdrawHeight, height, NULL);
@@ -1400,7 +1398,7 @@ readFileCallback(Widget paint, XtPointer fileArg, XtPointer imageArg)
 static void 
 readCallback(Widget w, XtPointer paint, XtPointer junk)
 {
-    GetFileName((Widget) paint, 0, NULL, readFileCallback, NULL);
+    GetFileName((Widget) paint, BROWSER_READ, NULL, readFileCallback, NULL);
 }
 
 static void 
@@ -1616,7 +1614,7 @@ makePaletteArea(LocalInfo * info, RCInfo * rcInfo)
 	pix = None;
 	rcInfo->images[i]->refCount++;
 	ImageToPixmapCmap(rcInfo->images[i], info->fat, &pix,
-                          info->map->cmap, 0);
+                          info->map->cmap);
 	AddPattern(info, pix, 0);
     }
 }
@@ -1657,6 +1655,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     Position x, y;
     Arg args[6];
     int nargs = 0;
+    char *ptr;
 
     dpy = XtDisplay(w);
     depth = DefaultDepthOfScreen(XtScreen(w));
@@ -1681,7 +1680,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 	initViews(info);
         XMapWindow(dpy, XtWindow(info->r_arrow));
         XUnmapWindow(dpy, XtWindow(info->g_arrow));
-	XMapRaised(dpy, XtWindow(Global.patternshell));
+	RaiseWindow(dpy, XtWindow(Global.patternshell));
 	return;
     }
     StateSetBusyWatch(True);
@@ -2001,6 +2000,8 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 
     copyPaletteArea(info, pixels, patterns, npixels, npatterns);
     XtPopup(Global.patternshell, XtGrabNone);
+    XtVaGetValues(Global.patternshell, XtNtitle, &ptr, NULL);
+    StoreName(Global.patternshell, ptr);
    
     XtUnmanageChild(info->bar);
     if (info->cpick)
