@@ -39,7 +39,7 @@
 #define	IsFill(x)	(x & FILL)
 
 typedef struct {
-    int flag;
+    int flag, zoom;
     int startX, startY, endX, endY, button;
     int drawn, first, tracking, go;
     int npoints;
@@ -156,12 +156,11 @@ finish(Widget w, LocalInfo * l, Boolean flag)
     if (l->npoints<=1) return;
    
     if (polygonType>=2) {
-        Vertices(l->real, l->real[0].x, l->real[0].y, l->real[1].x, l->real[1].y);
-        
+        Vertices(l->real, l->real[0].x,l->real[0].y, l->real[1].x,l->real[1].y);
     }
 
     if (IsRegion(l->flag)) {
-        CreatePolygonalRegion(w, l->points, l->npoints);
+        CreatePolygonalRegion(w, l->real, l->npoints);
         l->npoints = -1;
         return;
     }
@@ -179,8 +178,9 @@ finish(Widget w, LocalInfo * l, Boolean flag)
 	l->npoints++;
     }
 
-    SetCapAndJoin(w, l->lgc, JoinMiter);
-
+    SetCapAndJoin(w, l->lgc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+                  ((Global.join)?Global.join-1:JoinMiter));
     if (!l->isFat)
 	XDrawLines(XtDisplay(w), XtWindow(w), l->lgc,
 		   l->real, l->npoints, CoordModeOrigin);
@@ -378,9 +378,9 @@ static
 LocalInfo * createLocalInfo()
 {
     LocalInfo * l;
-    l = (LocalInfo *) malloc(sizeof(LocalInfo));
-    l->points = (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
-    l->real =   (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
+    l = (LocalInfo *) xmalloc(sizeof(LocalInfo));
+    l->points = (XPoint *) xmalloc(MAXPOINTS*sizeof(XPoint));
+    l->real =   (XPoint *) xmalloc(MAXPOINTS*sizeof(XPoint));
     return l;
 }
 
@@ -403,6 +403,7 @@ PolygonAdd(Widget w)
     l = (LocalInfo *) createLocalInfo();
 
     l->flag = POLY;
+    XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
     l->drawn = False;
     l->first = True;
     l->tracking = False;
@@ -443,6 +444,7 @@ FilledPolygonAdd(Widget w)
 
     l = (LocalInfo *) createLocalInfo();
     l->flag = POLY | FILL;
+    XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
     l->drawn = False;
     l->first = True;
     l->tracking = False;
@@ -483,6 +485,7 @@ SelectPolygonAdd(Widget w)
 
     l = (LocalInfo *) createLocalInfo();
     l->flag = POLY | REGION;
+    XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
     l->drawn = False;
     l->first = True;
     l->tracking = False;
@@ -498,11 +501,6 @@ SelectPolygonAdd(Widget w)
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
 
-    OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) selectKeyPress, l);
-    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) selectKeyRelease, l);
-
     SetCrossHairCursor(w);
 
     return l;
@@ -517,10 +515,6 @@ SelectPolygonRemove(Widget w, void *l)
 			 (OpEventProc) motion, l);
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
-    OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-		         (OpEventProc) selectKeyPress, l);
-    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		         (OpEventProc) selectKeyRelease, l);
 
     finish(w, (LocalInfo *) l, True);
     freeLocalInfo((LocalInfo *) l);
@@ -533,6 +527,7 @@ BrokenlineAdd(Widget w)
 
     l = createLocalInfo();
     l->flag = 0;
+    XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
     l->drawn = False;
     l->first = True;
     l->tracking = False;
