@@ -71,6 +71,7 @@ static int LineBreak;
 static int formatting = 0;
 static int printdata = 0;
 static int nocompress = 0;
+static int ifhex = 0;
 static int bpp_in, bpp_out;
 static int predictor0 = 4;
 
@@ -105,16 +106,16 @@ static void set_linebreak()
 
 /***********************************************************************
  *                                                                     *
- * Name: ASCII85encode                               Date:    05.11.93 *
+ * Name: ASCIIencode                               Date:    05.11.93 *
  * Author: E.Chernyaev (IHEP/Protvino)               Revised: 20.04.95 *
  *                                                                     *
- * Function: ASCII85 encode and output byte buffer                     *
+ * Function: ASCII85 / ASCIIHex encode and output byte buffer          *
  *                                                                     *
  * Input: k   - number of bytes                                        *
  *        Buf - byte buffer                                            *
  *                                                                     *
  ***********************************************************************/
-static void ASCII85encode(k, Buf)
+static void ASCIIencode(k, Buf)
                      int  k;
                      byte Buf[];
 {
@@ -123,6 +124,14 @@ static void ASCII85encode(k, Buf)
   char s[8];
 
   if (k == 0) return;
+  if (ifhex) {
+    for (j=0; j<k; j++) {
+      sprintf(s, "%02x", Buf[j]);
+      put_string(s);
+    }
+    return;
+  }
+
   s[5] = '\0';
   for (j=0; j<k; j+=4) {
     Value = Buf[j]<<24;
@@ -183,7 +192,7 @@ static void PutCode(Code, CodeSize)
     SizeB  = SizeA + SizeB - 8;
     Accum[k++] = PartA | (PartB >> SizeB);
     if (k == 60) {
-      ASCII85encode(k,Accum);
+      ASCIIencode(k,Accum);
       k = 0;
     }
     PartB &= mask[SizeB];
@@ -195,7 +204,7 @@ static void PutCode(Code, CodeSize)
   PartA = PartB << (8-SizeB);
   if (Code == EOD) {
     if (SizeA != 0) Accum[k++] = PartA;
-    ASCII85encode(k, Accum);
+    ASCIIencode(k, Accum);
   }
 }
 
@@ -324,8 +333,8 @@ static void WriteData(Width, Height, ScanLine)
  *                                                                     *
  * Name: psencode                                                      *
  *                                                                     *
- * Function: Output image in PostScript format with ASCII85 and        *
- *           Lempel-Ziv Welch encoding.                                *
+ * Function: Output image in PostScript format with ASCII85/ASCIIHex   *
+ *           and Lempel-Ziv Welch encoding.                            *
  *                                                                     *
  * See top of file for a description of this routine.                  *
  *                                                                     *
@@ -351,6 +360,10 @@ long psencode(mode, Width, Height, ScanLine,
   ifgray     = (mode%10)&4;
   predictor0 = (mode/10)%10;
   nocompress = (mode/100)%10;
+  if (nocompress == 1) {
+    nocompress = 0;
+    ifhex = 1;
+  }
   printdata   = (mode/1000)%10;
 
   if (Width <= 0 || Height <= 0) {
@@ -410,12 +423,13 @@ long psencode(mode, Width, Height, ScanLine,
   else
     sprintf(buffer,
     "save 9 dict begin {\n"
-    "/T currentfile/ASCII85Decode filter def/%s setcolorspace /F T\n"
+    "/T currentfile %s filter def/%s setcolorspace /F T\n"
     "<< /BitsPerComponent 8/Columns %d/Colors %d/Predictor 10 >>\n"
     "/LZWDecode filter def << /ImageType 1"
     "/Width %d/Height %d/BitsPerComponent 8\n"
     "/ImageMatrix[1 0 0 -1 0 %d]/Decode %s/DataSource F >> image\n"
     "F closefile T closefile} exec\n",
+    ifhex? "/ASCIIHexDecode" : "/ASCII85Decode",
     ifgray? "DeviceGray" : "DeviceRGB", Width, ifgray? 1 : 3,
     Width, Height, Height,
     ifgray?"[0 1]" : "[0 1 0 1 0 1]");
@@ -426,7 +440,10 @@ long psencode(mode, Width, Height, ScanLine,
 
   if (!nocompress) {
       EncodeData(Width, Height, ScanLine);
-      put_string("~>");
+      if (ifhex)
+          put_string(">");
+      else
+          put_string("~>");
   } else {
       predictor0 = 5;
       WriteData(Width, Height, ScanLine);
