@@ -44,6 +44,7 @@
 #include "Paint.h"
 #include "PaintP.h"
 #include "palette.h"
+#include "print.h"
 #include "xpaint.h"
 #include "menu.h"
 #include "image.h"
@@ -536,7 +537,7 @@ fileselectCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     PopdownMenusGlobal();
     XtVaGetValues(info->filename, XtNstring, &name, NULL);
     XtVaSetValues(info->filetoggle, XtNstate, True, NULL);
-    GetFileName(GetToplevel(w), 3, name, 
+    GetFileName(GetToplevel(w), BROWSER_SIMPLESAVE, name, 
                 (XtCallbackProc) selectionOkCallback, (XtPointer)infoArg);
 }
 
@@ -584,6 +585,7 @@ closePrintCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     Display *dpy;
     char *ptr;
 
+    PopdownMenusGlobal();
     XtVaGetValues(info->printcmd, XtNstring, &ptr, NULL);
     if (ptr) {
        if (printcmd_str) free(printcmd_str);
@@ -1052,7 +1054,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     if (printinfo->shell) {
         if (printinfo->paintwidget == (PaintWidget)paintArg) {
-            XMapRaised(dpy, XtWindow(printinfo->shell));
+            RaiseWindow(dpy, XtWindow(printinfo->shell));
             return;
 	} else
 	    XtDestroyWidget(printinfo->shell);
@@ -1177,30 +1179,35 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->wpplus = XtVaCreateManagedWidget("wpplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->wplus = XtVaCreateManagedWidget("wplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wpplus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->wminus = XtVaCreateManagedWidget("wminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wplus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->wmminus = XtVaCreateManagedWidget("wmminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wminus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->wcenter = XtVaCreateManagedWidget("wcenter",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wmminus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->hpplus = XtVaCreateManagedWidget("hpplus",
@@ -1208,30 +1215,35 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->bar,
 				       XtNvertDistance, 6,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->hplus = XtVaCreateManagedWidget("hplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hpplus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->hminus = XtVaCreateManagedWidget("hminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hplus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->hmminus = XtVaCreateManagedWidget("hmminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hminus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->hcenter = XtVaCreateManagedWidget("hcenter",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hmminus,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->equal = XtVaCreateManagedWidget("equal",
@@ -1239,6 +1251,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNstate, printinfo->tied,
+				       XtNwidth, 18,
 				       NULL);
 
     if (translations == None) {
@@ -1266,6 +1279,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->bar,
 				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->landscapelabel = XtVaCreateManagedWidget("landscapelabel",
@@ -1279,11 +1293,12 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->landscapetoggle = XtVaCreateManagedWidget("landscapetoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, printinfo->orient,
-				       XtNfromHoriz, printinfo->landscapelabel,
+				       XtNfromHoriz, printinfo->hpplus,
                                        XtNradioGroup,printinfo->portraittoggle,
 				       XtNfromVert, printinfo->bar,
-				       XtNhorizDistance, 12,
+				       XtNhorizDistance, 324,
 				       XtNvertDistance, 6,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->graylabel = XtVaCreateManagedWidget("graylabel",
@@ -1302,6 +1317,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->portraittoggle,
 				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->compresslabel = XtVaCreateManagedWidget("compresslabel",
@@ -1309,6 +1325,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->graytoggle,
 				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 16,
 				       XtNvertDistance, 6,
 				       NULL);
 
@@ -1320,23 +1337,26 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
 				       XtNstate, True,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->pdflabel = XtVaCreateManagedWidget("pdflabel",
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
-				       XtNfromHoriz, printinfo->compresstoggle,
+				       XtNfromHoriz, printinfo->hpplus,
 				       XtNfromVert, printinfo->portraittoggle,
 				       XtNvertDistance, 6,
+				       XtNhorizDistance, 280,
 				       NULL);
 
     printinfo->pdftoggle = XtVaCreateManagedWidget("pdftoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, 0,
-				       XtNfromHoriz, printinfo->pdflabel,
+				       XtNfromHoriz, printinfo->hpplus,
 				       XtNfromVert, printinfo->portraittoggle,
-				       XtNhorizDistance, 12,
+				       XtNhorizDistance, 324,
 				       XtNvertDistance, 6,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->sizelabel = XtVaCreateManagedWidget("sizelabel",
@@ -1345,7 +1365,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->hpplus,
 				       XtNfromVert, printinfo->graytoggle,
 				       XtNhorizDistance, 30,
-				       XtNvertDistance, 16,
+				       XtNvertDistance, 12,
 				       NULL);
 
     printinfo->size = XtVaCreateManagedWidget("sizevalue",
@@ -1357,7 +1377,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->hpplus,
 				       XtNfromVert, printinfo->graytoggle,
 				       XtNwidth, 150,
-				       XtNvertDistance, 16,
+				       XtNvertDistance, 12,
 				       NULL);
 
     printinfo->positionlabel = XtVaCreateManagedWidget("positionlabel",
@@ -1404,7 +1424,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->hpplus,
 				       XtNhorizDistance, 30,
-				       XtNvertDistance, 20,
+				       XtNvertDistance, 14,
 				       NULL);
 
     printinfo->printcmd = XtVaCreateManagedWidget("printcmd",
@@ -1415,7 +1435,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNvertDistance, 20,
+				       XtNvertDistance, 14,
 				       XtNwidth, 150, XtNheight, 40,
 				       NULL);
     if (printcmd_str) 
@@ -1437,11 +1457,11 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNmenuName, "printermenu",
 				       XtNborderWidth, 1,
 				       XtNlabel, "?",
-				       XtNwidth, 17,
+				       XtNwidth, 18,
 				       XtNheight, 16,
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->printcmd,
-				       XtNvertDistance, 21,
+				       XtNvertDistance, 15,
 #ifdef XAW3D
                                        XtNborderWidth, 0,							    
 #endif							    
@@ -1469,7 +1489,8 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->printcmd,
 				       XtNvertDistance, 
-					  (printinfo->printerlist==0)? 32: 41,
+					  (printinfo->printerlist==0)? 28: 37,
+				       XtNwidth, 18,
 				       NULL);
    
     printinfo->filelabel = XtVaCreateManagedWidget("filelabel",
@@ -1498,7 +1519,7 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->fileselect = XtVaCreateManagedWidget("fileselect",
 				       commandWidgetClass, printinfo->form,
 				       XtNlabel, "?",
-				       XtNwidth, 17,
+				       XtNwidth, 18,
 				       XtNheight, 16,
 				       XtNfromVert, printinfo->printcmd,
 				       XtNfromHoriz, printinfo->filename,
@@ -1512,6 +1533,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->filename,
 				       XtNradioGroup, printinfo->printtoggle,
                                        XtNvertDistance, 5,
+				       XtNwidth, 18,
 				       NULL);
 
     printinfo->psviewlabel = XtVaCreateManagedWidget("psviewlabel",
@@ -1538,7 +1560,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNborderWidth, 0,
 				       XtNfromVert, printinfo->psviewcmd,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNvertDistance, 20,
+				       XtNvertDistance, 14,
 				       XtNhorizDistance, 30,
 				       NULL);
 
@@ -1550,7 +1572,8 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->psviewcmd,
 				       XtNfromHoriz, printinfo->resultlabel,
-				       XtNvertDistance, 20,
+				       XtNvertDistance, 14,
+				       XtNheight, 40,
 				       NULL);
 
     if (psviewcmd_str) 
@@ -1684,6 +1707,7 @@ static void
 closeExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 {
     ExternInfo *info = (ExternInfo *) infoArg;
+    PopdownMenusGlobal();    
     XtDestroyWidget(info->shell);
     info->shell = 0;
 }
@@ -1817,7 +1841,7 @@ ExternPopup(Widget w, XtPointer paintArg)
     }
     if (externinfo->shell) {
         if (externinfo->paintwidget == (PaintWidget)paintArg) {
-            XMapRaised(dpy, XtWindow(externinfo->shell));
+            RaiseWindow(dpy, XtWindow(externinfo->shell));
             return;
 	} else
 	    XtDestroyWidget(externinfo->shell);
