@@ -217,7 +217,7 @@ add(Widget w, LocalInfo * l)
 			   CopyFromParent, valuemask, &attributes);
 
     if (DO_MAP(l))
-	XMapRaised(l->dpy, l->win);
+	RaiseWindow(l->dpy, l->win);
 }
 
 
@@ -301,13 +301,13 @@ setBusy(LocalInfo * cur, Boolean isWatch, Boolean flag)
     }
     if (cur->watchCount == 1) {
 	if (isWatch) {
-	    XMapRaised(cur->dpy, cur->win);
+	    RaiseWindow(cur->dpy, cur->win);
 	    XDefineCursor(cur->dpy, cur->win, watchCursor);
 	    doFlush = True;
 	}
     } else if (cur->computingCount == 1) {
 	if (!isWatch) {
-	    XMapRaised(cur->dpy, cur->win);
+	    RaiseWindow(cur->dpy, cur->win);
 	    XDefineCursor(cur->dpy, cur->win, cursorInfo[currentCursor].cursor);
 	    doFlush = True;
 	}
