@@ -25,6 +25,8 @@
 #define FALSE 0
 #endif
 
+extern void *xmalloc(size_t n);
+
 static void nop(void)
 {
 }
@@ -62,12 +64,12 @@ static int getImage(TIFF * tif, Image * image, void *dp,
     if (planarconfig == PLANARCONFIG_SEPARATE) {
 	scansize = TIFFStripSize(tif);
 
-	sline = (unsigned char *) malloc(3 * scansize);
+	sline = (unsigned char *) xmalloc(3 * scansize);
 	sptr0 = sline;
 	sptr1 = sptr0 + scansize;
 	sptr2 = sptr1 + scansize;
     } else {
-	if ((sline = (unsigned char *) malloc(TIFFScanlineSize(tif))) == NULL) {
+	if ((sline = (unsigned char *) xmalloc(TIFFScanlineSize(tif))) == NULL) {
 	    RWSetMsg("Unable to allocate memory for scanline");
 	    return TRUE;
 	}
@@ -335,10 +337,10 @@ Image *ReadTIFF(char *file)
 	    goto out;
 
 	ImageMakeMask(image);
-	if (getImage(tif, image, (void *) image->maskData,
+	if (getImage(tif, image, (void *) image->alpha,
 		     bitsPS, samplesPP, photometric)) {
-	    free(image->maskData);
-	    image->maskData = NULL;
+	    free(image->alpha);
+	    image->alpha = NULL;
 	}
     }
   out:
