@@ -10,8 +10,10 @@
  
 #include <stdio.h>
 #include <stdlib.h>
+
 #include <X11/Intrinsic.h>
-#include <xpm.h>
+#include <X11/xpm.h>
+
 #include "xpaint.h"
 #include "image.h"
 #include "libpnmrw.h"
@@ -28,6 +30,8 @@
 #define WIN_INFOHEADER_LEN 40
 #define OS2_INFOHEADER_LEN 12
 
+extern void *xmalloc(size_t n);
+
 /* Header structure definition. */
 typedef struct {
 	int class;			/* Windows or OS/2 */
@@ -65,7 +69,7 @@ bfill(char *s, int n, int c)
         int b;
                 
         /* bytes to next word */
-        b = (0 - (int) s) & (sizeof(unsigned long) - 1);
+        b = (0 - (long) s) & (sizeof(unsigned long) - 1);
         if (n < b)
                 b = n;
         while (n != 0) {
@@ -162,7 +166,8 @@ static Boolean read_bmpHeader(FILE * fd, bmpHeader * hp)
 	    && hp->biBitCount != 4
 	    && hp->biBitCount != 8
 	    && hp->biBitCount != 24) {
-	  fprintf(stderr, "bmpLoad: Illegal image BitCount\n");
+	    fprintf(stderr, "bmpLoad: Illegal image BitCount %d\n", 
+                    hp->biBitCount);
 		return FALSE;
 	}
 	if ((hp->biCompression != BI_RGB
@@ -171,11 +176,12 @@ static Boolean read_bmpHeader(FILE * fd, bmpHeader * hp)
 	    || (hp->biCompression == BI_RLE8 && hp->biBitCount != 8)
 	    || (hp->biCompression == BI_RLE4 && hp->biBitCount != 4)) {
 		fprintf(stderr,
-			"bmpLoad: Illegal image compression type\n");
+			"bmpLoad: Illegal image compression type %ld\n",
+                        hp->biCompression);
 		return FALSE;
 	}
 	if (hp->biPlanes != 1) {
-	  fprintf(stderr, "bmpLoad: Illegal image Planes value\n");
+	    fprintf(stderr, "bmpLoad: Illegal image Planes value %d\n", hp->biPlanes);
 		return FALSE;
 	}
 	/* Fix up a few things */
@@ -297,7 +303,7 @@ Image
 		padlen = (((image->width + 31) / 32) * 4) - illen;
 		/* start at bottom */
 		data = image->data + (image->height - 1) * image->width;
-		buf = (unsigned char *)malloc(illen+1);
+		buf = (unsigned char *)xmalloc(illen+1);
 		if (!buf) goto data_short;
 		for (y = image->height; y > 0; y--) {
 			/* BMP files are left bit == ms bit,
@@ -519,6 +525,28 @@ Image
 					goto data_short;
 			}
 		}
+	} else if (hdr.biBitCount == 16) {
+		unsigned char *data, pad[4];
+		int illen, padlen, y;
+		illen = 3 * image->width;
+	        /* extra bytes to word boundary */
+	        padlen = ((illen + 3) & ~3) - illen;
+		/* start at bottom */	
+		data = image->data + (image->height - 1) * illen;	
+		for (y = image->height; y > 0; y--, data -= illen) {
+			int i;
+
+			/* BMP files are RGB, so read straight in. */
+			if (fread(data, 1, illen*2/3, fd) != illen
+                            || fread(pad, 1, 0, fd) != 0)
+				goto data_short;
+			/* Oh, no they're not */
+			for (i = 3 * image->width - 1; i > 0; i -= 3) {
+				int t = data[i];
+				data[i] = data[i - 2];
+				data[i - 2] = t;
+			}
+		}
 	} else {		/* hdr.biBitCount == 24 */
 		unsigned char *data, pad[4];
 		int illen, padlen, y;
@@ -541,7 +569,6 @@ Image
 				data[i - 2] = t;
 			}
 		}
-
 	}
 	if (data_bounds)
 		fprintf(stderr, "bmpLoad: Data outside image area\n");
@@ -608,7 +635,7 @@ WriteBMP(char *file, Image * image)
         bpl = ((w+31)/32)*4;
     } else {
         /* try compressing image to palette mode, but don't force if too big */
-        if (!image->maskData)   /* can't store alpha mask with palette image */
+        if (!image->alpha)   /* can't store alpha mask with palette image */
             cmapImage = ImageCompress(image, 256, 1);
         if (cmapImage) {
             image = cmapImage;  /* original was deleted in ImageCompress() */
