@@ -19,8 +19,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+
 #include <X11/Intrinsic.h>
-#include <xpm.h>
+#include <X11/xpm.h>
+
 #include "xpaint.h"
 #include "image.h"
 #include "libpnmrw.h"
@@ -344,7 +346,7 @@ ReadICO (char *file)
 
     if (have_transparent_pixels) {
 	ImageMakeMask(xpaint_image);
-	mp = xpaint_image->maskData;
+	mp = xpaint_image->alpha;
     }
     /* Write the image */
     for (y=0; y<height; y++)
