@@ -32,7 +32,6 @@ typedef char *String;
 #  endif
 #endif
 
-extern int file_isSpecialImage;
 extern int file_transparent;
 
 extern void *xmalloc(size_t n);
@@ -77,7 +76,7 @@ ReadPNG(char *file)
     png_color_16p palette16;
     int           num_trans;
     int           bit_depth, color_type, interlace_type, num_palette, rowbytes;
-    int           i, j, hasAlpha;
+    int           i, hasAlpha;
     int           level /* , npasses */ ;
     Image         *image = NULL;
 
@@ -140,7 +139,6 @@ ReadPNG(char *file)
             image = ImageNewCmap(width, height, num_palette);
             png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, &palette16);
             if (num_trans && trans) {
-                file_isSpecialImage = 1;
                 file_transparent = 1;
                 ImageMakeMask(image);   /* alpha is 1 byte deep */
 	    }
@@ -213,7 +211,6 @@ ReadPNG(char *file)
             }
             /* need to split alpha and RGB/gray the hard way, sigh */
             hasAlpha = TRUE;
-            file_isSpecialImage = 1;
             file_transparent = 1;
             image = ImageNew(width, height);
             ImageMakeMask(image);   /* alpha is 1 byte deep */
@@ -229,7 +226,6 @@ ReadPNG(char *file)
                 png_set_strip_16(png_ptr);
             }
             hasAlpha = TRUE;
-            file_isSpecialImage = 1;
             file_transparent = 1;
             image = ImageNewGrey(width, height);
             ImageMakeMask(image);   /* alpha is 1 byte deep */
@@ -469,41 +465,41 @@ WritePNG(char *file, Image *image, int interlace_type)
      * ImageCompress() worked
      */
     if (image->scale == 1) {
-        color_type = PNG_COLOR_TYPE_PALETTE;
-        if (image->alpha) {
-            fprintf(stderr, "%s", 
-              msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
-            fflush(stderr);
-        }
+        int must_remap = FALSE;
+        unsigned char *p, *q;
+
         Trace((stderr, "GRR WritePNG:  palette..."));
         if (!image->cmapPacked)
             compressColormap(image);   /* apparently not destructive... */
 
-        if (image->cmapSize > 16) {
-            unsigned char *p;
-            int must_remap = FALSE;
+        /* first guess = gray - will be changed just afterwards if non gray */
+        if (image->alpha)
+	    color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
+        else
+            color_type = PNG_COLOR_TYPE_GRAY;
 
+        p = image->cmapData;   /* 1D array of RGB triplets */
+        for (i = image->cmapSize;  i > 0;  --i, p += 3) {
+              /*   red != green ||   red != blue */
+            if (p[0] != p[1]  ||  p[0] != p[2]) {
+                color_type = PNG_COLOR_TYPE_PALETTE;  /* not gray, alas */
+                break;
+            }
+            /* red = green = blue, but if != index, will have to remap */
+            if (p[0] != i)
+                must_remap = TRUE;
+        }
+
+        if (image->cmapSize > 16) {
             bit_depth = 8;
             /* if palette has more than 16 entries, all of which are gray, it's
              * more efficient to omit palette entirely and write a grayscale
              * PNG */
-            color_type = PNG_COLOR_TYPE_GRAY;
-            p = image->cmapData;   /* 1D array of RGB triplets */
-            for (i = image->cmapSize;  i > 0;  --i, p += 3) {
-                /*   red != green ||   red != blue */
-                if (p[0] != p[1]  ||  p[0] != p[2]) {
-                    color_type = PNG_COLOR_TYPE_PALETTE;  /* not gray, alas */
-                    break;
-                }
-                /* red = green = blue, but if != index, will have to remap */
-                if (p[0] != i)
-                    must_remap = TRUE;
-            }
-            if (color_type == PNG_COLOR_TYPE_GRAY) {
+
+            if (color_type == PNG_COLOR_TYPE_GRAY ||
+                color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
                 Trace((stderr, "no, grayscale..."));
                 if (must_remap) {
-                    unsigned char *q;
-
                     /* reset all pixels to [red] palette value indexed by
                      * current pixel value (now know red == green == blue) */
                     Trace((stderr, "remapping..."));
@@ -585,17 +581,18 @@ WritePNG(char *file, Image *image, int interlace_type)
     png_set_packing(png_ptr);   /* squish multiple pixels into each byte */
 /*  png_set_compression_level(png_ptr, Z_BEST_COMPRESSION);  */
 
-#if 0
-    if (image->alpha) {
+    /* shouldn't occur !! */
+    if (image->alpha && color_type == PNG_COLOR_TYPE_PALETTE) {
         /* alpha channel version */
-        fprintf(stderr, "%s", msgText[WRITEPNG_SORRY_CANT_WRITE_ALPHA_IMAGES_YET]);
+        RWSetMsg(msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
+        fprintf(stderr, "%s", msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
         fflush(stderr);
         png_destroy_write_struct(&png_ptr, &info_ptr);
         fclose(fp);
         return 1;
 
     }
-#endif
+
     rowbytes = (image->scale+((image->alpha)?1:0)) * image->width;
     row_pointers = (png_bytep *)xmalloc(image->height * sizeof(png_bytep));
     if (!row_pointers) {
