@@ -29,9 +29,12 @@
 #include <time.h>
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
+#include <X11/StringDefs.h>
 #include "xpaint.h"
 #include "image.h"
 #include "libpnmrw.h"
+#include "misc.h"
+#include "print.h"
 
 #if defined(sco) || defined(__CYGWIN__)
 #include <time.h>
@@ -48,14 +51,7 @@
 #define FALSE	0
 #endif
 
-enum {NO_FMT=0, PS_FMT, PDF_FMT, TEX_FMT, LTX_FMT, DVI_FMT, SVG_FMT};
-
-typedef struct {
-    float wpercent, hpercent;
-    int wbbox, hbbox, width, height, rx, ry, wsubdiv, hsubdiv;
-    Boolean orient, eps, gray, compress, pdf;
-    char *title;
-} PageInfo;
+enum {NO_FMT=0, PS_FMT, PDF_FMT, TEX_FMT, LTX_FMT, DVI_FMT, SVG_FMT, TXT_FMT};
 
 typedef unsigned char byte;
 typedef unsigned int  uint;
@@ -65,6 +61,7 @@ extern int file_transparent;
 extern int file_numpages;
 extern int file_force;
 extern int file_bbox;
+extern int file_specified_zoom;
 extern int paper_sizes[15][2];
 
 extern void * xmalloc(size_t n);
@@ -654,14 +651,17 @@ TestPS_(char *file)
     if (!strncasecmp(header, "%TEX", 4)) return TEX_FMT;
     if (!strncasecmp(header, "%LTX", 4)) return LTX_FMT;
     if (!strncasecmp(header, "\367\002", 2)) return DVI_FMT;
-    if (strstr(file, ".ltx")) return 4;
-    if (strstr(file, ".tex")) return 3;
+    if (strstr(file, ".ltx")) return LTX_FMT;
+    if (strstr(file, ".tex")) return TEX_FMT;
     if (strstr(file, ".svg") && !strncasecmp(header, "<?xml", 5)) {
        fp = fopen(file, "rb");
        fread(header, 1, 512, fp);
        fclose(fp);
        if (strstr(header, "<svg")) return SVG_FMT;
     }
+    if (strstr(file, ".txt") || !strncasecmp(header, "\\*text", 6) ||
+        Global.astext)
+        return TXT_FMT;
     return 0;
 }
 
@@ -675,12 +675,16 @@ ReadPS_(char *file)
     static char * last_rad = NULL;
     static int  last_type = 0;
     char buffer[2048];
-    char *sc, *dc;
+    char *sc, *dc, *ptr;
     FILE *fp;
     int wth, hth, type_doc;
     int i, j;
     double x1=0.0, y1=0.0, x2=1000.0, y2=1414.0, res;
     Image *image = NULL;
+    Pixmap pix;
+    Colormap cmap;
+    void *info = NULL;
+    char *dir;
 
     fp = fopen(file, "rb");
     if (!fp) return NULL;
@@ -699,6 +703,7 @@ ReadPS_(char *file)
     file_transparent = 1;
 
     openTempDir(rad);
+    dir = strdup(rad);
     strcat(rad, "/");
     strcpy(buffer, file);
     if ((sc=strrchr(buffer, '/'))) {
@@ -715,16 +720,18 @@ ReadPS_(char *file)
            sprintf(buffer, 
                "pdf2ps %s %s.ps ", file, rad);
        else
-       if (type_doc==TEX_FMT)
+       if (type_doc==TEX_FMT) {
            sprintf(buffer, 
-               "tex \"\\\\nonstopmode\\\\input\" %s ; dvips %s.dvi -o %s.ps ",
-               file, rad, rad);
-       else
-       if (type_doc==LTX_FMT)
+               "cd %s ; tex \"\\\\nonstopmode\\\\input\" %s ; dvips %s.dvi -o %s.ps ",
+	       dir, file, rad, rad);
+           free(dir);
+       } else
+       if (type_doc==LTX_FMT) {
            sprintf(buffer, 
-               "latex \"\\\\nonstopmode\\\\input\" %s ; dvips %s.dvi -o %s.ps ",
-               file, rad, rad);
-       else
+               "cd %s ; latex \"\\\\nonstopmode\\\\input\" %s ; dvips %s.dvi -o %s.ps ",
+	       dir, file, rad, rad);
+           free(dir);
+       } else
        if (type_doc==DVI_FMT)
            sprintf(buffer, 
                "dvips %s -o %s.ps ", file, rad);
@@ -741,6 +748,40 @@ ReadPS_(char *file)
     } else
        strcpy(rad, file);
 
+    if (type_doc==TXT_FMT) {
+       fp = fopen(rad, "rb");
+       fgets(buffer, 2040, fp);
+       wth = 1536;
+       hth = 2172;
+       if (!strncmp(buffer, "\\*text:", 7)) {
+	   sscanf(buffer+7, "%d %d*", &wth, &hth);
+           if (wth<=0) wth = 1536;
+           if (hth<=0) hth = 2172;
+           ptr = strstr(buffer, "\\*zoom:");
+           if (ptr)
+	       file_specified_zoom=atoi(ptr+7);
+       }
+
+       /* initialize pixmap and data structures */
+       sprintf(buffer, "%d %d", wth, hth);
+       info = NULL;
+       WriteText(Global.toplevel, &info, buffer);
+
+       rewind(fp);
+       while (!feof(fp)) {
+	   if (fgets(buffer, 2040, fp))
+               WriteText(Global.toplevel, &info, buffer);
+       }
+       pix = GetLocalInfoDrawable(info);
+       XtVaGetValues(Global.toplevel, XtNcolormap, &cmap, NULL);
+       image = PixmapToImage(Global.toplevel, pix, cmap);
+
+       /* free structures */
+       WriteText(Global.toplevel, &info, NULL);
+       fclose(fp);
+       return image;
+    }
+
     wth = Global.default_width;
     hth = Global.default_height;
 
@@ -787,12 +828,13 @@ ReadPS_(char *file)
        x2 = (double)i;
        y2 = (double)j;
     }
+
     if (i && file_bbox==1) {
        int mx = 1191, my = 1684;
        x1 = y1 = 0;
        if (x2<y2) {
           for (j = 0; j<15; j++) {
-	      if (x2<paper_sizes[j][0]+1 && y2<paper_sizes[j][1]+1 &&
+	      if (x2<paper_sizes[j][0]+1.5 && y2<paper_sizes[j][1]+1.5 &&
 		  paper_sizes[j][0]<=mx && paper_sizes[j][1]<=my) {
 		 mx = paper_sizes[j][0];
 		 my = paper_sizes[j][1];
@@ -803,7 +845,7 @@ ReadPS_(char *file)
        }
        if (x2>=y2) {
           for (j = 0; j<15; j++) {
-	      if (y2<paper_sizes[j][0]+1 && x2<paper_sizes[j][1]+1 &&
+	      if (y2<paper_sizes[j][0]+1.5 && x2<paper_sizes[j][1]+1.5 &&
 		  paper_sizes[j][0]<=mx && paper_sizes[j][1]<=my) {
 		 mx = paper_sizes[j][0];
 		 my = paper_sizes[j][1];
