@@ -23,7 +23,6 @@
 #include "image.h"
 #include "rwTable.h"
 
-extern int file_isSpecialImage;
 extern int file_transparent;
 
 int WriteXPM(char *file, Image * image)
@@ -74,23 +73,20 @@ Image * ReadXPM(char *file)
     Display *dpy = Global.display;
     XImage *xim, *mim = NULL;
     int x, y, i;
-    Image *image, *input;
+    Image *image;
     XpmAttributes attr;
     XpmColorSymbol bg;
     XColor bgcolor;
     XColor *col;
-    unsigned char c, r, g, b, *ucp, *ump = NULL;
+    unsigned char *ucp, *ump = NULL;
     unsigned short *usp;
     Colormap cmap = DefaultColormap(dpy, DefaultScreen(dpy));
     int status;
 
-    r = (Global.background>>16)&255;
-    g = (Global.background>>8)&255;
-    b = (Global.background)&255;
     bgcolor.flags = DoRed | DoGreen | DoBlue;
-    bgcolor.red = r*257;
-    bgcolor.green = g*257;
-    bgcolor.blue = b*257;
+    bgcolor.red = Global.bg[0]*257;
+    bgcolor.green = Global.bg[1]*257;
+    bgcolor.blue = Global.bg[2]*257;
     XAllocColor(dpy, cmap, &bgcolor);
     attr.valuemask = XpmColormap | XpmReturnPixels | XpmColorSymbols;
     attr.colormap =  cmap;
@@ -137,7 +133,6 @@ Image * ReadXPM(char *file)
     XQueryColors(dpy, cmap, col, attr.npixels);
 
     if (mim != NULL) {
-        file_isSpecialImage = 1;
         file_transparent = 1;
 	ImageMakeMask(image);
 	ump = image->alpha;
