@@ -10,9 +10,10 @@ typedef int (*RWtestFunc) (char *);
 
 void *RWtableGetEntry(char *);
 #ifdef __IMAGE_H__
+int writeMagic(char *, Image *);
+Image *readMagic(char *);
 RWreadFunc RWtableGetReader(void *);
 RWwriteFunc RWtableGetWriter(void *);
-RWwriteFunc RWtableGetWriterFromSuffix(char *suffix);
 #endif
 char **RWtableGetReaderList(void);
 char **RWtableGetWriterList(void);
