@@ -23,7 +23,6 @@
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
 #include <X11/Shell.h>
-#include <X11/keysym.h>
 
 #include "xaw_incdir/Form.h"
 #include "xaw_incdir/AsciiText.h"
@@ -168,74 +167,16 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     l->real_p[0].x = info->x;
     l->real_p[0].y = info->y;
+#if 0
     l->window_p[0].x = l->wx - info->zoom / 2;
     l->window_p[0].y = l->wy - info->zoom / 2;
+#endif
+    l->window_p[0].x = l->wx;
+    l->window_p[0].y = l->wy;
     l->npoint = 1;
     l->tracking = True;
 }
 
-void 
-selectKeyPress(Widget w, void * l, XKeyEvent * event, void * info)
-{
-    KeySym keysym;
-    int len, i, dx, dy;
-    char buf[21];
-
-    len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
-
-    dx = dy = 0;
-    switch (keysym) {
-    case XK_Up:
-	dy = -1;
-	break;
-    case XK_Down:
-	dy = 1;
-	break;
-    case XK_Left:
-	dx = -1;
-	break;
-    case XK_Right:
-	dx = 1;
-	break;
-    case XK_space:
-        Global.transparent = !Global.transparent;
-        PwRegionTear(w);
-	RegionTransparency((PaintWidget) w);
-	break;
-    default:
-	break;
-    }
-    if (event->state & ControlMask) {
-	dx *= 5;
-	dy *= 5;
-    }
-    if (dx || dy) {
-	RegionMove((PaintWidget) w, dx, dy);
-	return;
-    }
-    /*
-    **  Look for either backspace or delete and remove region
-     */
-    for (i = 0; i < len; i++) {
-	if (buf[i] == 0x08 || buf[i] == 0x7f) {
-	    PwRegionClear(w);
-	    return;
-	}
-    }
-}
-
-void 
-selectKeyRelease(Widget w, void * l, XKeyEvent * event, void * info)
-{
-    KeySym keysym;
-    int len;
-    char buf[21];
-
-    len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
-
-    /* Noop at this time */
-}
-
 /*
 **  motion
  */
@@ -256,8 +197,12 @@ motionBoxBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 	    XDrawArc(XtDisplay(w), XtWindow(w), l->gcx, rect.x, rect.y,
 		     rect.width, rect.height, 0, 360 * 64);
     }
+#if 0
     l->lastX = event->x - info->zoom / 2;
     l->lastY = event->y - info->zoom / 2;
+#endif
+    l->lastX = event->x;
+    l->lastY = event->y;
     if (shapeMode==0)
         l->lastType = (1&BoxGetStyle()?2:0) + ((event->state & ShiftMask)?1:0);
     if (shapeMode==1)
@@ -584,7 +529,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     mpix = None;
 
-    if (l->areaType == 0)
+   if (l->areaType == 0)
 	releaseBoxBand(w, l, event, info, &mpix);
     else if (releaseSelectFreehandBand(w, l, event, info, &mpix) == False) {
 	PwRegionFinish(w, True);
@@ -629,10 +574,7 @@ SelectBoxOrOvalAdd(Widget w)
 		      (OpEventProc) motion, l);
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
-    OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) selectKeyPress, l);
-    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) selectKeyRelease, l);
+
     SetCrossHairCursor(w);
 
     return l;
@@ -663,11 +605,6 @@ SelectBoxRemove(Widget w, void *p)
 			 (OpEventProc) motion, l);
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
-    OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-			 (OpEventProc) selectKeyPress, l);
-    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-			 (OpEventProc) selectKeyRelease, l);
-    PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
     XtFree((XtPointer) l->real_p);
@@ -701,10 +638,6 @@ SelectFreehandAdd(Widget w)
 		      (OpEventProc) motion, l);
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
-    OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) selectKeyPress, l);
-    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) selectKeyRelease, l);
 
     SetPencilCursor(w);
 
@@ -722,11 +655,6 @@ SelectFreehandRemove(Widget w, void *p)
 			 (OpEventProc) motion, l);
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
-    OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-			 (OpEventProc) selectKeyPress, l);
-    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-			 (OpEventProc) selectKeyRelease, l);
-    PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
     XtFree((XtPointer) l->real_p);
