@@ -127,20 +127,35 @@ drawOutline(Widget w, LocalInfo * l, int x, int y, Boolean flag)
     Window window = XtWindow(w);
     XArc arc;
 
-    arc.width = radius * l->zoom * 2;
-    arc.height = radius * l->zoom * 2;
+    if (l->zoom>0) {
+        arc.width = 2 * radius * l->zoom;
+        arc.height = 2 * radius * l->zoom;
+    } else {
+        arc.width = 2 * radius / (-l->zoom);
+        arc.height = 2 * radius / (-l->zoom);
+    }
     arc.angle1 = 0;
     arc.angle2 = 360 * 64;
 
     if (l->trackDrawn) {
-	arc.x = l->lastX - radius * l->zoom;
-	arc.y = l->lastY - radius * l->zoom;
+        if (l->zoom>0) {
+	    arc.x = l->lastX - radius * l->zoom;
+	    arc.y = l->lastY - radius * l->zoom;
+	} else {
+	    arc.x = l->lastX - radius / (-l->zoom);
+	    arc.y = l->lastY - radius / (-l->zoom);
+	}
 	XDrawArcs(dpy, window, l->gcx, &arc, 1);
 	l->trackDrawn = False;
     }
     if (flag) {
-	arc.x = x - radius * l->zoom;
-	arc.y = y - radius * l->zoom;
+        if (l->zoom>0) {
+	    arc.x = x - radius * l->zoom;
+	    arc.y = y - radius * l->zoom;
+	} else {
+	    arc.x = x - radius / (-l->zoom);
+	    arc.y = y - radius / (-l->zoom);
+	}
 	XDrawArcs(dpy, window, l->gcx, &arc, 1);
 
 	l->lastX = x;
