@@ -96,6 +96,10 @@ okCallback(Widget junkW, XtPointer lArg, XtPointer junk)
     for (i = 0; i < p->nprompt; i++)
 	XtVaGetValues(p->prompts[i].w, XtNstring, &p->prompts[i].rstr, NULL);
 
+#if DEBUG
+    fprintf(stderr, "text.c/okCallback : %ld (%ld, %ld, %ld):\n", uf, w, c, p);
+#endif
+
     if (uf != NULL)
 	uf(w, c, (XtPointer) p);
 
@@ -192,30 +196,36 @@ textshellResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg
     TextPromptInfo *p = l->prmps;
     Position x;
     int i, new;
+
     XtVaGetValues(l->shell, XtNwidth, &width, NULL);
+    if (width<100) width = 100;
     XtVaGetValues(l->form, XtNheight, &height, NULL);
     XtResizeWidget(l->form, width, height, 0);
     XtVaSetValues(l->form, XtNwidth, width, XtNheight, height, NULL);   
-    for (i = 0; i < p->nprompt; i++) {
+
+    for (i = 0; i < p->nprompt; i++)
+      if (p->prompts[i].w) {
         XtVaGetValues(p->prompts[i].w, XtNx, &x,
-			               XtNheight, &height, NULL);
+	  		               XtNheight, &height, NULL);
 	new = width - x - 20;
+        fflush(stdout);
 	if (new<50) new=50;
-	XtResizeWidget(p->prompts[i].w, new, height, 1);
+        if (height<16) height = 16;
+	XtResizeWidget(p->prompts[i].w, (Dimension)new, (Dimension)height, 1);
         XtVaSetValues(p->prompts[i].w, XtNwidth, (Dimension)new, NULL);
-    }
+      }
 }
 
   
-static void 
+static Widget
 textPrompt(Widget w, char *name, TextPromptInfo * prompt,
    XtCallbackProc okProc, XtCallbackProc nokProc, void *data, Pixmap pix)
 {
-    XFontStruct * fi;
+    XFontStruct * fi=NULL;
     char *str;
     int i, j, k, twidth;
     int nargs = 0;
-    Arg args[6];
+    Arg args[8];
     Position x, y;
     static String textAccelerators = "#override\n\
 				<Key>Return: set() notify() unset()\n\
@@ -241,10 +251,12 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
     textUpList *cur;
     Dimension width, maxWidth = 0;
 
+    if (!w || !XtIsRealized(w)) return None;
+
     for (cur = head; cur != NULL; cur = cur->next) {
 	if (strcmp(cur->name, name) == 0 && cur->parent == w) {
 	    XtPopup(cur->shell, XtGrabNone);
-	    return;
+	    return cur->shell;
 	}
     }
 
@@ -265,13 +277,21 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 	accel = XtParseAcceleratorTable(textAccelerators);
 	toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
     }
+
     wshell = GetShell(w);
     if (!XtIsRealized(wshell))
 	wshell = GetToplevel(w);
     XtVaGetValues(wshell, XtNx, &x, XtNy, &y, NULL);
+    x += 24; y += 24;
+    if (x<24) x=24;
+    if (x>WidthOfScreen(XtScreen(wshell))-400)
+       x=WidthOfScreen(XtScreen(wshell))-400;
+    if (y<24) y=24;
+    if (y>HeightOfScreen(XtScreen(wshell))-300)
+       y=HeightOfScreen(XtScreen(wshell))-300;
 
-    XtSetArg(args[nargs], XtNx,  x+24);  nargs++;
-    XtSetArg(args[nargs], XtNy,  y+24);  nargs++;
+    XtSetArg(args[nargs], XtNx,  x);  nargs++;
+    XtSetArg(args[nargs], XtNy,  y);  nargs++;
     shell = XtVisCreatePopupShell(name == NULL ? "popup-dialog" : name,
 				 transientShellWidgetClass, wshell,
 				 args, nargs);
@@ -310,7 +330,7 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 	label = XtVaCreateManagedWidget("label",
 					labelWidgetClass, form,
 					XtNfromVert, last,
-				     XtNlabel, prompt->prompts[i].prompt,
+				        XtNlabel, prompt->prompts[i].prompt,
 					XtNborderWidth, 0,
 					XtNleft, XtChainLeft,
 					XtNright, XtChainLeft,
@@ -320,14 +340,15 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 	if (width > maxWidth)
 	    maxWidth = width;
 
-	twidth = 100;
+	twidth = 120;
+        
 	XtVaGetValues(w, XtNfont, &fi, NULL);
-	if ((k = prompt->prompts[i].len + 2) > 10) {
+	if (fi && (k = prompt->prompts[i].len + 2) > 10) {
 	    str = (char *)xmalloc((k+1)*sizeof(char));
 	    for (j=0; j<k; j++) str[j] = '=';
 	    str[k] = '\0';
 	    twidth = XTextWidth(fi, str, k);
-	    if (twidth < 100) twidth = 100;
+	    if (twidth < 120) twidth = 120;
 	    free(str);
 	}
 
@@ -342,7 +363,7 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 				       XtNlength, prompt->prompts[i].len,
 				       XtNwidth, twidth,
 				       XtNstring, prompt->prompts[i].str,
-		       XtNinsertPosition, strlen(prompt->prompts[i].str),
+		                       XtNinsertPosition, strlen(prompt->prompts[i].str),
 				       XtNleft, XtChainLeft,
 				       XtNdisplayCaret, False,
 				       NULL);
@@ -366,7 +387,6 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 					   XtNfromHoriz, okButton,
 					   NULL);
 
-
     l->widget = w;
     l->curFocus = None;
     l->shell = shell;
@@ -384,7 +404,6 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 	XtInstallAccelerators(text, okButton);
     }
 #endif
-    setFocus(l);
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) l);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) l);
@@ -406,11 +425,13 @@ textPrompt(Widget w, char *name, TextPromptInfo * prompt,
 
     XtAddEventHandler(shell, StructureNotifyMask, False,
 		      (XtEventHandler) textshellResized, (XtPointer) l);
+    setFocus(l);
+    return shell;
 }
 
-void 
+Widget
 TextPrompt(Widget w, char *name, TextPromptInfo * prompt,
 	   XtCallbackProc okProc, XtCallbackProc nokProc, void *data)
 {
-    textPrompt(w, name, prompt, okProc, nokProc, data, None);
+    return textPrompt(w, name, prompt, okProc, nokProc, data, None);
 }
