@@ -587,7 +587,7 @@ draw_landscape(int w, int h, int clouds)
 	    }
     }
 
-    XtFree(cp);
+    XtFree((char *) cp);
     XtFree((char *) u);
     XtFree((char *) u1);
     XtFree((char *) bxf);
