@@ -45,6 +45,9 @@
 #define NULL 0
 #endif				/* AIXV3 */
 
+enum {BROWSER_READ=0, BROWSER_MULTIREAD, BROWSER_LOADED, 
+      BROWSER_SAVE, BROWSER_SIMPLEREAD, BROWSER_SIMPLESAVE};
+
 extern char *routine;
 
 typedef struct {
@@ -68,31 +71,39 @@ extern struct Global_s {
         unsigned char *alpha;
     } clipboard;
     Display *display;
+    Visual *visual;
     XtAppContext appContext;
+    int depth;
     Boolean timeToDie;
     Boolean explore;
     Boolean transparent;
+    Boolean astext;
     Time currentTime;
     XVisualInfo vis;
     char * xft_name;   /* name of Xft font */
+    char * xft_text;
     void * xft_font;
     double xft_size;   /* size of font */
     double xft_height; /* height of font */
     double xft_ascent, xft_descent, xft_maxadv; 
                        /* ascent, descent, max advance */
-    double xft_rot;    /* angle of rotation of font */
+    double xft_rotation;     /* angle of rotation of font */
+    double xft_inclination;  /* inclination */
+    double xft_dilation;     /* dilation */
     double xft_linespacing;  /* linespacing factor */
-    unsigned int background;
-    unsigned int alpha_mode, alpha_threshold, alpha_default;
+    unsigned int alpha_threshold, alpha_bg, alpha_fg;
     unsigned int default_width, default_height;
+    int operation;
     int default_zoom;
     int numpage;
     int cap, join, escape;  /* line caps and joins */
     int dashnumber, dashoffset; /* dash parameters */
     char dashlist[72];
+    unsigned char bg[8];
     double dpi;
     Widget toplevel, bar, back, canvas, curpaint, 
            patternshell, brushpopup, popped_up, popped_parent;
+    Pixmap brushpix;
     int nbrushes;
     void ** brushes;
     void * patterninfo;
@@ -126,8 +137,6 @@ EXTERN(OperationPair * CurrentOp, NULL)
 struct Global_s Global;
 #endif
 
-#define DEFAULT_TITLE	"Untitled"
-
 #define AllButtonsMask \
 	(Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)
 
